################################################################
# Modifications beyond tkWWW 0.9 are Copyright 1993, 1994 by
# The University of Chicago Biological Sciences Division, Academic Computing
# Philip Stylianos
# Lee Newberg
################################################################

# clipboard.tcl

# This file controls what is in the clipboard.  (Note that
# selection.tcl contains routines that control what has been
# selected, but not necessarily "copied" into the clipboard.)
# The menu items, Cut, Copy, Paste, etc. call these routines
# directly.  Clipboard routines controlling text (as opposed to
# widgets) may call to routines in edit.tcl to get the hard work
# done. 

proc phxW3ClipboardCut {} {
    phxW3ClipboardCopy
    phxW3SelectionDelete
    phxW3EditPickPlace
}

proc phxW3ClipboardCopy {} {
    global phxW3Clipboard phxW3ClipboardWidget

    # Find out what we're putting in the clipboard.
    set selection [phxW3SelectionGet]
    if {$selection == ""} {
	phxW3ClipboardClear
	return
    }

# Allow ClipboardCopy of form widgets and list bullets!!!
# Also Cut, Paste, etc.  Distinguish between whole widgets and parts!!!
    switch [phxW3WidgetClass $selection] {
	para {
	    set phxW3Clipboard {para}
	    phxW3EditCopyTextSelection $selection
	    phxW3ClipboardBroadcast $phxW3Clipboard $phxW3ClipboardWidget
	}
	text -
	textarea {
	    if {[set range [$selection tag ranges sel]] != ""} {
		set phxW3Clipboard {text}
		$phxW3ClipboardWidget delete 1.0 end
		$phxW3ClipboardWidget insert 1.0 [eval $selection get $range]
		phxW3ClipboardBroadcast $phxW3Clipboard $phxW3ClipboardWidget
	    } else {
		# Don't know what to do!  Clear the clipboard.
		phxW3ClipboardClear
	    }
	}
	entry {
	    set phxW3Clipboard {text}
	    $phxW3ClipboardWidget delete 1.0 end
	    $phxW3ClipboardWidget insert 1.0 [string range [$selection get] \
		[$selection index sel.first] [$selection index sel.last]]
	    phxW3ClipboardBroadcast $phxW3Clipboard $phxW3ClipboardWidget
	}
	rule {
	    set phxW3Clipboard {rule}
	    $phxW3ClipboardWidget delete 1.0 end
	    phxW3ClipboardBroadcast $phxW3Clipboard $phxW3ClipboardWidget
	}
	image {
	    global tkW3HtImages
	    set index $tkW3HtImages($selection)
	    set phxW3Clipboard [list image \
		$tkW3HtImages($index.url) \
		$tkW3HtImages($index.source) \
		$tkW3HtImages($index.anchor) \
		$tkW3HtImages($index.ismap)]
	    $phxW3ClipboardWidget delete 1.0 end
	    phxW3ClipboardBroadcast $phxW3Clipboard $phxW3ClipboardWidget
	}
	default {
	    # Don't know what to do!  Clear the clipboard.
	    phxW3ClipboardClear
	}
    }
}

proc phxW3ClipboardPaste {} {
    # Can only paste if there is a somewhere to paste to.
    # Must check that paste is enabled in case we're invoked by key-command.
    if {![phxW3SelectionIsFocused] || \
	[lindex [.menu.edit.m entryconfigure Paste -state] 4] == "disabled"} \
    {
	return
    }

    # If focus is a form widget or other entry-type, just do a plain X11 paste.
    set left [phxW3SelectionGetLeftOfFocus]
    if {$left == [phxW3SelectionGetRightOfFocus]} {
	switch {[phxW3WidgetClass $left]} {
	    entry -
	    text -
	    textarea -
	    password {
		$w insert insert [blt_cutbuffer get 0]
		return
	    }
	}
    }
    phxW3SelectionDelete
    phxW3ClipboardInsert
    phxW3EditPickPlace
}

proc phxW3ClipboardInsert {} {
    global phxW3Clipboard
    global phxW3ClipboardWidget

    set focusLeft [phxW3SelectionGetLeftOfFocus]
    set classLeft [phxW3WidgetClass $focusLeft]
    set focusRight [phxW3SelectionGetRightOfFocus]
    set classRight [phxW3WidgetClass $focusRight]

    # If what is in the X/Mac clipboard doesn't match the HTML we
    # have then it must be plain text.  This would occur if an
    # application other than Phoenix was the last to copy into the
    # clipboard.
    if {[blt_cutbuffer get 0]!=[$phxW3ClipboardWidget get 1.0 end]} {
	set phxW3Clipboard "text"
    }

    # The type of the contents of the clipboard
    set classClip [lindex $phxW3Clipboard 0]

    if {$classClip == "para" || $classClip == "text"} {
	if {$classLeft == "para"} {
	    if {$focusLeft != $focusRight} {
		$focusLeft mark set insert end
	    }
	    if {[phxW3EditInsertTextSelection $focusLeft]} {phxW3SaveDirtyBit}
	} elseif {$classRight == "para"} {
	    $focusRight mark set insert 1.0
	    if {[phxW3EditInsertTextSelection $focusRight] == 0} {
		phxW3SelectionSetFocus $focusRight $focusRight
		phxW3SaveDirtyBit
	    }
	} elseif {$focusLeft == $focusRight} {
	    # In a forms text, textarea, password, or entry widget.
	    $focusLeft insert insert [blt_cutbuffer get 0]
	} else {
	    # Split the imageholder if necessary.
	    set focusRight [phxW3WidgetSplitImageHolder $focusRight]
	    set textNew [phxW3WidgetCreateText]
	    phxW3WidgetPackBefore $textNew $focusRight
	    $textNew mark set insert 1.0
	    phxW3SelectionSetFocus $textNew $focusRight
	    if {[phxW3EditInsertTextSelection $textNew] == 0} {
		phxW3SaveDirtyBit
	    }
	}
    } else {
	# We're inserting a non-text widget.
	if {$classLeft == "para" && $focusLeft == $focusRight} {
	    # Split the text widget in two.
	    set focusRight [phxW3WidgetSplitPara $focusRight]
	    phxW3SelectionSetFocus $focusLeft $focusRight
	} else {
	    # Split the imageholder if necessary.
	    set focusRight [phxW3WidgetSplitImageHolder $focusRight]
	    phxW3SelectionSetFocus $focusLeft $focusRight
	}

	# Set "before" to the anscestor which is a child of the appropriate 
	# frame.
	# !!!This probably needs changing to reflect the recursive structure
	# of frames that arise in the presence of lists.  I suspect that
	# $before can no longer be a widget, but now must be a packing
	# command.  This applies to all routines that use $before.
	switch [phxW3WidgetClass $focusRight] {
	    end {set before ""}
	    image -
	    textarea {set before [winfo parent $focusRight]}
	    default {set before $focusRight}
	}

	if {$classClip == "rule"} {
	    # It would be good if an <HR> could be an anchor!!
	    set rule [phxW3WidgetAddHR $before]
	    phxW3SelectionSetFocus $rule $focusRight
	} elseif {$classClip == "image"} {
	    # phxW3WidgetNewImage looks at the current widget to
	    # see if a new imageholder is to be created.  Also
	    # looks at its "before" argument to figure out where
	    # to pack the image once it is created.  We must set
	    # both appropriately.
	    phxW3WidgetSetCurrentWidget $focusLeft
	    if {[lindex $phxW3Clipboard 3] != ""} {
		tkW3HtBeginAnc "" [lindex $phxW3Clipboard 3]
	    }
	    set image [phxW3WidgetNewImage \
		[lindex $phxW3Clipboard 1] \
		[lindex $phxW3Clipboard 2] \
		[lindex $phxW3Clipboard 4] $before]
	    if {[lindex $phxW3Clipboard 3] != ""} {
		tkW3HtEndAnc
	    }
	    phxW3ModeConfigure $image
	    phxW3SelectionSetFocus $image $focusRight
	    phxW3WidgetWrap $image
	}
	phxW3SaveDirtyBit
    }
}

proc phxW3ClipboardClear {} {
    global phxW3Clipboard phxW3ClipboardWidget

    set phxW3Clipboard text
    $phxW3ClipboardWidget delete 1.0 end
    phxW3ClipboardBroadcast $phxW3Clipboard $phxW3ClipboardWidget
}

# phxW3ClipboardBroadcast
# 
# Sends the phxW3Clipboard and phxW3ClipboardWidget information
# to the blt_cutbuffer and all other Phoenix applications.

proc phxW3ClipboardBroadcast {type widget} {
    # Copy the string into the X and Mac cut buffers
    blt_cutbuffer set [$widget get 1.0 end] 0

    # Send the information to other phxclone applications.
    
    phxW3NavigateSendToTkWWW [list phxW3ClipboardReceiveCommands \
	[phxW3ClipboardSendCommands $type $widget]]
}

# phxW3ClipboardSendCommands
# 
# Generates commands which will set phxW3Clipboard and
# phxW3ClipboardWidget appropriately.  Because the created
# commands are strings, liberal use of the list command is used
# to ensure that any empty argument, argument containing spaces,
# or argument containing Tcl-significant characters remains as
# one un-Tcl-interpreted argument.

proc phxW3ClipboardSendCommands {{type ""} {widget ""}} {
    if {$type == ""} {global phxW3Clipboard ; set type $phxW3Clipboard}
    if {$widget == ""} {global phxW3ClipboardWidget ; set widget $phxW3ClipboardWidget}

    ################
    # Copy the text
    ################
    append cmd \
"global phxW3Clipboard phxW3ClipboardWidget tkW3HtAnchors tkW3HtMaxAnchor
set phxW3Clipboard [list $type]
\$phxW3ClipboardWidget delete 1.0 end
\$phxW3ClipboardWidget insert 1.0 [list [$widget get 1.0 end]]
"
    ################
    # Copy the tags
    ################
    global tkW3HtAnchors
    foreach tag [$widget tag names] {
	if {$tag == "sel"} continue
	if {[$widget tag ranges $tag] == ""} {
	    # This tag isn't in use.  Let's get it out of our hair.
	    $widget tag delete $tag
	    continue
	}
	set tagNew $tag
	if {[regexp {^h([0-9]+)$} $tag junk tagnum]} {
	    ################
	    # We have to create an anchor that the recipient understands.
	    ################
	    set tagnumNew "\$anchor($tagnum)"
	    set tagNew "h\$anchor($tagnum)"
	    append cmd \
"if \{\![info exists anchor($tagnum)\]\} \{
    set anchor($tagnum) \$tkW3HtAnchors(index)
    set tkW3HtAnchors(name.$tagnumNew) [list $tkW3HtAnchors(name.$tagnum)]
    set tkW3HtAnchors(href.$tagnumNew) [list $tkW3HtAnchors(href.$tagnum)]
    incr tkW3HtMaxAnchor
    incr tkW3HtAnchors(index)
\}
"
	}
	################
	# Copy the ranges
	################
	for {set index 1.0} \
	    {[set range [$widget tag nextrange $tag $index]] != ""} \
	    {set index [lindex $range 1]} \
	{
	    # Note that we want $range to split into two arguments!
	    append cmd "\$phxW3ClipboardWidget tag add [list $tagNew] $range\n"
	}
	################
	# Copy the anchor tag's configurations
	################
	foreach attr [$widget tag configure $tag] {
	    append cmd \
"\$phxW3ClipboardWidget tag configure [list $tagNew [lindex $attr 0] [lindex $attr 4]]\n"
	}
	################
	# Copy the anchor tag's bindings
	################
	foreach event [$widget tag bind $tag] {
	    set binding [$widget tag bind $tag $event]
	    set index [lsearch $binding $tagnum]
	    if {$index >= 0} {lreplace $binding $index $index $tagnumNew}
	    append cmd "\$phxW3ClipboardWidget tag bind [list $tagNew $event $binding]\n"
	}
    }
    return $cmd
}

# phxW3ClipboardReceiveCommands
# 
# Sets the clipboard (phxW3Clipboard and phxW3ClipboardWidget) by
# evaluating the supplied commands.

proc phxW3ClipboardReceiveCommands {commands} {
	eval $commands
}

# phxW3ClipboardInitialize
#
# Creates the clipboard widget and queries another tkWWW for
# the contents.  If the query isn't answered, then it assumes
# that the clipboard contains (possibly empty) plain text.
#
# Note: This is its own routine so that the "global declaration" doesn't
# transfer to files which are sourced after clipboard.tcl.

proc phxW3ClipboardInitialize {} {

    global phxW3Clipboard phxW3ClipboardWidget
    global phxW3ClipboardWidget
    set phxW3ClipboardWidget .clipboard
    text $phxW3ClipboardWidget

    set return [phxW3NavigateSendToOneTkWWW phxW3ClipboardSendCommands]
    if {$return == ""} {set phxW3Clipboard "text"} else {
	phxW3ClipboardReceiveCommands $return
    }
}
