################################################################
# Modifications beyond tkWWW 0.9 are Copyright 1993, 1994 by
# The University of Chicago Biological Sciences Division, Academic Computing
# Philip Stylianos
# Lee Newberg
################################################################

## edit.tcl main file for Phoenix user interface
## ==============
## Copyright (C) 1992-1993
## Global Network Academy
## Macvicar Institute for Educational Software Development
##
## See the file COPYRIGHT for conditions

## The procedures in this file are responsible for editing inside of Phoenix.

## Conventions:
##   All procedures in this file begin with tkW3Edit or phxW3Edit

## Bugs and Todo:
##   Some abstraction questions

# This is the cut buffer
#set tkW3EditSelection {}

## ********************
## Initialization procedures
## ********************

proc tkW3EditInitialize {} {
    global tkW3EditConvert

    set convert_list {
	{"" "default"}
	{H1 heading1}
	{H2 heading2}
	{H3 heading3}
	{H4 heading4}
	{H5 heading5}
	{H6 heading6}
	{ADDRESS address}
	{PRE preformatted}
	{TT typewriter}
	{B bold}
	{I italics}
	{U underline}
	{EM emphasis}
	{STRONG "strong emphasis"}
	{SAMP sample}
	{KBD keyboard}
	{VAR variable}
	{DFN definition}
	{CITE citation}
	{HR rule}
    }

    foreach item $convert_list {
	set tkW3EditConvert([lindex $item 0]) [lindex $item 1]
    }

    text .f.html -yscrollcommand {.vs set} -bd 0
}

# tkW3EditBind sets cursor movement, deletion, and insertion
# bindings, but not selection bindings for selection.  The last as
# well as the primary call of this function can be found in
# phxW3SelectionWidgetInitialize.

proc tkW3EditBind {w} {

    # Inserting

    bind $w <KeyPress> {
	if {"%A" != ""} {
		phxW3SelectionDelete
		phxW3EditInsert %W %A
		phxW3EditPickPlace
	}
    }

    bind $w <Shift-Return> {
	phxW3SelectionDelete
	phxW3WidgetInsertRule %W
	phxW3EditPickPlace
    }

    # Button

    bind $w <2> {}
    bind $w <3> {}
    bind $w <4> {}
    bind $w <5> {}

    # Deleting text
    bind $w <BackSpace> {
	tkW3EditBackspace %W
	phxW3EditPickPlace
    }
    bind $w <Delete> {
	tkW3EditBackspace %W
	phxW3EditPickPlace
    }
    bind $w <Control-d> {
	phxW3EditArrowRight %W
	tkW3EditBackspace %W
	phxW3EditPickPlace
    }

    # Moving the cursor

    # ... to the right
    bind $w <Control-f> {
	phxW3SelectionClear
	phxW3EditArrowRight %W
	phxW3EditPickPlace
    }
    bind $w <Right> {
	phxW3SelectionClear
	phxW3EditArrowRight %W
	phxW3EditPickPlace
    }

    # ... to the left
    bind $w <Control-b> {
	phxW3SelectionClear
	phxW3EditArrowLeft %W
	phxW3EditPickPlace
    }
    bind $w <Left> {
	phxW3SelectionClear
	phxW3EditArrowLeft %W
	phxW3EditPickPlace
    }

    # ... downward
    bind $w <Control-n> {
	phxW3SelectionClear
	phxW3EditArrowDown %W
	phxW3EditPickPlace
    }
    bind $w <Down> {
	phxW3SelectionClear
	phxW3EditArrowDown %W
	phxW3EditPickPlace
    }

    # ... upward
    bind $w <Control-p> {
	phxW3SelectionClear
	phxW3EditArrowUp %W
	phxW3EditPickPlace
    }
    bind $w <Up> {
	phxW3SelectionClear
	phxW3EditArrowUp %W
	phxW3EditPickPlace
    }
}

## ********************
## Keybinding actions
## ********************

# phxW3EditParaReturn
# If we're not between paragraphs then insert a blank line.
# Otherwise, go to the line starting the following paragraph.

proc phxW3EditParaReturn {window} {

    set start [$window index insert]
    set tags [$window tag names "insert - 1 c"]

    if {[lsearch $tags PRE] != -1} {

	# Within PRE markup, do as we're told.
	$window insert insert \n
    } else {

	# We're about to insert a return (or make use of an existing one)
	# so we need to delete any spaces after the insert.
	phxW3EditRemoveLeadingSpaces $window insert

	if {[$window compare insert == end]} {
	    if {[phxW3EditDetectBlankLine $window insert]} {
		return
	    } else {
		$window insert insert "\n"
	    }
	} elseif {[$window compare insert == 1.0]} {
	    if {[phxW3EditDetectBlankLine $window insert]} {
		$window mark set insert "insert + 1 c"
		return
	    } else {
		$window insert insert "\n"
	    }
	} elseif {[phxW3EditDetectBlankLine $window "insert + 1 c"]} {
	    # Make use of existing return
	    $window mark set insert "insert + 2 c"
	    return
	} elseif {[phxW3EditDetectBlankLine $window insert]} {
	    # Make use of existing return
	    $window mark set insert "insert + 1 c"
	    return
	} elseif {[phxW3EditDetectBlankLine $window "insert - 1 c"]} {
	    # Make use of existing return
	    return
	} else {
	    $window insert insert "\n\n"
	}
    }

    phxW3EditBreakHeadingAtNewlines $window $start insert
}

# phxW3EditRemoveLeadingSpaces
# Removes any spaces after a carriage return

proc phxW3EditRemoveLeadingSpaces {w index} {
    while {[$w compare $index < end] && [$w get $index] == " "} {
	$w delete $index
    }
}


proc phxW3EditInsert {window string} {
    set focusLeft [phxW3SelectionGetLeftOfFocus]
    set classLeft [phxW3WidgetClass $focusLeft]
    set focusRight [phxW3SelectionGetRightOfFocus]
    set classRight [phxW3WidgetClass $focusRight]

    regsub -all "\r\n" $string "\n" string
    regsub -all "\r" $string "\n" string

    if {$classLeft != "para" && $focusLeft == $focusRight} {
	# In a forms text, textarea, password, or entry widget.
	$focusLeft insert insert $string
    } elseif {[phxW3ModeGet] == "edit"} {
	if {$classLeft == "para"} {
	    phxW3SelectionSetFocus $focusLeft
	    if {$focusLeft != $focusRight} {
		$focusLeft mark set insert end
	    }
	    phxW3EditParaInsert $focusLeft $string
	} elseif {$classRight == "para"} {
	    $focusRight mark set insert 1.0
	    phxW3SelectionSetFocus $focusRight $focusRight
	    phxW3EditParaInsert $focusRight $string
	} else {
	    # Split the imageholder if necessary.
	    set focusRight [phxW3WidgetSplitImageHolder $focusRight]
    
	    set textNew [phxW3WidgetCreateText]
	    phxW3WidgetPackBefore $textNew $focusRight
	    phxW3SelectionSetFocus $textNew
	    $textNew mark set insert 1.0
	    phxW3SelectionSetFocus $textNew $focusRight
	    # The creation of the new text widget is in some sense a carriage
	    # return, so don't insert an additional one immediately afterwards.
	    if {[string index $string 0] == "\n"} {
		set string [string range $string 1 end]
	    }
	    phxW3EditParaInsert $textNew $string
	}
	phxW3SaveDirtyBit
    }
}

# phxW3EditParaInsert
# Insert a special string into the window with the specified tags.
# A special string is any of:
#     the empty string
#     a single character
#     text which is preformatted (e.g., PRE, ADRESS)
#     text which is not preformatted but has no "\n" and has no
#       consecutive spaces.
# Comments are outside the code for speed.

# Do nothing if the string is empty.
# Do special processing if the string as a \n
# Inherit tags from the previous character unless we're at the beginning
#   of a line.  In that case, inherit from the following character.
# If whitespace formatting is restricted
# Then
#   Cannot start a line with a space or have two spaces in a row.
#   If the current line is blank
#   Then
#      Create a new paragraph:
#      If no text yet, then there's nothing to separate the new text from.
#      Elseif at the end just separate from the preceding text.
#      Elseif at the beginning just separate from the following text.
#      Else separate from the preceding and following text.
# Insert the desired string.
# Clear any tags that are inherited according to Tk rules.
# Set the desired tags

proc phxW3EditParaInsert {window string {tags "inherit"}} {
    if {$string == ""} {return}
    if {$string == "\n" || $string == "\r"} {return [phxW3EditParaReturn $window]}
    if {$tags == "inherit"} {
	if {[$window compare insert == "insert linestart"]} \
	    {set tags [$window tag names insert]} \
	else {set tags [$window tag names "insert - 1 c"]}}
    if {![phxW3HtAllowWhitespaces $tags]} {
	if {$string == " " && \
	    ([$window compare insert == 1.0] || \
	    [$window get "insert - 1 c"] == "\n" || \
	    [$window get "insert - 1 c"] == " ")} {return}
	if {[phxW3EditDetectBlankLine $window insert]} {
	    if {[$window compare 1.0 == end]} {} \
	    elseif {[$window compare insert == end]} \
		{$window insert insert "\n"
		phxW3EditBreakHeadingAtNewlines $window "insert - 1 c" insert} \
	    elseif {[$window compare insert == 1.0]} \
		{$window insert insert "\n"
		$window mark set insert "insert - 1 c"
		phxW3EditBreakHeadingAtNewlines $window "insert - 1 c" insert} \
	    else {$window insert insert "\n\n"
		$window mark set insert "insert - 1 c"
		phxW3EditBreakHeadingAtNewlines $window \
		    "insert - 1 c" "insert + 1 c"}}}
    set start [$window index insert]
    $window insert insert $string
    foreach tag [$window tag names $start] {$window tag remove $tag $start insert}
    foreach tag $tags {$window tag add $tag $start insert}}

# tkW3EditBackspace
# What to do when backspace is typed.

proc tkW3EditBackspace {w} {

    # Delete the selection if there is one.
    if {[set selection [phxW3SelectionGet]] != ""} {
	phxW3SelectionDelete
	return
    }

    set left [phxW3SelectionGetLeftOfFocus]
    set right [phxW3SelectionGetRightOfFocus]
    set class [phxW3WidgetClass $left]
    if {$left == "start"} { return }
    if {$class == "entry"} {
	set insert [$w index insert]
	if {$insert > 0} {
	    $w delete [expr $insert - 1]
	}
    } elseif {$class != "para" && $left == $right} {
	$w delete "insert -1 c" insert
    } elseif {[phxW3ModeGet] == "edit"} {
	switch $class {
	    para {
		phxW3EditDeleteCharacter $w
		if {[$w compare 1.0 == end]} {
		    phxW3SelectionSetFocus \
			[phxW3WidgetLeft $w] [phxW3WidgetRight $w]
		    destroy $w
		    phxW3WidgetWrap [phxW3SelectionGetRightOfFocus]
		} elseif {[$w compare insert == 1.0]} {
		    phxW3SelectionSetFocus [phxW3WidgetLeft $w] $w
		} elseif {[$w compare insert == end]} {
		    phxW3SelectionSetFocus $w [phxW3WidgetRight $w]
		}
		phxW3SaveDirtyBit
	    }
	    image {
		set parent [winfo parent $left]
		destroy $left
		if {[llength [pack slaves $parent]] == 0} {destroy $parent}
		phxW3SelectionSetFocus [phxW3WidgetLeft $right] $right
		# Try to re-wrap.  Must use $right as an argument
		# so that Backspace on the first image works.
		phxW3WidgetWrap $right
		phxW3SaveDirtyBit
	    }
	    default {
		# Handle horizontal rule and miscellaneous forms widgets.
		destroy $left
		phxW3SelectionSetFocus [phxW3WidgetLeft $right] $right
		# Try to re-wrap.  Must use $right as an argument
		# so that Backspace on the first image works.
		phxW3WidgetWrap $right
		phxW3SaveDirtyBit
	    }
	}
    }
}

# phxW3EditDeleteCharacter
# Deletes the character before the insert mark.

proc phxW3EditDeleteCharacter {w} {
    # If we're just after a blank line, back up one character.
    # (Later we'll delete the blank line.)
    if {[phxW3EditDetectBlankLine $w "insert - 1 c"]} {
	$w mark set insert "insert - 1 c"
    }

    if {[phxW3EditDetectBlankLine $w insert]} {

	# Delete a blank line only if it merges two similar things.
	# Otherwise just go back another character
	if {[$w compare insert == 1.0] || \
	    [$w compare insert == end] || \
	    [phxW3EditDetectParaStyle $w "insert + 1 c"] == \
	    [phxW3EditDetectParaStyle $w "insert - 2 c"]} {
	    catch {$w delete "insert - 1 c" "insert + 1 c"}
	} else {
	    $w mark set insert "insert - 1 c"
	}
    } else {

	# Behave like a normal backspace
	$w delete "insert - 1 c" insert

	# Can't start a line with blanks.  Delete any extras.
	if {[$w compare insert == "insert linestart"]} {
	    while {[$w get insert] == " "} {
		$w delete insert
	    }
	}

	# The current line is supposed to have text.  If it doesn't,
	# delete it.
	if {[phxW3EditDetectBlankLine $w insert]} {
	    catch {$w delete "insert - 1 c" "insert + 1 c"}
	}
    }
}

# phxW3EditArrowLeft

# Note that this command should not move the insertion point from
# inside a forms element to outside of it.

proc phxW3EditArrowLeft {w} {
    # Nothing to do if there is no focus
    if {![phxW3SelectionIsFocused]} { return }

    set left [phxW3SelectionGetLeftOfFocus]
    set right [phxW3SelectionGetRightOfFocus]
    set classLeft [phxW3WidgetClass $left]
    set classRight [phxW3WidgetClass $right]

    # Within a para, text, password or textarea widget
    if {$left == $right || $classLeft == "para"} {
	if {$classLeft == "entry"} {
	    $left icursor [expr [$left index insert] - 1]
	} else {
	    $left mark set insert "insert - 1 c"
	    if {[$left compare insert == 1.0] && $classLeft == "para"} {
		phxW3SelectionSetFocus [phxW3WidgetLeft $left] $left
	    } else {
		phxW3SelectionSetFocus $left
	    }
	}
    } elseif {$left != "start"} {
	# Skipping over an entire widget
	phxW3SelectionSetFocus [phxW3WidgetLeft $left] $left
    }

    phxW3SelectionSetLeftOfAnchor [phxW3SelectionGetLeftOfFocus]
    phxW3SelectionSetRightOfAnchor [phxW3SelectionGetRightOfFocus]
}

# phxW3EditArrowRight

proc phxW3EditArrowRight {w} {
    # Nothing to do if there is no focus
    if {![phxW3SelectionIsFocused]} { return }

    set right [phxW3SelectionGetRightOfFocus]
    set left [phxW3SelectionGetLeftOfFocus]
    set classRight [phxW3WidgetClass $right]
    set classLeft [phxW3WidgetClass $left]

    # Within a para, text, password, or textarea widget
    if {$right == $left || $classRight == "para"} {
	if {$classRight == "entry"} {
	    $right icursor [expr [$right index insert] + 1]
	} else {
	    $right mark set insert "insert + 1 c"
	    if {[$right compare insert == end] && $classRight == "para"} {
		phxW3SelectionSetFocus $right [phxW3WidgetRight $right]
	    } else {
		phxW3SelectionSetFocus $right
	    }
	}
    } elseif {$right != "end"} {
	# Skipping over an entire widget
	phxW3SelectionSetFocus $right [phxW3WidgetRight $right]
    }

    phxW3SelectionSetLeftOfAnchor [phxW3SelectionGetLeftOfFocus]
    phxW3SelectionSetRightOfAnchor [phxW3SelectionGetRightOfFocus]
}

# phxW3EditArrowUp

proc phxW3EditArrowUp {w} {
    # Nothing to do if there is no focus
    if {![phxW3SelectionIsFocused]} { return }

    set left [phxW3SelectionGetLeftOfFocus]
    set right [phxW3SelectionGetRightOfFocus]
    set classLeft [phxW3WidgetClass $left]
    set classRight [phxW3WidgetClass $right]

    # Within a para, text, password, or textarea widget
    if {$left == $right || $classLeft == "para"} {
	if {$classLeft == "entry"} {
	    $left icursor 0
	} else {
	    set place [phxW3EditFindPlace $left insert]
	    set x [lindex $place 0]
	    set y [lindex $place 1]
	    while {[$left compare insert == @$x,$y] && $y > 0} {
		incr y -1
	    }
	    if {[$left compare insert != @$x,$y]} {
		$left mark set insert @$x,$y
	    } else {
		$left mark set insert 1.0
	    }
	    if {[$left compare insert == 1.0] && $classLeft == "para"} {
		phxW3SelectionSetFocus [phxW3WidgetLeft $left] $left
	    } else {
		phxW3SelectionSetFocus $left $left
	    }
	}
    } elseif {$left != "start"} {
	# Skipping over an entire widget
	phxW3SelectionSetFocus [phxW3WidgetLeft $left] $left
    }

    phxW3SelectionSetLeftOfAnchor [phxW3SelectionGetLeftOfFocus]
    phxW3SelectionSetRightOfAnchor [phxW3SelectionGetRightOfFocus]
}

# phxW3EditArrowDown

proc phxW3EditArrowDown {w} {
    # Nothing to do if there is no focus
    if {![phxW3SelectionIsFocused]} { return }

    set right [phxW3SelectionGetRightOfFocus]
    set left [phxW3SelectionGetLeftOfFocus]
    set classRight [phxW3WidgetClass $right]
    set classLeft [phxW3WidgetClass $left]

    # Within a para, text, password, or textarea widget
    if {$right == $left || $classRight == "para"} {
	if {$classLeft == "entry"} {
	    $left icursor end
	} else {
	    set place [phxW3EditFindPlace $right insert]
	    set x [lindex $place 0]
	    set y [lindex $place 1]
	    set height [winfo height $right]
	    while {[$right compare insert == @$x,$y] && $y < $height} {
		incr y
	    }
	    $right mark set insert @$x,$y
	    if {[$right compare insert == end] && $classRight == "para"} {
		phxW3SelectionSetFocus $right [phxW3WidgetRight $right]
	    } else {
		phxW3SelectionSetFocus $right $right
	    }
	}
    } elseif {$right != "end"} {
	# Skipping over an entire widget
	phxW3SelectionSetFocus $right [phxW3WidgetRight $right]
    }

    phxW3SelectionSetLeftOfAnchor [phxW3SelectionGetLeftOfFocus]
    phxW3SelectionSetRightOfAnchor [phxW3SelectionGetRightOfFocus]
}

# phxW3EditShiftArrowLeft

proc phxW3EditShiftArrowLeft {w} {
return

    # Nothing to do if there is no focus
    if {![phxW3SelectionIsFocused]} { return }

    set left [phxW3SelectionGetLeftOfFocus]
    set classLeft [phxW3WidgetClass $left]

    if {![phxW3SelectionIsAnchored]} {
	# The anchor doesn't yet exist.  Better set it now.
	phxW3SelectionSetLeftOfAnchor $left
	phxW3SelectionSetRightOfAnchor [phxW3SelectionGetRightOfFocus]
	# Note that a Text widget will take care of itself when
	# its binding is invoked later.
    }

    set anchorLeft [phxW3SelectionGetLeftOfAnchor]

    # Selections may only involve a single widget
    if {$left != $anchorLeft} { return }

# Here I am!!!

    switch $classLeft {
	text -
	password -
	textarea -
	para {
	    phxW3SelectionSetRightOfFocus $left
	    tk_textSelectTo $left insert
	    if {[$left compare insert == 1.0] && $classLeft == "para"} {
		phxW3SelectionSetLeftOfFocus [phxW3WidgetLeft $left]
	    }
	}
	start {}
	default {
	    phxW3SelectionSetFocus [phxW3WidgetLeft $left] $left
	    set old [phxW3SelectionGet]
	}
    }
}

# phxW3EditShiftArrowRight

proc phxW3EditShiftArrowRight {w} {
# Write ShiftArrowRight!!!
}

# phxW3EditShiftArrowUp

proc phxW3EditShiftArrowUp {w} {
# Write ShiftArrowUp!!!
}

# phxW3EditShiftArrowDown

proc phxW3EditShiftArrowDown {w} {
# Write ShiftArrowDown!!!
}


# phxW3EditDetectBlankLine
# returns true if the supplied line is blank.
# (Note that by default the result of the last command is returned by Tcl).

proc phxW3EditDetectBlankLine {window index} {$window compare "$index linestart" == "$index lineend"}

# phxW3EditDetectParaStyle
# Returns the paragraph style of the specified character or {}
# if its nothing special.

proc phxW3EditDetectParaStyle {window index} {
    global phxW3EditParaStyles

    set tags [$window tag names $index]
    foreach para $phxW3EditParaStyles {
	if {[lsearch $tags $para] != -1} { return $para }
    }
    return ""
}

# phxW3EditPickPlace
# Centers the supplied window in the hypertext widget if it is not
# currently visible.  For text widgets, the insertion mark is centered.

proc phxW3EditPickPlace {{window "focus"}} {

    # It's hard to do query's of widgets that aren't drawn so...
    update idletasks

    # Set the focus.
    if {$window == "focus"} { 
	set left [phxW3SelectionGetLeftOfFocus]
	set right [phxW3SelectionGetRightOfFocus]

	# We like text widgets better because they display the cursor.
	# If there is a text widget nearby use it.
	if {$left == $right} {
	    set window $left
	} elseif {[phxW3WidgetClass $left] == "para"} {
	    set window $left
	    $left mark set insert end
	} else {
	    set window $right
	    if {[phxW3WidgetClass $right] == "para"} {
		$right mark set insert 1.0
	    }
	}
    }

    if {![regexp {^\.f} $window]} {
	# We're not editing, there's usually probably nothing to do.
	# To make sure .titles.title_entry displays correctly we'll issue
	# a yview command.
	if {[winfo class $window] == "Text"} {$window yview 0}
	return
    }

    # Find the pixel location of the place we wish to view
    if {$window == "start"} { set place 0 } \
    elseif {$window == "end"} { set place 30000 } \
    else {
	switch [winfo class $window] {
	    Text {
		set place [expr \
		    [lindex [phxW3EditFindPlace $window insert] 1] + \
		    [phxW3EditGetHeight $window]]
	    }
	    default {
		set place [expr [winfo height $window]/2 + \
		[phxW3EditGetHeight $window]]
	    }
	}
    }

    # Convert from pixels to scroll units.
    set htext [phxW3CurrentHtext]
    set scrollunits [lindex [$htext configure -yscrollunits] 4]
    set place [expr $place / $scrollunits]

    # Compare the top and the bottom to the viewed range.
    set top [$htext yview]
    set hheight [expr [winfo height $htext] / $scrollunits]
    set bottom [expr $top + $hheight]

    if {$place < $top || $place > $bottom} {
	set place [expr {$place - $hheight/2}]
	if {$place < 0} { set place 0 }

	set scrollbar [lindex [lindex [$htext config -yscrollcommand] 4] 0]
	phxW3WidgetYview $scrollbar $htext $place
    }
}

# phxW3EditFindPlace
# Returns the lower left-hand corner pixel of the specified mark.
# This must be the lower left-hand corner of the window if the mark == "end"

proc phxW3EditFindPlace {window mark} {
    # Simplify the supplied mark.
    set mark [$window index $mark]

    # Find the pixel value of the mark
    set top 0
    set bottom [winfo height $window]
    # Coordinates start at zero, not one.
    incr bottom -1

    set ymiddle [expr ($top + $bottom) / 2]
    while {$ymiddle != $top && $ymiddle != $bottom} {
	if {[$window compare @0,$ymiddle <= $mark]} {
	    set top $ymiddle
	} else {
	    set bottom $ymiddle
	}
	set ymiddle [expr ($top + $bottom) / 2]
    }	
    
    set left 0
    set right [winfo width $window]
    # Coordinates start at zero, not one.
    incr right -1

    set xmiddle [expr ($left + $right) / 2]
    while {$xmiddle != $left && $xmiddle != $right} {
	if {[$window compare @$xmiddle,$ymiddle < $mark]} {
	    set left $xmiddle
	} else {
	    set right $xmiddle
	}
	set xmiddle [expr ($left + $right) / 2]
    }

    # Because of roundoff from the fact that $right == $left + 1,
    # $xmiddle should be one pixel to the left of the first column that
    # occurs on the mark.
    incr xmiddle

    return "$xmiddle $ymiddle"
}

# phxW3EditGetHeight
#
# Recursively calculate the y-coordinate of the specified window.

proc phxW3EditGetHeight {w} {
	if {[phxW3WidgetClass $w] == "frame"} { return 0 }
	expr [phxW3EditGetHeight [winfo parent $w]] + [winfo y $w]
}

# tkW3EditSetCharStyle
# Change the weight (e.g. bold, italics) of fonts within a paragraph.
# The various weights are not mutually exclusive.

proc tkW3EditSetCharStyle {html} {
# Don't SetCharStyle within text-like form widgets!!!
    global phxW3EditCharStyles

    set window [phxW3WidgetCurrentText]
    if {[phxW3WidgetClass $window]=="para" && [$window tag ranges sel] != {}} {
	if {$html != ""} {
	    # Add requested style
	    $window tag add $html sel.first sel.last
	} else {
	    # Remove old styles
	    foreach tag $phxW3EditCharStyles {
		$window tag remove $tag sel.first sel.last
	    }
	}
	phxW3SaveDirtyBit
    }
}

# tkW3EditSetParaStyle
# Change among paragraph styles, H1, H2, H3, H4, H5, H6, and
# PRE.
# These styles are mutually exclusive.

proc tkW3EditSetParaStyle {html} {
# Don't SetParaStyle within text-like form widgets!!!
    global phxW3EditParaStyles

    set window [phxW3WidgetCurrentText]
    if {$window != "" && [$window tag ranges sel] != {}} {
	# Remove old styles
	foreach tag $phxW3EditParaStyles {
	    $window tag remove $tag sel.first sel.last
	}
	# Add requested style
	if {$html != ""} {
	    $window tag add $html sel.first sel.last
	}

	phxW3EditBreakHeadingAtNewlines $window sel.first sel.last

	# Don't allow Headings of different kinds to be on the same line.
	if {[$window tag ranges sel] != {} && \
	    [$window compare sel.first > 1.0] && \
	    [phxW3EditDetectParaStyle $window "sel.first - 1 c"] !=
	    [phxW3EditDetectParaStyle $window sel.first]} \
	{
	    $window mark set ESPSinsert insert
	    $window mark set insert sel.first
	    phxW3EditParaInsert $window "\n"
	    $window mark set insert ESPSinsert
	    $window mark unset ESPSinsert
	}
	if {[$window tag ranges sel] != {} && \
	    [$window compare sel.last < end] && \
	    [phxW3EditDetectParaStyle $window "sel.last - 1 c"] !=
	    [phxW3EditDetectParaStyle $window sel.last]} \
	{
	    $window mark set ESPSinsert insert
	    $window mark set insert sel.last
	    phxW3EditParaInsert $window "\n"
	    $window mark set insert ESPSinsert
	    $window mark unset ESPSinsert
	}
	phxW3SaveDirtyBit
    }
}

# phxW3EditBreakHeadingAtNewlines
# Within headings, carriage returns are not allowed.  Thus each
# component (between blank lines) is assumed to be its own heading.

proc phxW3EditBreakHeadingAtNewlines {window start stop} {
    set start [$window index $start]
    set stop [$window index $stop]
    for {} \
	{[$window compare $start < $stop]} \
	{set start [$window index "$start + 1 c"]} \
    {
	if {[$window get $start] == "\n"} {
	    foreach tag {H1 H2 H3 H4 H5 H6} {
		$window tag remove $tag $start "$start + 1 c"
	    }
	}
    }
}
	
## ********************
## Save Generated HTML to a file
## ********************

proc tkW3EditSave {} {
    set URL [tkW3OutputGetAddress]
    set path [phxW3InfoFileEditable $URL]

    if {$path == ""} {
	# File isn't writeable.
	phxW3EditSaveAs
    } else {
	tkW3OutputCursorWait
	phxW3EditGenerateHTML
	tkW3EditOutputHTML $path
	phxW3SaveCleanBit
	tkW3OutputCursorNormal
    }
}

proc phxW3EditSaveAs {} {
    set path [phxW3Filebox w]
    if {$path != ""} {
	set newURL [phxW3InfoConvertToHTTP $path]
	set newTitle [tkW3OutputGetTitle]
	set oldURL [tkW3OutputGetAddress]

	if {$oldURL != ""} {
	    phxW3CacheRename $oldURL $newURL $newTitle
	} else {
	    phxW3CacheAdd $newURL [phxW3CurrentHtext] $newTitle
	}

	tkW3OutputEntryPrint .titles.address_entry $newURL
	tkW3OutputCursorWait
	phxW3EditGenerateHTML
	tkW3EditOutputHTML $path
	phxW3SaveCleanBit
	tkW3OutputCursorNormal
    }
}

# Warning: tkW3NavigateSaveText and tkW3EditOutputHTML are nearly
# identical.  They probably should be merged!!

proc tkW3EditOutputHTML {filename} {
    set file [open $filename "w"]
    puts $file [.f.html get 1.0 end]
    close $file
}

## ********************
## Procedures to generate HTML in the edit window from 
## stuff displayed in the display window
## ********************

proc phxW3EditGenerateHTML {} {

    tkW3OutputCursorWait
    .f.html configure -state normal

    # Empty .f.html and delete all tags (except "sel" which is not deletable).
    .f.html delete 1.0 end 
    foreach tag [.f.html tag names] {
	if {$tag != "sel"} { .f.html tag delete $tag }
    }

    # Recursively process the widgets.
    phxW3EditConvertWidgetToHTML [phxW3CurrentHtext].frame .f.html

    # Convert tags to HTML equivalents.
    tkW3EditConvertToHTML .f.html
    .f.html configure -state disabled
    tkW3OutputCursorNormal
}

proc phxW3EditConvertWidgetToHTML {wFrom wTo} {

    global tkW3HtImages

    switch -glob [phxW3WidgetClass $wFrom] {
	bullet {
	    if {[$wFrom tag ranges DT] != ""} {
		phxW3EditInsertWithTagsRight $wTo "<DT>"
	    } elseif {[$wFrom tag ranges DD] != ""} {
		phxW3EditInsertWithTagsRight $wTo "<DD>"
	    } else {
		phxW3EditInsertWithTagsRight $wTo "<LI>"
	    }
	}
	formsimage -
	checkbox -
	password -
	radio -
	reset -
	select -
	submit -
	text -
	textareaholder {
	    phxW3EditInsertWithTagsRight $wTo \
		[phxW3FormsConvertFormToHTML $wFrom]
	}
	frame -
	imageholder -
	item {
	    foreach ww [pack slaves $wFrom] {
		phxW3EditConvertWidgetToHTML $ww $wTo
	    }
	}
	image {
	    if {![info exists tkW3HtImages($wFrom).anchortag]} {
		set src $tkW3HtImages($tkW3HtImages($wFrom).url)
		phxW3EditInsertWithTagsRight $wTo "<IMG SRC=\"$src\">"
	    } else {
		set src $tkW3HtImages($tkW3HtImages($wFrom).url)
		set start [$wTo index insert]
		phxW3EditInsertWithTagsRight $wTo "<IMG SRC=\"$src\">"
		$wTo tag add Anchor $start insert
		$wTo tag add h$tkW3HtImages($tkW3HtImages($wFrom).anchortag) \
		    $start insert
	    }
	}
	list_* {
	    set listType [string range [phxW3WidgetClass $wFrom] 5 end]
	    if {$listType == "DLC"} {
		phxW3EditInsertWithTagsRight $wTo "<DL COMPACT>"
	    } else {
		phxW3EditInsertWithTagsRight $wTo "<$listType>"
	    }
	    foreach ww [pack slaves $wFrom] {
		phxW3EditConvertWidgetToHTML $ww $wTo
	    }
	    if {$listType == "DLC"} {
		phxW3EditInsertWithTagsLeft $wTo "</DL>"
	    } else {
		phxW3EditInsertWithTagsLeft $wTo "</$listType>"
	    }
	}
	para {
	    if {[$wFrom compare 1.0 < end]} {
		phxW3WidgetCopyText $wFrom 1.0 end $wTo end 1 1 0
		if {[$wFrom get end-1c] == "\n"} {$wTo delete end-1c end}
	    }
	}
	rule {
	    phxW3EditInsertWithTagsRight $wTo "<HR>"
	}
	space {}
	default {
	    phxW3EditInsertWithTagsRight $wTo "<HUH=$wFrom>"
	}
    }
}

proc tkW3EditConvertToHTML {w} {
    global tkW3NavigateIsIndex
    $w mark set insert 1.0

    set headerstring ""
    if {[set title [tkW3OutputGetTitle]] != ""} {
	append headerstring "<TITLE>\n$title\n</TITLE>\n"
    }
    if {$tkW3NavigateIsIndex} {
	append headerstring "<ISINDEX>\n"
    }

    if {$headerstring != ""} {
	phxW3EditInsertWithNoTags $w "<HEAD>\n$headerstring</HEAD>\n"
    }

    phxW3EditInsertWithNoTags $w "<BODY>\n"
    set startbody [$w index insert]

    for {set n 1} {$n <= 6} {incr n} {
	tkW3EditDelimitTag $w H$n "<H$n>" "</H$n>"
    }

# The CharStyles and ParaStyles exist in three places: (Merge, if possible!!)
#    phxW3FontNew
#    config.tcl
#    phxW3EditConvertToHTML
    set delimit_tags {
	{ADDRESS "<ADDRESS>" "</ADDRESS>"}
	{B "<B>" "</B>"}
	{CITE "<CITE>" "</CITE>"}
	{CODE "<CODE>" "</CODE>"}
	{DFN "<DFN>" "</DFN>"}
	{EM "<EM>" "</EM>"}
	{I "<I>" "</I>"}
	{KBD "<KBD>" "</KBD>"}
	{PRE "<PRE>\n" "</PRE>\n"}
	{SAMP "<SAMP>" "</SAMP>"}
	{STRIKE "<STRIKE>" "</STRIKE>"}
	{STRONG "<STRONG>" "</STRONG>"}
	{TT "<TT>" "</TT>"}
	{U "<U>" "</U>"}
	{VAR "<VAR>" "</VAR>"}
	{XMP "<XMP>" "</XMP>"}
    }

    foreach tag $delimit_tags {
	tkW3EditDelimitTag $w [lindex $tag 0] [lindex $tag 1] [lindex $tag 2]
    }

    tkW3EditConvertAnchors $w

    $w mark set insert end
    if {[$w compare "end linestart" < end]} {
	phxW3EditInsertWithNoTags $w "\n"
    }

    phxW3EditInsertParagraphMarks $w $startbody [$w index end]
    phxW3EditInsertWithNoTags $w "</BODY>\n"
    tkW3EditBreakLines $w
}

# phxW3EditInsertParagraphMarks
# 
# Apply these rules in order
# 
# IF \n is in PRE or XMP markup THEN it is ignored.
# IF \n is next to Heading markup (H1, ..., H6) THEN it is ignored.
# IF the \n is not consecutive with others
#    THEN replace \n with "<BR>\n".
# ELSE Replace all consecutive \n's with a "<P>\n\n".

proc phxW3EditInsertParagraphMarks {w startmark endmark} {
    # Find start and end lines of text to convert
    set start [lindex [split $startmark .] 0]
    set end [lindex [split $endmark .] 0]

    # Ignore the \n before the start of text (and the one at the end, too.)
    incr start

    for {set i $start} {$i < $end} {set i [incr j]} {

	# If this \n is PRE or XMP skip it.
	for {set tags [$w tag names $i.0-1c]} \
	    {$i<$end && ([lsearch $tags PRE]>=0 || [lsearch $tags XMP]>=0)} \
	    {incr i
		set tags [$w tag names $i.0-1c]} \
	    {}

	# If nothing was found then we're done.
	if {$i == $end} break

	# Starting with $i.0-1c there is a run of one or more \n's none
	# of which are PRE or XMP.  Find $j.0 so that it is the index
	# after the last \n of the run.
	for {set j $i
		set tags [$w tag names $j.0]} \
	    {$j<$end && [$w compare "$j.0 linestart" == "$j.0 lineend"] && \
		[lsearch $tags PRE]==-1 && [lsearch $tags XMP]==-1} \
	    {incr j
		set tags [$w tag names $j.0]} \
	    {}

	# The range $i.0-1c through $j.0 is consecutive \n's to look at.
	# Only proceed if there is no adjacent header.
	if {![regexp {^H[1-6]$} [phxW3EditDetectParaStyle $w $i.0-2c]] && \
	    ![regexp {^H[1-6]$} [phxW3EditDetectParaStyle $w $j.0]]} \
	{
	    set text [$w get $j.0 end]
	    if {$i == $j} {$w insert $i.0-1c "<BR>"} \
	    else {$w insert $i.0-1c "<P>"}
	}
    }
}
				
# tkW3EditConvertAnchors
#
# We look for anchors by looking for the "Anchor" tag.
# Unless we're careful, abutting anchors will cause problems.

proc tkW3EditConvertAnchors {w} {
    global tkW3HtAnchors
    for {$w mark set insert 1.0} \
	{[set range [$w tag nextrange "Anchor" insert]] != ""} \
	{} \
    {
	# We've found a range of one or more abutting anchors.
	$w mark set insert [lindex $range 0]
	$w mark set ECAend [lindex $range 1]

	while {[$w compare insert < ECAend]} {
	    # Find the tag of the next anchor in this group of abutting anchors
	    set tags [$w tag names insert]
	    set index [lsearch -regexp $tags {h[0-9]+}]
	    set anchortag [string range [lindex $tags $index] 1 end]

	    # Start the anchor
	    set string "<A"
	    if {$tkW3HtAnchors(name.$anchortag) != ""} {
		append string  " NAME=\"$tkW3HtAnchors(name.$anchortag)\""
	    }
	    if {$tkW3HtAnchors(href.$anchortag) != ""} {
		append string " HREF=\"$tkW3HtAnchors(href.$anchortag)\""
	    }
	    append string ">"
	    set ECAstart [$w index insert]
	    phxW3EditInsertWithTagsRight $w $string

	    # Find the end of this anchor.
	    $w mark set insert \
		[lindex [$w tag nextrange h$anchortag $ECAstart] 1]
	    phxW3EditInsertWithTagsLeft $w "</A>"
	}
    }
}

proc tkW3EditBreakLines {w} {
    for {set i 1} {[$w compare $i.0 < end]} {incr i} {
	# If a line contains <LI>, etc. after the first character then
	# insert a newline so that <LI> is at the first character.
	set line [$w get $i.1 "$i.0 lineend"]
	if {[regexp {<LI>|<DT>|<DD>} $line found]} {
	    $w insert $i.[expr [string first $found $line] + 1] "\n"
	}

	set chars [lindex [split [$w index "$i.0 lineend"] "."] 1]
	if {$chars > 72} {
	    for {set j 72} {$j > 0 && [$w get $i.$j] != " "} {incr j -1} {}
	    if {$j != 0} {
		$w delete $i.$j
		$w insert $i.$j "\n"
	    }
	}
    }
}

proc tkW3EditConvertBullets {w text_tag sgml_tag} {
    $w mark set tmp 1.0
    while {[set range [$w tag nextrange $text_tag tmp]] != ""} {
	$w mark set tmp [lindex $range 1]
	$w mark set insert [lindex $range 0]
	$w delete insert tmp
	phxW3EditInsertWithTagsRight $w $sgml_tag
    }
}

proc tkW3EditDeleteTextWithTag {w tag} {
    $w mark set tmp 1.0
    while {[set range [$w tag nextrange $tag tmp]] != ""} {
	$w mark set tmp [lindex $range 0]
	$w delete tmp [lindex $range 1]
    }
}

proc tkW3EditDelimitTag {w tag begin end} {
    $w mark set tmp 1.0
    while {[set range [$w tag nextrange $tag tmp]] != ""} {
	$w mark set tmp [lindex $range 1]

	$w mark set insert [lindex $range 0]
	phxW3EditInsertWithTagsRight $w $begin
	
	$w mark set insert [$w index tmp]
	phxW3EditInsertWithTagsLeft $w $end
    }
}

# phxW3EditInsertWithTagsRight
# Insert a string, inheriting tags from the text to the right.
# By default TK inherits tags from the text to the left.

proc phxW3EditInsertWithTagsRight {w string} {
    set start [$w index insert]
    $w insert insert $string
    foreach tag [$w tag names $start] {
	$w tag remove $tag $start insert
    }
    foreach tag [$w tag names insert] {
	$w tag add $tag $start insert
    }
}

# phxW3EditInsertWithTagsLeft
# Insert a string, inheriting tags from the text to the left.
# This is the default behavior of Tk when inserting.

proc phxW3EditInsertWithTagsLeft {w string} {
    $w insert insert $string
}

# phxW3EditInsertWithNoTags
# Insert a string.  Make sure it has not tags.

proc phxW3EditInsertWithNoTags {w string} {
    set start [$w index insert]
    $w insert insert $string
    foreach tag [$w tag names $start] {
	$w tag remove $tag $start insert
    }
}

## ********************
## These procedure are used to edit the display window
## ********************

proc phxW3EditDeleteTextSelection {window} {
    # Assume that the selection is just one range!!
    set range [$window tag nextrange sel 1.0]
    set start [lindex $range 0]
    set end [lindex $range 1]

    # Delete the selection
    $window delete $start $end

    # We need to patch things up if the before-selection and after-selection
    # are incompatible.  Possible problems:
    # 1. Other than exactly two newlines.
    # 2. Two different Para Styles on the same line
    # 3. Leading white spaces on a line.
    # 4. ???
    if {[$window compare $start != 1.0] && \
	[$window compare $start != end]} \
    {
	if {[$window get "$start -1c"] == "\n" || \
	    [$window get $start] == "\n"} \
	{
	    # There's at lease one new-line.
	    #
	    # Find the beginning of the range of newlines:
	    if {[$window get "$start -1c"] == "\n"} {
		if {[$window get "$start -2c"] == "\n"} {
		    set newlineFirst [$window index "$start -2c"]
		} else {
		    set newlineFirst [$window index "$start -1c"]
		}
	    } else {
		set newlineFirst $start
	    }

	    # Find the end of the range of newlines:
	    if {[$window get $start] == "\n"} {
		if {[$window get "$start +1c"] == "\n"} {
		    set newlineLast [$window index "$start +2c"]
		} else {
		    set newlineLast [$window index "$start +1c"]
		}
	    } else {
		set newlineLast $start
	    }

	    if {[$window compare "$newlineFirst + 2c" != $newlineLast]} {
		# We need to change to exactly two new-lines.
		$window delete $newlineFirst $newlineLast
		$window insert $newlineFirst "\n\n"
		set newlineLast [$window index "$newlineFirst +2c"]
		foreach tag [$window tag names $newlineFirst] {
		    $window tag remove $tag $newlineFirst $newlineLast
		}
	    }

	    # Look for (and delete) leading blanks after the newline.
	    while {[$window get $newlineLast] == " "} {
		$window delete $newlineLast "$newlineLast +1c"
	    }

	} elseif {[phxW3EditDetectParaStyle $window "$start -1c"] != \
	    [phxW3EditDetectParaStyle $window $start]} \
	{
	    # Paragraph style mismatch.  Break the paragraph in two.

	    $window insert $start "\n\n"
	    foreach tag [$window tag names $start] {
		$window tag remove $tag $start "$start +2c"
	    }
	}
    }
}

# phxW3EditCopyTextSelection
# Save the selected text along with its attributes in a cut buffer.

proc phxW3EditCopyTextSelection {w} {

    # Delete the old selection.
    global phxW3ClipboardWidget
    $phxW3ClipboardWidget delete 1.0 end

    # Copy the new selection in.
    set range [$w tag nextrange sel 1.0]
    phxW3WidgetCopyText $w [lindex $range 0] [lindex $range 1] \
	$phxW3ClipboardWidget 1.0
}

# phxW3EditInsertTextSelection
# Paste the cut buffer into the widget.  Returns 0 if one or
# more characters is inserted.

proc phxW3EditInsertTextSelection {w} {
    global phxW3ClipboardWidget

    # If the X buffer has something more recent than the last selection
    # from this application, we'll have to take it.  We lose any markup.
    set x_buff [blt_cutbuffer get 0]
    if {$x_buff == ""} {
	# Nothing inserted so return 1.
	return 1
    }

    if {![winfo exists $phxW3ClipboardWidget] || \
	$x_buff != [$phxW3ClipboardWidget get 1.0 end]} \
    {
	tkW3HtInsert $x_buff inherit
	# Characters inserted so return 0.
	return 0
    }

    # We've got markup with our text.

    # Cannot concatenate headers with plain text or other headers.
    if {[$w compare insert > 1.0] && \
	[phxW3EditDetectParaStyle $w "insert - 1 c"] != \
	[phxW3EditDetectParaStyle $phxW3ClipboardWidget 1.0]} \
    {
	phxW3EditParaInsert $w "\n"
    }

    # Insert the selection buffer
    $w mark set insert \
	[phxW3WidgetCopyText $phxW3ClipboardWidget 1.0 end $w insert]

    # Cannot concatenate headers with plain text or other headers.
    if {[$w compare insert < end] && \
	[phxW3EditDetectParaStyle $w "insert"] != \
	[phxW3EditDetectParaStyle $phxW3ClipboardWidget "end - 1 c"]} \
    {
	phxW3EditParaInsert $w "\n"
    }

    # Characters inserted so return 0.
    return 0
}

proc tkW3EditClearSelection {w} {
    $w tag remove sel 1.0 end
}

## tkW3EditAddAnchor
## -------------
## Turn currently highlighted area into an anchor

proc tkW3EditBeginAnchor {w {link_number ""}} {
    global tkW3HtAnchors
    if {[set t [phxW3WidgetCurrentText]] == ""} {
	return
    }

    if {[$t tag ranges sel] != {} || $link_number != ""} {
	phxW3AnchorDialog
    }
}


proc phxW3EditFillFiles {w {path ""}} {

    global phxW3curdir
    global doctype

    if {$path == ""} {
	if {![info exists phxW3curdir] || $phxW3curdir == ""} {
	    set path [eval pwd]
	} else {
	    set path $phxW3curdir
	}
    } else {
	set files [exec ls -a $path]
	if {[regexp "not found" $files] == 1} {
	    set path [eval pwd]
	}    
    }

    set files [exec ls -a $path]
    if {[regexp "not found" $files] == 1} {
	$w delete 0 end
	return
    }

    set phxW3curdir $path
    $w delete 0 end

    foreach i $files {
	if {$doctype == "html"} {
	    if {[file isdirectory $i] == 1} {
		$w insert end $i
	    } else {
		if {[file isfile $i] == 1} {
		    set fin [open $i]
		    set a [read $fin 20]
		    close $fin
		    if {[regexp <HEAD> $a] == 1} {
			$w insert end $i
		    }
		}
	    }
	} else {
	    if {$doctype == "text"} {
		$w insert end $i
	    }
	}
    }
}

 
proc tkW3EditAddAnchor {name href} {
    global tkW3HtAnchors
    tkW3HtBeginAnc $name $href
    set number $tkW3HtAnchors(index)
    tkW3HtEndAnc

    if {[set t [phxW3WidgetCurrentText]] == ""} {
	return
    }

    if {[$t tag ranges sel] != {}} {
	set start [$t index sel.first]
	set finish [$t index sel.last]
	foreach tag [$t tag names $start] {
	    if [regexp {h[0-9]+} $tag] {
		$t tag remove $tag $start $finish
	    }
	}
	$t tag add Anchor $start $finish 
	$t tag add h$number $start $finish
	$t tag remove sel $start $finish
    }
}

proc tkW3EditTagSelected {w tag {begin 0} {end 0}} {

    set list [tkW3EditClearTagsFromSelected $w]

    if {$list != {}} {
	if {$begin != 0} {
	    $w mark set insert [lindex $list 0]
	    
	}
	$w tag add $tag [lindex $list 0] [lindex $list 1]
	$w mark set insert [lindex $list 1]
	if {$end != 0} {
	    $w mark set insert [lindex $list 1]
	}
	phxW3SaveDirtyBit
    }
}

# phxW3EditBeginList
#
# Create a new list widget in the current frame.

proc phxW3EditBeginList {ltype} {
    global phxW3CurrentWidget

    if {[phxW3ModeGet] != "edit"} {return}
    set right [phxW3SelectionGetRightOfFocus]

    set placement ""
    switch [phxW3WidgetClass $right] {
	end {
	    set placement "-after [phxW3WidgetLeft end]"
	}
	para {
	    if {[phxW3SelectionGetLeftOfFocus] == $right} {
		set before [phxW3WidgetSplitPara $right]
		phxW3SelectionSetFocus $right $before
		set placement "-before $before"
	    }
	}
	image {
	    # Split the imageholder if necessary.
	    set before [phxW3WidgetSplitImageHolder $right]
	    phxW3SelectionSetFocus $right $before
	    set placement "-before $before"
	}
	default { set placement "-before $right" }
    }

    phxW3WidgetCreateList $ltype
    pack configure [phxW3CurrentFrame] $placement

    phxW3SelectionSetFocus $phxW3CurrentWidget
    phxW3SaveDirtyBit
}    

# tkW3EditBeginListItem
#
# Create a new list item in the current frame widget.

proc tkW3EditBeginListItem {} {
    global tkW3EditCurrentStyle phxW3CurrentWidget

    set frame [phxW3CurrentFrame]
    switch -glob [phxW3WidgetClass $frame] {
	list_* -
	item {}
	default {cmdtrace off; return}
    }

    regexp {\.(list_[A-Z]*)[0-9]*(\.item[0-9]*)?$} $frame {} listtype
    set bul ""
    switch $listtype {
	list_DL { set bul "* ";  set listtype "DL" }
	list_UL { set bul "* ";  set listtype "UL" }
	list_OL {
	    set left [phxW3SelectionGetLeftOfFocus]
	    if [regexp {(.*\.item[0-9]*)\.[^.]*$} $left {} lastitem] {
		set bul [$lastitem.bullet get 0.0 end]
	    } else {
		set bul "1."
	    }
	    set listtype "OL"
	}
    }

    phxW3WidgetCreateListItem [string length $bul] $frame
    set frame [phxW3CurrentFrame]
    set bwidget $frame.bullet
    $bwidget insert end $bul
    $bwidget tag add $listtype 1.0 end
    tkW3FontSetTextFonts $bwidget

    if {$listtype == "list_OL"} {
	set siblings [pack slaves [winfo parent $frame]]
	set i [lsearch -exact $siblings $frame]
	while {$i < [llength $siblings]} {
	    set item [lindex $siblings $i]
	    set bul [$item.bullet get 0.0 end]
	    $item.bullet delete 0.0 end
	    $item.bullet insert end [phxW3HtNextBullet $bul]
	    incr i
	}
    }

    phxW3SelectionSetFocus $phxW3CurrentWidget
    phxW3SaveDirtyBit

}

proc tkW3EditClearTagsFromSelected {w} {
    global tkW3EditCurrentStyle

    if {[$w tag ranges sel] != {}} {
	set start [$w index sel.first]
	set finish [$w index sel.last]
	foreach tag [$w	 tag names ] {
	    $w tag remove $tag $start $finish
	}
	return [list $start $finish]
    }
    set tkW3EditCurrentStyle Normal
    return {}
}

proc tkW3EditInsertBullet {w type} {
    global tkW3EditCurrentStyle
    switch $type {
	"LI" {
	    tkW3HtBuf offset " "
	    tkW3HtInsert "* " ListItem
	}
	"DD" {
	    tkW3HtInsert "* " GlossaryDefinition
	}
	"DT" {
	    tkW3HtInsert "* " GlossaryTerm
	}
	"P" {
	    tkW3HtInsert "\n\n" ParagraphBreak
	}
    }
}

proc tkW3EditCreateNewPage {page_name} {

    global CurrentHtext
    if {[info exists CurrentHtext]} {
	if [phxW3SaveWarnAboutLostChanges] { return }
	# Record the current page in the history list.
	phxW3HistoryRecordAndPush \
	    [tkW3OutputGetTitle] [tkW3OutputGetAddress] \
	    [tkW3OutputGetScrollPosition]  
    }
    
    # Get a new htext and forget the old
    if {[phxW3CacheGet {}] != ""} {
	# Load the page from the cache.
	phxW3CacheLoadPageFromServer {}
    } else {
	set htext [phxW3CacheHtextNewName]
	if {[info exists CurrentHtext]} { pack forget $CurrentHtext }
	phxW3OutputMakeHtext $htext
	pack $htext -side top -fill both
	set CurrentHtext $htext

	# Put empty contents into the new page
	tkW3HtBeginDoc
	# Create a text widget within the document.
	set text [phxW3WidgetCurrentText]
	tkW3HtSetName $page_name "New Page"
	tkW3HtEndDoc
	tkW3OutputSetScrollPosition 0

	# Remember our new page in the cache
	phxW3CacheAdd {} $htext [tkW3OutputGetTitle]
    }

    tkW3ConfigDisplay
    phxW3HistoryInsensitize [tkW3OutputGetTitle] [tkW3OutputGetAddress] \
	[tkW3OutputGetScrollPosition] 

    # Now go to edit mode on our new page.
    phxW3ModeEdit
    phxW3SaveCleanBit

    # Set the focus appropriately
    phxW3SelectionSetFocus start [phxW3WidgetRight start]
}

proc phxW3EditCreateNewPageName {} {
    return ""
}
