################################################################
# Modifications beyond tkWWW 0.9 are Copyright 1993, 1994 by
# The University of Chicago Biological Sciences Division, Academic Computing
# Philip Stylianos
# Lee Newberg
################################################################

proc phxW3FileboxInitialize {} {
    global currentPath currentFile
    global envhome

    set currentPath [pwd]
    set currentPath $envhome
    set currentFile ""
    set permissions "r"
}


### Call this procedure from other procedures to get a filename from the
### user.  Pass it nothing if you want user to select from Web-readable,
### OS readable files.  Pass a w if you want to choose only from OS
### writable files.  Pass a list with filename extensions as the second arg
### if you want to limit the types of files (as in selecting an image).
### It returns name of file selected or "" if none selected.
### All other procs in this file support this procedure.

proc phxW3Filebox {{p r} {filter ""}} {
    global FileboxResult currentFile currentPath
    global permissions
    global phxW3FileboxFilter
    global tkW3ConfigFileSuffixes

    # set scope for "filename" variable
    set filename ""

    # Use global var to hold permissions so don't have to pass to every
    # proc
    set permissions $p

    # Set filter
    set phxW3FileboxFilter $filter

    # Build or deiconify already-drawn filebox
    phxW3FileboxCreate 
    wm deiconify .filebox    

    # Set the current file to the highlighted file in the listbox, if any
    set currentFile ""
    if {[set newfile [lindex [.filebox.files.listbox curselect] 0]] != ""} {
	phxW3FileboxSetFile [.filebox.files.listbox get $newfile]
    }

    # Wait for OK or Cancel
    tkwait variable FileboxResult

    # Set the return val to null initially; if OK and either Path or
    # File is non-null, build full path out of those
    if {$FileboxResult == "OK"} {
	switch $currentFile {
	    "" {set filename $currentPath}
	    / {set filename /$currentFile}
	    default {set filename $currentPath/$currentFile}
	}
    }

    # Hide the box, return the filename
    wm withdraw .filebox
    return $filename
}


# This proc builds the filebox if not created yet.  It builds the directory
# menu in either case

proc phxW3FileboxCreate {} {
    
    global phxfont

    # If filebox already around, reset directory menu to reflect Current
    # Dir
    if [winfo exists .filebox] {
	phxW3FileboxSetPath 
	return
    }

    # The window
    toplevel .filebox
    wm withdraw .filebox
    wm geometry .filebox 400x250+100+100 
    wm title .filebox {File select box}
    wm maxsize .filebox 1000 1000
    wm minsize .filebox 100 100
    wm sizefrom .filebox user

    # Frame 1/3 holds directory menu to be built later
    frame .filebox.path -relief flat

    # Frame 2/3 builds the 2 listboxes; dirs on left, files on right
    frame .filebox.files -relief flat
    listbox .filebox.files.listbox \
	-relief sunken -bd 1 \
	-yscrollcommand {.filebox.files.scrollbar set}

    scrollbar .filebox.files.scrollbar \
	-command {.filebox.files.listbox yview}
    PhxApp_standardize_windowwidg .filebox.files.scrollbar
    .filebox.files.scrollbar configure -relief sunken -bd 1

    frame .filebox.files.space -width 4 -relief flat

    scrollbar .filebox.files.dirscrollbar \
	-command {.filebox.files.dirlistbox yview}
    PhxApp_standardize_windowwidg .filebox.files.dirscrollbar
    .filebox.files.dirscrollbar configure -relief sunken -bd 1

    listbox .filebox.files.dirlistbox \
	-relief sunken -bd 1 \
	-yscrollcommand {.filebox.files.dirscrollbar set}

    # Bindings for the listboxes

    # Single click - write the file in the text widget
    bind .filebox.files.listbox <ButtonRelease-1> \
	"+ if {\[set newfile \[lindex \[%W curselect\] 0\]\] != \"\"} {
	    phxW3FileboxSetFile \[%W get \$newfile\]
	}    "
    # Double click - give the OK for the selected file
    bind .filebox.files.listbox <Double-1> \
	"+phxW3FileboxGotoFile "

    # Triple click - nop
    bind .filebox.files.listbox <Triple-1> ""

    # Single click dirbox - same as filebox, write dir in text widget
    bind .filebox.files.dirlistbox <ButtonRelease-1> \
	"+ if {\[set newfile \[lindex \[%W curselect\] 0\]\] != \"\"} {
	    phxW3FileboxSetFile \[%W get \$newfile\]
	}    "

    # Double click - same as filebox, go to the directory (no OK)
    bind .filebox.files.dirlistbox <Double-1> \
	"+phxW3FileboxGotoFile "

    # Triple click - nop
    bind .filebox.files.dirlistbox <Triple-1> ""

    # Pack widgets in files frame
    pack configure .filebox.files.dirlistbox -side left -expand yes -fill both
    pack configure .filebox.files.dirscrollbar -side left -fill y
    pack configure .filebox.files.space -side left -fill y
    pack configure .filebox.files.scrollbar -side left -fill y
    pack configure .filebox.files.listbox -side left -expand yes -fill both

    # The text widget for typing in filenames - return = OK button
    frame .filebox.filename -relief flat
    text .filebox.filename.text -bd 1 -height 1 \
	-font $phxfont(bigbuttonlabel)
    bind .filebox.filename.text <Return> { .filebox.buttons.ok invoke }
    pack configure .filebox.filename.text -side left -fill x


    # Button frame 

    frame .filebox.buttons -relief flat
    button .filebox.buttons.ok \
	-text OK \
	-command "phxW3FileboxGotoFile"
    button .filebox.buttons.cancel \
	-text Cancel \
	-command "global FileboxResult ; set FileboxResult Cancel"
    pack configure .filebox.buttons.ok .filebox.buttons.cancel \
	-side left -expand yes

    pack configure .filebox.path .filebox.filename -side top -fill x
    pack configure .filebox.buttons -side bottom -fill x
    pack configure .filebox.files -side top -fill both -expand yes

    phxW3FileboxSetPath 
}


# This proc takes a file name, partial or full path and sets the currentPath
# and File globals.  Also writes pertinent data to text widget

proc phxW3FileboxSetFile {file} {
    global currentPath
    global currentFile

    # In case more than one filename sent form listbox, take first one
    set filelist [split $file " "]    
    set file [lindex $filelist 0]

    # If full path given, set Path and File, otherwise just set file
    if {[regexp ^/ $filelist]} {
	set currentPath [file dirname $file]
	set currentFile [file tail $file]
    } else {
	set currentFile $file
    }

    .filebox.filename.text delete 1.0 end
    if {![file isdirectory $currentPath/$currentFile]} {
	.filebox.filename.text insert 1.0 $file
    }
}

proc phxW3FileboxGotoFile {{file ""}} {
    global currentPath currentFile
    global permissions

    tkW3OutputCursorWait
    update idletasks

    if {[.filebox.filename.text get 1.0 end] != ""} {
	phxW3FileboxSetFile [.filebox.filename.text get 1.0 end]
    }

    if {$file == ""} {
	set file $currentFile
    }
    set filelength [string length $file]
    if {[string index $file [expr $filelength - 1]] == "\n"} {
	set file [string range $file 0 [expr $filelength - 2]]
    }
    set filelist [split $file " "]
    set file [lindex  $filelist 0]
    if {[file isdirectory $currentPath/$file]} {
	phxW3FileboxChangeDir $file
	tkW3OutputCursorNormal
	return
    } 

    if {[regexp r $permissions]} {
	if {![file readable $currentPath/$file]} {
	    DLG:error . .error \
	      "You do not have permission to read this file" "OK"
	    tkW3OutputCursorNormal
	    return
	}
    }

    if {[regexp w $permissions]} {
	if {[file exists $currentPath/$file] && \
	    ![file writable $currentPath/$file]} {
	    DLG:error . .error \
	      "You do not have permission to write to this file" OK
	    tkW3OutputCursorNormal
	    return
	}
	if {![file exists $currentPath/$file] && \
	    ![file writable $currentPath]} {
	    DLG:error . .error \
	      "You do not have permission to write to this directory" OK
	    tkW3OutputCursorNormal
	    return
	}
	set filelist [split $file .]
	if {[lindex $filelist [expr [llength $filelist] - 1]] \
	    != "html"} \
	{
	    lappend filelist "html"
	    set file [join $filelist .]
	    set result [DLG:warning . .warning \
	      "Saving instead as $file" OK Cancel]
	    if {$result == 2} {
		tkW3OutputCursorNormal
		return
	    }
	}
    }

    global FileboxResult 
    set currentFile $file
    set FileboxResult OK
    tkW3OutputCursorNormal
    update idletasks
}

proc phxW3FileboxChangeDir {dir} {

    global currentPath

    if {$dir == ".."} {
	set tmp [file dir $currentPath]
    } else {
	if [regexp /$ $currentPath] {
	    set tmp $currentPath$dir
	} else {
	    set tmp $currentPath/$dir
	}
    }
    phxW3FileboxSetPath $tmp
}

proc phxW3FileboxSetPath {{path ""} } {

    global currentPath

    tkW3OutputCursorWait
    update idletasks

    if {$path == ""} {
	set path $currentPath
    } 

    if {![file isdirectory $path] || ![file executable $path]} {
	DLG:error . .error \
	    "You do not have permission to enter this directory" "OK"
	tkW3OutputCursorNormal
	return
    }

    set currentPath $path

    foreach i [winfo children .filebox.path] {
	foreach j [winfo children $i] {
	    destroy $j
	}
	destroy $i
    }

    phxW3FileboxBuildPath $path
    phxW3FileboxGetFiles "" 
    tkW3OutputCursorNormal
}

proc phxW3FileboxGetFiles {{path ""} } {

    global currentPath
    global phxW3FileboxFilter

    if {$path == ""} {
	set path $currentPath
    }

    set files [exec ls -a $path]

    .filebox.files.listbox    delete 0 end
    .filebox.files.dirlistbox delete 0 end

    set filei 0
    foreach i $files {
	if {$i == "."} {continue}
	if {[file isdirectory $path/$i]} {
	    .filebox.files.dirlistbox insert end "$i /"
	} else {
	    if {$phxW3FileboxFilter == "" || \
		[lsearch -regexp $phxW3FileboxFilter [file extension $i]]>=0} \
	    {
		.filebox.files.listbox insert $filei $i
		incr filei
	    }
	}
    }


    if {[set file [.filebox.files.listbox curselect]] != ""} {
	set file [.filebox.files.listbox get [lindex $file 0]]
	if {![file isdirectory $currentPath/$file]} {
	    phxW3FileboxSetFile $file
	}
    } else {
	phxW3FileboxSetFile ""
    }
}

proc phxW3FileboxBuildPath {path} {

    global phxfont

    set dirs [split $path /]

    set i 0
    set lasti [expr [llength $dirs]-1]
    if {[lindex $dirs $lasti] == ""} {
	incr lasti -1
	set dirs [lrange $dirs 0 $lasti]
    }
    set pathsofar ""
    foreach dir $dirs {
	set pathsofar "$pathsofar/$dir"
	if {$i == 0} { set dir / }

	menubutton .filebox.path.$i \
	    -text $dir \
	    -font $phxfont(bigbuttonlabel) \
	    -relief raised \
	    -bd 1 \
	    -padx 2 \
	    -menu .filebox.path.$i.menu \
	    -activeforeground blue

	.filebox.path.$i configure -activebackground \
	    [lindex [.filebox.path.$i configure -bg] 4]

	set menuname [menu .filebox.path.$i.menu -activeforeground blue]
	.filebox.path.$i.menu configure -activebackground \
	    [lindex [.filebox.path.$i.menu configure -bg] 4]

	set files [exec ls -a $pathsofar]
	set subdirs ""
	foreach j $files {
	    if {$j == "."} {
		if {$pathsofar == ""} { 
		    set path "/"
		} else {
		    set path $pathsofar
		}
	    } else {
		set path $pathsofar/$j
	    }
	    #if {$pathsofar == ""} {puts "$path $j [file isdirectory $path]"}
	    if {[file isdirectory $path]} {
		if {$j != ".."} {
		    lappend subdirs [list $path $j]
		}
	    }
	}

	if {$i == 0} {set pathsofar ""}

	set count 0
	set numleft [llength $subdirs]
	set nummenus 1
	foreach j $subdirs {

	    set path [lindex $j 0]
	    set name [lindex $j 1]

	    if {$count == 10} {
		set count 0
		set menuname .filebox.path.$i.menu$nummenus
	    }
	    if {$count == 0 && $numleft > 10} {
		incr nummenus
		$menuname add cascade \
		    -label More \
		    -menu .filebox.path.$i.menu$nummenus
		menu .filebox.path.$i.menu$nummenus -activeforeground blue
		.filebox.path.$i.menu$nummenus configure -activebackground \
		    [lindex [.filebox.path.$i.menu$nummenus configure -bg] 4]
	    }
	    $menuname add command \
		-label $name \
		-command "phxW3FileboxSetPath \
		    $path"
	    incr count
	    incr numleft -1
	}

	pack configure .filebox.path.$i -side left
	incr i
    }

    eval tk_menuBar .filebox.path [winfo children .filebox.path]

}
