################################################################
# Modifications beyond tkWWW 0.9 are Copyright 1993, 1994 by
# The University of Chicago Biological Sciences Division, Academic Computing
# Philip Stylianos
# Lee Newberg
################################################################

proc phxW3HotlistInitialize {} {

	global phxW3Hotlist envhome
	set phxW3Hotlist(filename) $envhome/.mosaic-hotlist-default
	phxW3HotlistRead $phxW3Hotlist(filename)
}

proc phxW3HotlistRead {{filename ""}} {

	global phxW3Hotlist

	set phxW3Hotlist(count) 0
	set phxW3Hotlist(addrlist) ""

	if {$filename == ""} {
		set filename $phxW3Hotlist(filename)
	}

	if [catch {set file [open $filename "r"]}] {
		return
	}
	if [regexp Error $file] {
		return
	}

	set state "address"

	while {[gets $file line] != -1} {
		if {[llength $line] == 1 && \
			[regexp ncsa-xmosaic-hotlist-format $line]} {

			gets $file line
			continue
		}

		case $state {
			address {
				set address [lindex $line 0]
				set date [lreplace $line 0 0]
				set state title
			}
			title {
				set title $line
				set state address
				phxW3HotlistAdd $title $address $date
			}
		}
	}
		
	close $file
}

proc phxW3HotlistAdd {title address {date ""}} {
	
	global phxW3Hotlist

	if {$date == ""} {
		set date [exec date]
	}

	if [info exists phxW3Hotlist(count)] {
		set i [incr phxW3Hotlist(count)]
	} else {
		set i [set phxW3Hotlist(count) 1]
	}

	set phxW3Hotlist($i.address) $address
	set phxW3Hotlist($i.title) $title
	set phxW3Hotlist($i.date) $date

	if [info exists phxW3Hotlist(addrlist)] {
		lappend phxW3Hotlist(addrlist) $address 
	} else {
		set phxW3Hotlist(addrlist) $address
	}

	tkW3MenuAddToNodeMenu .menu.hotlist.m \
		[list $title $address 0]
}


proc phxW3HotlistAddPage {} {

	global phxW3Hotlist

	set title [tkW3OutputGetTitle]
	set address [tkW3OutputGetAddress]

	if {[lsearch $phxW3Hotlist(addrlist) $address] > -1} {
		return
	}

	phxW3HotlistAdd $title $address

	phxW3HotlistWrite
}


proc phxW3HotlistWrite {} {

	global phxW3Hotlist

	set file [open $phxW3Hotlist(filename) "w"]
	if [regexp "Error" $file] {
		tkW3OutputError "Cannot write to your hotlist file"
		return
	}

	puts $file "ncsa-xmosaic-hotlist-format-1\nDefault"

	for {set i 1} {$i <= $phxW3Hotlist(count)} {incr i} {

		puts $file "$phxW3Hotlist($i.address) $phxW3Hotlist($i.date)"
		puts $file $phxW3Hotlist($i.title)
	}

	close $file
}

proc phxW3HotlistDelete {} {

	
	if [info exists .delhotlist] {
		phxW3HotlistFillDelBox
		wm deiconify .delhotlist
		return
	}
	toplevel .delhotlist
	wm withdraw .delhotlist
	wm geometry .delhotlist 400x250+100+100 
	wm title .delhotlist {Delete from Hotlist}
	wm maxsize .delhotlist 1000 1000
	wm minsize .delhotlist 100 100
	wm sizefrom .delhotlist user

	frame .delhotlist.files
	frame .delhotlist.buttons \
		-relief flat

	set l [listbox .delhotlist.files.listbox]
	set sl [listbox .delhotlist.files.secretlistbox]
	bind .delhotlist.files.listbox <Double-1> \
		"
		$l delete \[%W nearest %y\]
		$sl delete \[%W nearest %y\]
		"
	button .delhotlist.buttons.delete \
		-text Delete \
		-command "
			set s \[$l curselect\] 
			if {\$s != \"\"} {
				$sl delete \[lindex \$s 0\] \\
				\[lindex \$s \[expr \[llength \$s\]-1\]\]
				$l delete \[lindex \$s 0\] \\
				\[lindex \$s \[expr \[llength \$s\]-1\]\]
			}
			"
	button .delhotlist.buttons.ok \
		-text OK \
		-command "phxW3HotlistSaveFromListboxes ; wm withdraw .delhotlist"
	button .delhotlist.buttons.cancel \
		-text Cancel \
		-command "wm withdraw .delhotlist"

	pack append .delhotlist.files .delhotlist.files.listbox {left fill expand}
	#pack append .delhotlist.files .delhotlist.files.secretlistbox {left fill}
	pack append .delhotlist.buttons .delhotlist.buttons.delete {left expand}
	pack append .delhotlist.buttons .delhotlist.buttons.ok {left expand}
	pack append .delhotlist.buttons .delhotlist.buttons.cancel {left expand}
	pack append .delhotlist .delhotlist.buttons {bottom fillx}
	pack append .delhotlist .delhotlist.files {top fill expand}

	wm deiconify .delhotlist
	phxW3HotlistFillDelBox
}

proc phxW3HotlistFillDelBox {} {

	global phxW3Hotlist

	set l ".delhotlist.files.listbox"
	set sl ".delhotlist.files.secretlistbox"

	if {[winfo exists $l] == 0} {
		return
	}

	$l delete 0 end
	$sl delete 0 end

	for {set i 1} {$i <= $phxW3Hotlist(count)} {incr i} {
		$l insert end $phxW3Hotlist($i.title)
		$sl insert end \
			"$phxW3Hotlist($i.address) $phxW3Hotlist($i.date)"
	}
}

proc phxW3HotlistSaveFromListboxes {} {

	global phxW3Hotlist
	global tkW3ConfigMenus

	set phxW3Hotlist(count) 0
	set phxW3Hotlist(addrlist) ""

	set l ".delhotlist.files.listbox"
	set sl ".delhotlist.files.secretlistbox"

	for {set i 0} {$i < [$l size]} {} {
		set title [$l get $i]
		set address [lindex [$sl get $i] 0]
		set date [lreplace [$sl get $i] 0 0]
		incr i
		set phxW3Hotlist($i.title) $title
		set phxW3Hotlist($i.address) $address
		set phxW3Hotlist($i.date) $date
		lappend phxW3Hotlist(addrlist) $address
	}
	set phxW3Hotlist(count) $i

	phxW3HotlistWrite
	tkW3MenuMakeNodeMenu .menu.hotlist.m [lindex [lindex $tkW3ConfigMenus 4] 1]
	phxW3HotlistRead
}
