################################################################
# Modifications beyond tkWWW 0.9 are Copyright 1993, 1994 by
# The University of Chicago Biological Sciences Division, Academic Computing
# Philip Stylianos
# Lee Newberg
# Tim Pierce
################################################################

## hypertext.tcl Procedures for processing the hypertext
## ==============
## Copyright (C) 1992-1993
## Global Network Academy
## Macvicar Institute for Educational Software Development
##
## See the file COPYRIGHT for conditions

## Conventions:
## The procedures in this procedure have names which begin with tkW3Ht

## Comments/additions by Phoenix programmers Philip Stylianos, Lee Newberg.

proc phxW3HtInitialize {} {
    global tkW3HtAnchors tkW3HtMaxAnchor tkW3HtNameToHnumber
    global phxW3HtBuildingDocument

    set tkW3HtAnchors(index) 1
    set tkW3HtAnchors(in_anchor) 0
    set tkW3HtAnchors(in_textarea) 0
    set tkW3HtMaxAnchor 0
    set tkW3HtNameToHnumber() bar
    set phxW3HtBuildingDocument 0
}

# tkW3HtInsert
# Insert arbitary text into the current text widget.
# Comments are outside the code for speed.

# Configure the window for writing
# If the string is a single character or if arbitrary formatting is allowed,
#   then call phxW3EditParaInsert immediately.
# Else
#   Split the string into paragraphs
#   Insert the first paragraph
#   For each additional paragraph
#      Insert a paragraph separator (\n)
#      Insert the additional paragraph
# Return the widget to its previous state

proc tkW3HtInsert {text {tags ""}} {
	[set window [phxW3WidgetCurrentText]] config -state normal
	if {[string length $text] <= 1 || [phxW3HtAllowWhitespaces $tags]} \
		{ phxW3EditParaInsert $window $text $tags } \
	else {
		set paragraphs [split $text "\n"]
		phxW3HtInsertParagraph $window [lindex $paragraphs 0] $tags
		foreach paragraph [lrange $paragraphs 1 end] {
			phxW3EditParaInsert $window \n
			phxW3HtInsertParagraph $window $paragraph $tags }}
	global phxW3ModeTextState
	$window config -state $phxW3ModeTextState
}

# tkW3HtInsertParagraph
# Insert unpreformatted text containing no "\n".
# Comments are outside the code for speed.

# Compress multiple spaces to a single space.
# If the string starts with a space we insert it separately so that
#   consecutive spaces do not occur.
# Insert the bulk of the text.
# A trailing space is inserted separately.

proc phxW3HtInsertParagraph {window para tags} {
	regsub -all " +" $para " " new_para
	if {[string index $new_para 0] == " "} {
		phxW3EditParaInsert $window " " $tags
		set new_para [string range $new_para 1 end] }
	set length [string length $new_para]
	if {[string index $new_para [incr length -1]] == " "} {
		phxW3EditParaInsert $window [string range $new_para 0 [incr length -1]] $tags
		phxW3EditParaInsert $window " " $tags } \
	else { phxW3EditParaInsert $window $new_para $tags }}

# phxW3HtAllowWhitespaces
# Checks whether consecutive white spaces are permitted.

proc phxW3HtAllowWhitespaces {tags} {
	foreach tag $tags \
		{ if {[lsearch {PRE DT DD LI ADDRESS} $tag] != -1} { return 1 } }
	return 0
}

proc tkW3HtSetName {address title} {

    # The HT way to change the title and URL for the current document
    tkW3OutputSetAddress $address $title
    return ""
}
 
proc tkW3HtBeginDoc {} {
	global tkW3HtAnchors  tkW3HtImages
	global phxHtList tkW3HtStyleStack tkW3NavigateIsIndex
	global tkW3HtNameToHnumber tkW3HtCharStyle 
	global tkW3HtMaxAnchor
	global phxW3HtBuildingDocument

	# Initialize
	#set tkW3HtAnchors(index) 1
	#set tkW3HtMaxAnchor 0
	set tkW3HtImages(index) 1
	set tkW3HtAnchors(in_anchor) 0
	set phxHtList(level) 0
	set tkW3HtStyleStack {}
	set tkW3HtCharStyle {}
	set tkW3NavigateIsIndex 0
#	unset tkW3HtNameToHnumber
	set tkW3HtNameToHnumber() bar
	set phxW3HtBuildingDocument 1

	# Clear the text widget 
	tkW3OutputClearBody

	set htext [phxW3CurrentHtext]
	$htext config -text "%%$htext append $htext.frame
		$htext childconfig $htext.frame -fill both -relwidth 1
		%%
		"
	#update 

	#tkW3HtHide
}

proc tkW3HtBeginAnc {name href} {
	global tkW3HtAnchors tkW3HtNameToHnumber tkW3HtMaxAnchor phxW3CurrentWidget

	# Set HtAnchor array to reflect new anchor in text
	set i $tkW3HtAnchors(index)
	set tkW3HtAnchors(name.$i) $name
	set tkW3HtAnchors(href.$i) $href
	set tkW3HtAnchors(widg.$i) $phxW3CurrentWidget
	set tkW3HtNameToHnumber([phxW3CurrentHtext],$name) $i
	set tkW3HtAnchors(in_anchor) 1

	# ??? Make sure index has caught up to count of anchors ???
	if {$tkW3HtAnchors(index) <= $tkW3HtMaxAnchor} {return}
	incr tkW3HtMaxAnchor

	if {[phxW3WidgetClassCurrentWidget] != "para"} {return}
	tkW3HtMakeAnchor $i $name $href
}

# tkW3HtMakeAnchor

# The anchor is being created or changed.  It has button-like
# properties if $href is not empty.  It is red if $href is not
# empty, blue if $name is not empty, and red and blue
# (i.e. magenta) if both $href and $name are not empty.  The
# string "h$i" is the tag used to represent the anchor in the
# text widget.

proc tkW3HtMakeAnchor {i {name ""} {href ""}} {

    set text [phxW3WidgetCurrentText]

    if {$href != ""} {
	if {$name == ""} {$text tag configure h$i -foreground red} \
	else {$text tag configure h$i -foreground magenta}

	# Make anchor act like a button
	$text tag bind h$i <ButtonRelease-1> \
	    "phxW3ModeSingleClickAnchor %W $i"
	$text tag bind h$i <Double-Button-1> \
	    "phxW3ModeDoubleClickAnchor %W $i"
    } else {
	$text tag configure h$i -foreground blue
	$text tag bind h$i <ButtonRelease-1> ""
	$text tag bind h$i <Double-Button-1> ""
    }
}

proc tkW3HtEndAnc {} {
	global tkW3HtAnchors
	set tkW3HtAnchors(in_anchor) 0
	incr tkW3HtAnchors(index)
}


# Insert given string using the current styles.
# This procedure and tkW3HtInsert are called when text is read
# from a file.	Once in edit mode, phxW3EditParaInsert is used.
# The comments have been removed from the body of the code for speed.

# This sets styles by appending
#  The current tkW3HtCharStyle list
#  Anchor and h$index (if appropriate)
#  Top of the style stack

proc tkW3HtAdd {string} {
	if {$string == ""} return
	global tkW3HtAnchors tkW3HtStyleStack tkW3HtCharStyle HtForms
	if {$HtForms(in_textarea)} {return [phxW3FormsAddTextarea $string]}
	if {$HtForms(in_select)} {return [phxW3FormsAddSelect $string]}
	set styles $tkW3HtCharStyle
	if {$tkW3HtAnchors(in_anchor) == 1} \
		{ lappend styles Anchor h$tkW3HtAnchors(index) }
	lappend styles [lindex $tkW3HtStyleStack [expr [llength $tkW3HtStyleStack]-1]]
	tkW3HtInsert $string $styles
	return
}




proc tkW3HtBeginPar {elem} {

		global tkW3HtStyleStack phxHtList
	# Add to the style stack
		lappend tkW3HtStyleStack $elem
   
	# If element is an heading /*or address*/, generate blank lines
	if {[regexp {^H[1-9]+$} $elem] } {
		tkW3HtBlankLines 2
	} else {
		# For lists
		if {$phxHtList(level) == 0 } {
			if {$elem == "MENU" || $elem == "DL" || \
			$elem == "DLC" || $elem == "UL" || $elem == "OL"} {
#				tkW3HtBlankLines 1
			}
		}
	}
  
	# If a list is being started, create a new list widget in which
	# to pack list items
	if {$elem == "MENU" || $elem == "DL" || $elem == "DLC" || \
		$elem == "UL" || $elem == "OL" || $elem == "BLOCKQUOTE"} {
		set l [incr phxHtList(level)]
		set phxHtList($l.type) $elem
		set phxHtList($l.count) 0
		if {$l == 1} {
			tkW3HtBlankLines 1
		}
		phxW3WidgetCreateList $elem
#		tkW3HtInsert " " $elem.mark
	}
}

proc tkW3HtEndPar {elem} {

	global tkW3HtStyleStack phxHtList phxW3CurrentWidget

	# Pop top element off style stack
		set index [expr "[llength $tkW3HtStyleStack]-2"]
		set tkW3HtStyleStack [lrange $tkW3HtStyleStack 0 $index]

	# Blank lines follow a heading /* or address */
	if {[regexp {^H[1-9]+$} $elem]	} {
		tkW3HtBlankLines 2
	} 

	# End of list
	if {$elem == "MENU" || $elem == "DL" || $elem == "DLC" || \
		$elem == "UL" || $elem == "OL" || $elem == "BLOCKQUOTE"} {

		# Insert a carriage return in the closing list style
		# Do we really need this?
#		tkW3HtInsert "" "/$elem.mark"

		# Reset list type param]]
		set l $phxHtList(level)
		set phxHtList($l.type) ""

		incr phxHtList(level) -1

		# Insert a blank line
		if {$phxHtList(level) == 0} {
			tkW3HtInsert "\n"
		}

		# Remind phxW3CurrentWidget and Frame that we've ended the list
		set frame [phxW3CurrentFrame]
		regsub {\.list_[A-Z0-9]*(\.item[0-9]*)?$} $frame "" newframe
		set phxW3CurrentWidget [phxW3CurrentFrame $newframe]
	}
}

proc tkW3HtBeginChar {elem} {

	global tkW3HtCharStyle

	set tkW3HtCharStyle $elem
}

proc tkW3HtEndChar {elem} {

	global tkW3HtCharStyle

	set tkW3HtCharStyle ""
}



proc tkW3HtEndDoc {} {
	global tkW3HtImages
	global phxW3HtBuildingDocument

	set phxW3HtBuildingDocument 0
	set address [tkW3OutputGetAddress]
}

# tkW3HtAddBul
# Comments are outside code for speed.

# Special cases for type "P" and "HR".
# Insert a blank line for a new bullet.
# Put in "tabs" reflecting the nesting of lists.
# Insert a bullet for the list item.

proc tkW3HtAddBul {type} {
    global phxW3ModeTextState phxHtList phxW3CurrentWidget
    switch $type {
	"P" {if {[tkW3HtBlankLines 2] == 0} {tkW3HtInsert \n P} ; return}
	"DT" {tkW3HtBlankLines 1}
	"HR" {phxW3WidgetAddHR ; return}
	"BR" {tkW3HtInsert "\n"; return}}
    tkW3HtBlankLines 1
    switch $type {
	"LI" {
	    set l $phxHtList(level)
	    set ltype $phxHtList($l.type)
	    if {$l == 0 || $ltype == "UL" || $ltype == "MENU"} {
		set bul "* "
	    } else {
		incr phxHtList($l.count)
		set bul "[tkW3HtListCounter $l $phxHtList($l.count)] "
	    }
	}
	"DT" {set bul "* "}
	"DD" {set bul "   "}
	default {return}}
    phxW3WidgetCreateListItem [string length $bul]
    set bwidget [phxW3CurrentFrame].bullet
    $bwidget configure -state normal
    $bwidget insert end $bul
    $bwidget configure -state $phxW3ModeTextState
    $bwidget tag add $type 1.0 end
    tkW3FontSetTextFonts $bwidget
}

proc tkW3HtListCounter {level count} {
    global ordinal
    set prefix ""
    set suffix ""
    for {} {$level > 2} {incr level -2} {
	append prefix "("
	append suffix ")"
    }
    append suffix "."
    if {$level == 1} {
	append prefix $count
    } else {
	for {} {$count > 0} {set count [expr $count / 26]} {
	    set suffix $ordinal([expr $count % 26])$suffix
	}
    }
    return $prefix$suffix
}

proc phxW3HtNextBullet {prevbullet} {
    global ordinal
    regexp {([ \(]*)([a-z0-9]+)([\).]*)} $prevbullet {} prefix content suffix
    if [string match {*[a-z]} $content] {
	for {set total 0; set i 0} {$i < [string length $content]} {incr i} {
	    set total [expr $total*26 + $ordinal([string index $content $i])]
	}
	incr total;
	for {set content ""} {$total > 0} {set total [expr $total / 26]} {
	    set content $ordinal([expr ($total % 26)])$content
	}
    } else {
	incr content
    }
    return $prefix$content$suffix
}

proc tkW3HtSetImg {url ismap {before ""}} {

    # This code mimics part of phxW3FormsInsertInput to some degree!!
    global tkW3HtImages tkW3HtAnchors

    set address [tkW3OutputGetAddress]

    set page_value [tkW3NavigateServerSend [list HtParseName $url $address]]
    set result [tkW3NavigateServerSend [list HtLoad $page_value]]

    # Check that it was loaded.
    if {[regexp {(^Error)} $result]} { return }

    set source ""
    if {[llength $result] == 6 && [lindex $result 2] != "tkW3HtSetName"} {
	set source [eval [lrange $result 0 2]]
    }
    if {$source == ""} {
	return
    }

    # Display the new image
    set widget [phxW3WidgetNewImage $url $source $ismap $before]    

    return
}

proc tkW3HtSetInd {} {

    global tkW3NavigateIsIndex

    # Back end way to indicate the page is index
    set tkW3NavigateIsIndex 1
}

# tkW3HtBlankLines n
# Insert n carriage returns here if the current line is not blank.

proc tkW3HtBlankLines {n} {

    # See if current line is blank
    set blank [regexp {^[ ]*$} \
	    [[phxW3WidgetCurrentText] get {insert linestart} \
	    {insert - 1 c lineend}]]
	# If not a blank line, insert all the blank lines you want
	if {!$blank} {

	    for {set i 0} {$i<$n} {incr i} {
		tkW3HtInsert "\n" 
	    }
	    return $n
	}

    return 0
}

################################################################
# Forms routines are in forms.tcl

# phxW3HtBeginForm is called when the <FORM> markup is
# encountered.  Each argument is a list.  If it is an empty list
# then no value was supplied for that parameter.  Note that a
# list with one element which is the empty string is not the
# same as an empty list!  Nonetheless, when we wish them to be
# equivalent, we'll take advantage of the fact that [lindex {{}} 0]
# is the same as [lindex {} 0].

proc phxW3HtBeginForm {action enctype method} {
	phxW3FormsBeginForm $action $enctype $method
}
proc phxW3HtEndForm {} {
	phxW3FormsEndForm
}
proc phxW3HtInsertFormsInput {align checked maxlength name size src type value} {
	phxW3FormsInsertInput $align $checked $maxlength $name $size $src $type $value
}
proc phxW3HtBeginFormsSelect {multiple name} {
	phxW3FormsBeginSelect $multiple $name
}
proc phxW3HtEndFormsSelect {} {
	phxW3FormsEndSelect
}
proc phxW3HtInsertFormsOption {selected value} {
	phxW3FormsInsertOption $selected $value
}
proc phxW3HtBeginFormsTextarea {cols name rows} {
	phxW3FormsBeginTextarea $cols $name $rows
}
proc phxW3HtEndFormsTextarea {} {
	phxW3FormsEndTextarea
}
