################################################################
# Modifications beyond tkWWW 0.9 are Copyright 1993, 1994 by
# The University of Chicago Biological Sciences Division, Academic Computing
# Philip Stylianos
# Lee Newberg
################################################################


## menu.tcl: make menu out of list of node names
## ==============
## Copyright (C) 1992-1993
## Global Network Academy
## Macvicar Institue for Educational Software Development
##
## See the file COPYRIGHT for conditions

## Conventions:
## All procedures in this file begin with tkW3Menu

# tkW3MenuMakeMenus 
# Add items to the menu bar to allow direct access of
# some information
# The format for the input list is
# {{menuname1 title1} {{title1 address1} {title2 address2}}} 
#  {menuname2 title2} {{title1 address1}}}}
#
# returns a list of menus created

proc tkW3MenuMakeMenus {menubar list} {

    if {[winfo exists $menubar] == 0} {
	PhxApp_build_menubar $menubar
	wm protocol [winfo toplevel $menubar] WM_DELETE_WINDOW {
		tkW3FileQueryCloseWindow
	}
    }
    set menu_list ""
    foreach menu_item $list {
	set menu [lindex $menu_item 0]
	set menu_name $menubar.[lindex $menu 0]
	if [winfo exists $menu_name] {
		destroy $menu_name
	}
	set menu_title [lindex $menu 1]
	#set menu_underline [lindex $menu 2]
	set menu_underline -1
	set menu_command [lindex $menu 3]
	
	if {$menu_command == "right"} {
	    set position "right"
	} {
	    set position "left"
	}

	pack configure \
	    [menubutton $menu_name -text $menu_title -menu $menu_name.m] \
	    -in $menubar -side $position

	if {$menu_underline != ""} {
	    $menu_name configure -underline $menu_underline
	}

	if {"$menu_command" == "goto"} {
	    tkW3MenuMakeNodeMenu $menu_name.m [lindex $menu_item 1]
	} {
	    tkW3MenuMakeMenuPane $menu_name.m [lindex $menu_item 1]
	}
	lappend menu_list $menu_name
    }
    eval tk_menuBar $menubar $menu_list
    tk_bindForTraversal . 

    PhxApp_standardize_menubar $menubar
  return $menubar
}


# tkW3MenuMakeNodeMenu
# make a menu from a list of titles and addresses
# This is called by the bookmarks and the history utilities
# the format for the input list is {{title1 address1} {title2 address2} ...}

proc tkW3MenuMakeNodeMenu {menuname list} {
    # Destroy the menu if it exists
    catch {destroy $menuname}
    # Create a new one
    menu $menuname
    # Run through the list of titles and add menu items
    foreach item $list {
	if {$item == {}} then {
	    $menuname add separator
	} else {
	    set address [lindex $item 1]
	    if {"$address" == "cascade_goto"} {
		$menuname add cascade -label "[lindex $item 0] =>" \
		    -menu [tkW3MenuMakeNodeMenu $menuname.m \
			   [lindex $item 2]]
		
		if {[lindex $item 3] != ""} {
		    $menuname entryconfigure last \
			-underline [lindex $item 3]
		} else {
		    $menuname entryconfigure last \
			-underline -1
		}
	    } else {
		set command [list phxW3ModeRecordAndGoto [lindex $item 1]]
		set label [lindex $item 0]
		set menu_text_position [ lindex $item 3 ]
		if {$menu_text_position != "" && \
			$menu_text_position != 0} {
		    lappend command $menu_text_position
		    set label "$label  (Line $menu_text_position)"
		}
		$menuname add command -label $label \
		    -command $command

		if {[lindex $item 2] != ""} {
		    $menuname entryconfigure last \
			-underline [lindex $item 2]
		} else {
		    $menuname entryconfigure last \
			-underline -1
		}
	    }
	}
    }
	PhxApp_standardize_menuwidg $menuname
    return $menuname
}

# tkW3MenuAddToNodeMenu
# Add one entry into the node menu

proc tkW3MenuAddToNodeMenu {menuname e} {
    if {![winfo exists $menuname]} {
	menu $menuname
    }
    set label [lindex $e 0]
    set address [lindex $e 1]
    set scroll_pos [lindex $e 2]
    set command [list phxW3ModeRecordAndGoto $address $scroll_pos]
    if {$scroll_pos != 0} { append label "  (Line $scroll_pos)" }
    $menuname add command -label $label -command $command -underline -1
} 

proc phxW3MenuChangeNodeMenu {menu index entry} {
    set label [lindex $entry 0]
    set address [lindex $entry 1]
    set scroll_pos [lindex $entry 2]
    set command [list phxW3ModeRecordAndGoto $address $scroll_pos]
    if {$scroll_pos != 0} { append label "  (Line $scroll_pos)" }
    $menu entryconfigure $index -label $label -command $command -underline -1
}

proc tkW3MenuMakeMenuPane {w menu_list} {
    global tk_version
    menu $w
    set tk_version_list [split $tk_version "."]
foreach item $menu_list {
if {[llength $item] == 0} {
	   $w add separator
       } { 
	   if {"[lindex $item 1]" == "cascade_external"} {
	       set name [lindex $item 0]
	       if {[lindex $tk_version_list 0] <= 3 &&
		   [lindex $tk_version_list 1] <= 2} {
		       append name " =>"
		   }
	       $w add cascade -label $name -menu $w.[lindex $item 2]
	       menu $w.[lindex $item 2]
	   } {
	       if {"[lindex $item 1]" == "cascade"} {
		   set name [lindex $item 0]
		   if {[lindex $tk_version_list 0] <= 3 &&
		       [lindex $tk_version_list 1] <= 2} {
			   append name " =>"
		       }
		   $w add cascade -label $name \
		       -menu [tkW3MenuMakeMenuPane $w.m [lindex $item 2]]
	       } {
		   $w add command -label [lindex $item 0] \
		       -command [lindex $item 1]
		   if {[lindex $item 2] != ""} {
		       $w entryconfigure last -underline [lindex $item 2]
		   }
		   if {[lindex $item 3] != ""} {
		       $w entryconfigure last -accelerator [lindex $item 3]
		       foreach i {Text Blt_htext Label Button Frame Toplevel} {
			bind $i [lindex $item 3] [lindex $item 1] 
			}
		   } 
	       }
	   }
       }
}
return $w
}
