################################################################
# Modifications beyond tkWWW 0.9 are Copyright 1993, 1994 by
# The University of Chicago Biological Sciences Division, Academic Computing
# Philip Stylianos
# Lee Newberg
################################################################



proc phxW3ModeGet {} {
	return [string tolower [lindex [.menu.edit config -text] 4]]
}

# phxW3ModeConfigure
#
# Set's mode-dependent features about the widget that should
# be set from creation.
# Features that should be set only once we're not busy are
# set with phxW3ModeEditWidgets and phxW3ModeBrowseWidgets.
#
# The names don't explain the differences in functionality.  Fix this!!

proc phxW3ModeConfigure {widget} {

	if {[phxW3ModeGet] == "edit"} {
		$widget config -bg gray90
		catch {$widget config -background gray90}
		catch {$widget config -activebackground gray90}
		if {[phxW3WidgetClass $widget] == "password"} {
			$widget config -fg gray90
		}
	} else {
		$widget config -bg gray
		catch {$widget config -background gray}
		catch {$widget config -activebackground gray}
		if {[phxW3WidgetClass $widget] == "password"} {
			$widget config -fg gray
		}
	}
}

proc phxW3ModeToggle {{w .buttons.mode}} {

	global phxW3ModeTextState
	set left [phxW3SelectionGetLeftOfFocus]
	set right [phxW3SelectionGetRightOfFocus]
	# Top puts Web into browse mode; bottom into edit mode

	if {$phxW3ModeTextState == "normal"} {
		if {[phxW3SaveWarnAboutLostChanges]} return
		phxW3ModeBrowse $w
	} else {
		phxW3ModeEdit $w
	}
	phxW3SelectionSetFocus $left $right
}

proc phxW3ModeToggleHtml {} {
    global CurrentHtext
    if {[catch {pack newinfo .f.html}]} {
	phxW3EditGenerateHTML
	tkW3FontSetTextFonts .f.html
	.menu.file.m entryconfigure "View HTML" -label "Hide HTML"
	pack forget $CurrentHtext
	pack .f.html -side top -fill both -expand yes
	.vs config -command {phxW3WidgetYview .vs .f.html}
	grab .vs
	set left [phxW3SelectionGetLeftOfFocus]
	set right [phxW3SelectionGetRightOfFocus]
	bind .f.html <Any-KeyPress> \
"if \{\"%A\" != \"\"\} \{phxW3ModeToggleHtml
phxW3SelectionSetFocus $left $right
\}"
	phxW3SelectionSetFocus .f.html
    } else {
	grab release .vs
	.menu.file.m entryconfigure "Hide HTML" -label "View HTML"
	pack forget .f.html
	pack $CurrentHtext -in .f -side top -fill both -expand yes
    	.vs config -command {phxW3WidgetYview .vs [phxW3CurrentHtext]}
	tkW3OutputSetScrollPosition [$CurrentHtext yview]
    }
}

proc phxW3ModeToggleButtons {} {
    if {[catch {pack newinfo .buttons}]} {
	.menu.file.m entryconfigure "*Button Bar" -label "Hide Button Bar"
	pack configure .buttons -before .style -side left -fill y
    } else {
	.menu.file.m entryconfigure "*Button Bar" -label "Show Button Bar"
	pack forget .buttons
    }
}

proc phxW3ModeRefresh {{w .buttons.mode}} {
	
	# Top puts Web into edit mode; bottom into browse mode
	if {[regexp Browse [$w config -text]]} {
		phxW3ModeEdit $w
	} else {
		phxW3ModeBrowse $w
	}
}

# phxW3ModeRecursiveChange
# Change the widgets and their descendants.
# It is important that supplied widgets are supplied in the order
# they are packed!!  Also, they must be the last in the packing
# sequence of their parent(s)!!

proc phxW3ModeRecursiveChange {widgets color state cursor} {
    . config -bg $color

    # The hypertext widget has some redrawing problems, so we're going to
    # forget these widgets and remember them later.
    set packCommand ""
    foreach widget $widgets {
	append packCommand "pack $widget " [pack newinfo $widget] "\n"
	if {[lindex [$widget config -bg] 4] != $color} {
	    pack forget $widget
	}
	$widget configure -cursor $cursor
    }
	
    phxW3ModeRecursiveChangeInner $widgets $color $state

    eval $packCommand
}

proc phxW3ModeRecursiveChangeInner {widgets color state} {
    foreach widget $widgets {
	phxW3ModeRecursiveChangeInner \
	    [winfo children $widget] $color $state
	$widget config -bg $color
	if {[phxW3WidgetClass $widget] == "para"} {
	    $widget config -state $state
	}
    }
}

# phxW3ModeBrowse
# View in Browse mode.

proc phxW3ModeBrowseWidgets {} {
    phxW3ModeRecursiveChange .f gray disabled left_ptr
}

proc phxW3ModeBrowse {{w .buttons.mode}} {
	global phxsrcdir
	global phxW3ModeTextState

	$w config -bitmap @$phxsrcdir/mode1.icon

	phxW3ModeBrowseWidgets
	set phxW3ModeTextState disabled
	phxW3SaveCleanBit

	set addr [split [tkW3OutputGetAddress] ":"]
	set type [lindex $addr 0]
	set file [lindex $addr 1]

	.buttons.space1 config -state disabled
	.buttons.space2 config -state disabled
	.buttons.space3 config -state disabled
	.buttons.hot config -state normal
	.buttons.find config -state normal
		
	.menu.file.m entryconfig *New* -label "Create New Page"
	.menu.file.m entryconfig "Goto * Mode" -label "Goto Edit Mode"
	.menu.edit config -text Browse
	.menu.annotate config -state disabled
	.menu.format config -state disabled
	.menu.edit.m entryconfig "Insert Image*" -state disabled
	.menu.edit.m entryconfig Undo -state disabled
	.menu.edit.m entryconfig Redo -state disabled
	.menu.edit.m entryconfig Cut -state disabled
	.menu.edit.m entryconfig Clear -state disabled
	.menu.edit.m entryconfig "Select All" -state disabled
	.menu.edit.m entryconfig "Select None" -state disabled
	.menu.edit.m entryconfig Edit* -state disabled
	.menu.edit.m entryconfig Reauthorize -state normal
	.menu.hotlist.m entryconfig "Add Page to Hotlist" -state normal
	.menu.hotlist.m entryconfig "Delete from Hotlist..." -state normal

	.titles.title_entry config -state disabled -bg gray
}



### This procedure configures the Web to edit mode.

proc phxW3ModeEditWidgets {} {
    phxW3ModeRecursiveChange .f gray90 normal xterm
}

proc phxW3ModeEdit {{w .buttons.mode}} {

	global phxsrcdir
	global phxW3ModeTextState

	$w config -bitmap @$phxsrcdir/mode2.icon

	phxW3ModeEditWidgets
	set phxW3ModeTextState normal

	.buttons.space1 config -state disabled
	.buttons.space2 config -state disabled
	.buttons.space3 config -state disabled
	.buttons.hot config -state disabled
	.buttons.find config -state disabled
		
	.menu.file.m entryconfig *New* -label "New"
	.menu.file.m entryconfig "Goto * Mode" -label "Goto Browse Mode"
	.menu.edit config -text Edit
	.menu.annotate config -state disabled
	.menu.format config -state normal
	.menu.format.m entryconfig Preformatted* -state disabled
	.menu.edit.m entryconfig "Insert Image*" -state normal
	.menu.edit.m entryconfig Undo -state disabled
	.menu.edit.m entryconfig Redo -state disabled
	.menu.edit.m entryconfig Cut -state normal
	.menu.edit.m entryconfig Clear -state normal
	.menu.edit.m entryconfig "Select All" -state disabled
	.menu.edit.m entryconfig "Select None" -state disabled
	.menu.edit.m entryconfig Edit* -state normal
	.menu.edit.m entryconfig Reauthorize -state disabled
	.menu.hotlist.m entryconfig "Add Page to Hotlist" -state disabled
	.menu.hotlist.m entryconfig "Delete from Hotlist..." -state disabled

	.titles.title_entry config -state normal -bg gray90
}

proc phxW3ModeSingleClickAnchor {w ai} {

    global tkW3HtAnchors 

    if {[phxW3SelectionGet] == ""} {
	phxW3ModeRecordAndGoto $tkW3HtAnchors(href.$ai)
    }
}

proc phxW3ModeSingleClickIsmap {w ai x y} {
    # A modified version of tk_butUp
    global tk_priv
    if {$w == $tk_priv(buttonWindow)} {
	$w config -relief $tk_priv(relief)
	if {$w == $tk_priv(window) && \
	    [lindex [$w config -state] 4] != "disabled" &&
	    [phxW3SelectionGet] == ""} \
	{
	    global tkW3HtAnchors
	    phxW3ModeRecordAndGoto $tkW3HtAnchors(href.$ai)?$x,$y
	}
	set tk_priv(buttonWindow) ""
    }
}

proc phxW3ModeSingleClickFormsImage {w form name x y} {
    # A modified version of tk_butUp
    global tk_priv
    if {$w == $tk_priv(buttonWindow)} {
	$w config -relief $tk_priv(relief)
	if {$w == $tk_priv(window) && \
	    [lindex [$w config -state] 4] != "disabled" &&
	    [phxW3SelectionGet] == ""} \
	{
	    phxW3FormsSubmit $form [list $name.x $x] [list $name.y $y]
	}
	set tk_priv(buttonWindow) ""
    }
}

proc phxW3ModeGoto {href {scrollpos 0}} {

	if {$href != ""} {
		if {[phxW3ModeGet] == "browse"} {
			tkW3NavigateGoto $href $scrollpos
		}
		phxW3AnchorBroadcastHref $href 
	} else {
		# A request to go nowhere will get you a new page...
		tkW3EditCreateNewPage [phxW3EditCreateNewPageName]
	}
}		

proc phxW3ModeRecordAndGoto {href {scroll 0}} {

	if {$href != ""} {
		if {[phxW3ModeGet] == "browse"} {
			tkW3NavigateRecordAndGoto $href $scroll
		}
		phxW3AnchorBroadcastHref $href
	} else {
		# A request to go nowhere will get you a new page...
		tkW3EditCreateNewPage [phxW3EditCreateNewPageName]
	}
}		

proc phxW3ModeDoubleClickAnchor {w ai} {

    global tkW3HtAnchors 
	
	if {[phxW3ModeGet] == "browse"} {
		tkW3NavigateClone $tkW3HtAnchors(href.$ai)
		set h $tkW3HtAnchors(href.$ai)
	} else {
		phxW3SelectionClear
		phxW3SelectionSet $w
		phxW3AnchorDialog
		set h [phxW3AnchorCurrentHref]
	}
	phxW3AnchorBroadcastHref $h
}
