################################################################
# Modifications beyond tkWWW 0.9 are Copyright 1993, 1994 by
# The University of Chicago Biological Sciences Division, Academic Computing
# Philip Stylianos
# Lee Newberg
# Tim Pierce
################################################################

## navigate.tcl Procedures for navigation
## ==============
## Copyright (C) 1992-1993
## Global Network Academy
## Macvicar Institute for Educational Software Development
##
## See the file COPYRIGHT for conditions

## The procedures in this file are responsible for navigation between 
## different hypertext pages on World Wide Web.  They are called by the
## objects in other Phoenix files in response to a user's request to 
## display a WWW page.

## Conventions:
##   All procedures in this file begin with tkW3Navigate

proc tkW3NavigateBacktrack {} {
    if {[set backpage [tkW3HistoryPop]] !={}} {
	phxW3HistoryRecord [tkW3OutputGetTitle] [tkW3OutputGetAddress] \
	    [tkW3OutputGetScrollPosition]
	phxW3ModeGoto [lindex $backpage 1] [lindex $backpage 2]
    } else {
	tkW3OutputError "Error: No pages previous to this"
    }
}

proc tkW3NavigateClone {{page ""} {anchor ""} {ismap 0}} {
    global tkW3ConfigHomePage

    if {$page == ""} {set page [tkW3OutputGetAddress]} \
    else {
	set page [tkW3NavigateServerSend \
	    [list HtParseName $page [tkW3OutputGetAddress]]]
    }

    tkW3OutputSetMessage "Opening New Window ..." 5000
    global phxbindir
    set command "exec $phxbindir/phxclone -wwwstart $page -wwwhome $tkW3ConfigHomePage"
    if {$anchor != ""} {
	lappend command "-wwwimageanc" $anchor
	if {$ismap != 0} {
	    lappend command "-wwwimageismap"
	}
    }
    lappend command "&"
    eval $command
}

		# This closes this tkW3 window
proc tkW3NavigateCloseThisWindow {} {
  tkW3NavigateServerClose
		# Necessary to clear color map
  tkW3ImageReleaseColorCells
		# Necessary to clear name from interpreter table
  destroy .
  catch "exit"
}

# This closes all Phoenix windows 
proc tkW3NavigateCloseAllWindows {} {
    phxW3NavigateSendToTkWWW "tkW3NavigateCloseThisWindow"
    tkW3NavigateCloseThisWindow
}

# phxW3NavigateSetToTkWWW sends the specified command to all
# other tkWWW processes that are running.

proc phxW3NavigateSendToTkWWW {command} {
    set self [winfo name .]
    foreach interp [winfo interps] {
	if {[regexp {^WWW Editor} $interp] && $interp != $self} {
	    catch {send $interp [list after 1 $command]}
	}
    }
}

# phxW3NavigateSetToOneTkWWW sends the specified command to another
# tkWWW process that is running and returns the response.

proc phxW3NavigateSendToOneTkWWW {command} {
    set return ""
    set self [winfo name .]
    foreach interp [winfo interps] {
	if {[regexp {^WWW Editor} $interp] && $interp != $self} {
	    catch {set return [send $interp [list $command]]}
	    break
	}
    }
    return $return
}

## tkW3NavigateFind: Does keyword search off current page
## ----------
## Arguments:
##   key - a space separated list of keywords
## No return value
##
## This procedure performs a keyword search off the currently displayed
## page.  It works by converting the keywords and current page into a
## World Wide Web address, and then calls the procedure tkW3NavigateGoto
## and goes to that page.
##

proc tkW3NavigateFind {keys} {
    set encoded ""
    foreach key $keys { append encoded [phxW3NavigateEncodeURL $key] "+" }
    set encoded [string trim $encoded "+"]
    set address [string trim [tkW3OutputGetAddress] " \n?"]
    set address [lindex [split $address "?"] 0]
    append address "?" $encoded
    phxW3ModeRecordAndGoto $address
}

proc tkW3NavigateGoto {page scroll_position} {
    global tkW3HtNameToHnumber
    global tkW3HtAnchors

    # Change the cursor to a busy signal and tell user we are getting file
    tkW3OutputCursorWait
    
    .anchor configure -command [list phxW3AnchorBroadcastHref $page]
    set page_name [tkW3NavigateServerSend \
	[list HtParseName $page [tkW3OutputGetAddress]]]

    if {![string match "#*" $page]} {
	switch -glob $page_name {
	    {telnet:*} {-}
	    {tn3270:*} {-}
	    {rlogin:*} {
		tkW3TelnetPage $page_name
		set retval 0
	    }
	    {exec:*} {
		# Find the command
		if [regexp {^exec:(.*)} $page_name {} command] {
		    regsub -all %20 $command " " command

		    tkW3OutputSetAddress $page_name {}
		    
		    tkW3NavigatePreface "System Call
Execute system command \"$command\"
" {Push here to execute System Call} "Executing System Call \"$command\"..." \
"exec $command &"
			set retval 0
		} else {
			tkW3OutputSetMessage "No command to execute"
			set retval 1
		}
	    }
	    default {
		# if the file type is file, http, gopher, or wais we need 
		# to load the page in from the server
		set retval [phxW3CacheLoadPageFromServer $page_name]
	    }
	} 
	if {$retval == 0} { tkW3OutputSetScrollPosition $scroll_position }
    } else {
	set page_name $page
	set retval 0
    }

    if {$retval == 0} {
	# Goto to the anchor
	set page_tag ""
	regexp {#(.*)$} $page_name {} page_tag
	if {$page_tag != ""} {
	    set htext [phxW3CurrentHtext]
	    set number $tkW3HtNameToHnumber($htext,$page_tag)
	    if {$number != ""} {

		# Calculate the location of the tag and scroll to it.
		# w: the widget which owns anchor "number".
		# top: the y-coordinate of the top of $w, relative to
		#      the current htext.
		# offset: the pixel offset of the destination anchor,
		#      relative to the destination widget.

		set w $tkW3HtAnchors(widg.$number)
		set top [winfo y $w]
		set offset ""
		set offset [lindex [phxW3EditFindPlace $w h$number.first] 1]
		set yscrollunits [lindex [$htext configure -yscrollunits] 4]
		$htext yview [expr "($top + $offset) / $yscrollunits - 2"]

		# The reason for -2 is that phxW3EditFindPlace returns
		# the coordinates for the bottom portion of the specified
		# tag, and this seems to correct it.
	    }
	}
	# Display
	tkW3ConfigDisplay 
    }
    phxW3HistoryInsensitize [tkW3OutputGetTitle] [tkW3OutputGetAddress] \
				[tkW3OutputGetScrollPosition] 

    # Clear the message and set the cursor back to normal
    tkW3OutputSetMessage {}
    tkW3OutputCursorNormal
}

proc phxW3NavigateMethodHTTP {{method {}}} {
    global NavigateMethodHTTP
    if {$method == {}} {
	return $NavigateMethodHTTP
    } else {
	set NavigateMethodHTTP $method
    }
}

# tkW3NavigateLoadPageFromServer

# page_name is an absolute URL.

proc tkW3NavigateLoadPageFromServer {page_name} {
    # Assume success
    set return 0

    # Chop off the anchor
    regexp {^[^#]*} $page_name page_value

    # Set the correct method for retrieval
    tkW3NavigateServerSend [list HtMethodHTTP [phxW3NavigateMethodHTTP]]

    # Remember the current page in case we run into trouble.
    set pageCurrent \
	[list [tkW3OutputGetAddress] [tkW3OutputGetScrollPosition]]

    # Fetch the page and get the returned commands (or error message).
    global phxW3NavigateAuthorizationFailures
    set phxW3NavigateAuthorizationFailures 0
    set return_value [phxW3NavigateFetchCommands $page_value]

    # Restore the default method.  (Be sure this happens before
    # we GET inline images!)
    tkW3NavigateServerSend "HtMethodHTTP GET"

    # If the return code still contains an "Error", signal an error
    if {[regexp {^Error}  $return_value]} {
	# Ooops there's an error
	if {[regexp {Unable to acces} $return_value] &&
	    [regexp {^file:} $page_name]} {
		set return_value "Could not open\n$page_value"
	} elseif {[regexp {HTTP/[0-9]+(\.[0-9]+)* 401} $return_value]} {
	    set return_value "Insufficient authorization for\n$page_value"
	}
	tkW3NavigateBackOutFromLoad $return_value $pageCurrent $page_value
	set return 1
    } else { 
	# No error from backend, flip to new page
	# What we should get back from the backend are
	# commands to generate the hypertext.
	tkW3OutputSetMessage "Formatting page ..."
	set return_value [phxW3NavigateNullifySpecialChars $return_value]

	global errorInfo
	set catch_val [catch {eval $return_value} errmsg]
	set savedInfo $errorInfo

	if {$catch_val} {
	    tkW3NavigateBackOutFromLoad "$errmsg, errorInfo = $savedInfo" \
		$pageCurrent $page_value
	    set return 1
	}
    }

    # We've got the page.  Turn off authentication.
    tkW3NavigateServerSend {HtAuthorization Basic "" ""}
    return $return
}

# phxW3NavigateFetchCommands takes an abosolute URL and fetches
# the commands to display the data returned by the data server.
# This procedure handles recoverable errors from the data
# server.

proc phxW3NavigateFetchCommands {url} {
    global phxW3NavigateAuthorizationFailures

    set return_value [tkW3NavigateServerSend [list HtLoad $url]]
    set return_list [split [string trim $return_value] \n]

    if {[lsearch -regexp $return_list {^Error}] == 0} {
	set HTTP_error [lindex [lindex $return_list 1] 1]
	switch $HTTP_error {
	301 -
	302 {
	    set index [lsearch -regexp $return_list {^Location: }]
	    if {$index != -1} {
		set url [lindex [lindex $return_list $index] 1]
		set phxW3NavigateAuthorizationFailures 0
		# Check if the page is cached before re-fetching it!!
		return [phxW3NavigateFetchCommands $url]
	    }
	  }
	401 {
	    incr phxW3NavigateAuthorizationFailures
	    if {($phxW3NavigateAuthorizationFailures == 1 && \
		 [set auth [phxW3AuthorizationGet $url]] != "") || \
		 [set auth [phxW3AuthorizationSet $url]] != ""} \
	    {
		tkW3NavigateServerSend $auth
		return [phxW3NavigateFetchCommands $url]
	    }
	  }
	}
    }

    # We were either sucessful, or we're giving up.
    return $return_value
}

# tkW3NavigateBackOutFromLoad.
# While viewing $page (= [list $url $scrollposition]) we failed
# to load $newpage (= $url).  We wish to go back to $page or
# a substitute.

proc tkW3NavigateBackOutFromLoad {msg page newpage} {
    if {[lindex $page 0] == $newpage} {
	# We failed reloading the current page.  Try the next most
	# recently viewed page.
	set history [tkW3HistoryPop]
	set page [lrange $history 1 2]
	tkW3OutputSetAddress [lindex $history 1] [lindex $history 0]
    }

    if {[lindex $page 0] == ""} {
	# We died trying to the home page.  
	global tkW3ConfigHomePage
	# Mark the current home page as no good.	
	phxW3ConfigHomePageInitialize $tkW3ConfigHomePage
	# Try with the new homepage.
	set page [list $tkW3ConfigHomePage 0]
    } else {
	# Tell the user about the error
	tkW3OutputError $msg
    }
    eval [linsert $page 0 tkW3NavigateGoto]
}

proc phxW3NavigateNullifySpecialChars {c} {
	regsub -all (\[^\\\])\\\$ $c \\1\\\$ c
	regsub ^\\\$ $c \\\$ c
#	regsub -all (\[^\\\])\\\[ $c \\1\\\[ c
#	regsub ^\\\$ $c \\\$ c
#	regsub -all (\[^\\\])\\\] $c \\1\\\] c
#	regsub ^\\\$ $c \\\$ c
#	regsub -all (\[^\\\])\" $c \\1\\\" c
#	regsub ^\\\" $c \\\" c

	return $c
}

proc tkW3NavigatePreface {body_text button_text message command} {
    tkW3OutputClearBody
    tkW3HtInsert "$body_text"
    tkW3HtInsert "$button_text" action
    set htext [phxW3CurrentHtext]
    $htext tag configure action -relief raised -borderwidth 2 -background \
	[lindex [$htext configure -background] 4]

    $htext tag bind action <ButtonPress> "%W tag configure action \
-relief sunken"
    $htext tag bind action <ButtonRelease-1> \
	"%W tag configure action -relief raised
tkW3OutputSetMessage {$message} 5000
$command"
}
    

# Warning: tkW3NavigateSaveText and tkW3EditOutputHTML are nearly
# identical.  They probably should be merged!!

proc tkW3NavigateSaveText {filename} {
   # Generate the HTML in .f.html
   tkW3EditGenerateHTML
   # Create a file/pipe to write to.
   set file [open $filename w]
   # Write to the file
   puts $file [.f.html get 1.0 end]
   # Catch the "close" because a pipe may try to return an error.
   catch {close $file}
} 

proc tkW3NavigatePrintUsingCommand {command} {
    after 250 [list tkW3NavigateSaveText "| $command > /dev/null"]
}

proc tkW3NavigateMailText {to subject} {
    phxW3EditGenerateHTML
    tkW3ConfigSendMail $to $subject [.f.html get 1.0 end]
}

# Record current address on the history list and go to the requested page

proc tkW3NavigateRecordAndGoto {page scroll} {

    phxW3HistoryRecordAndPush \
	[tkW3OutputGetTitle] [tkW3OutputGetAddress] \
	[tkW3OutputGetScrollPosition]
    tkW3NavigateGoto $page $scroll
}
    
proc tkW3NavigateReload {} {
    set address [tkW3OutputGetAddress]
    phxW3CacheUncache $address
    set mode [phxW3ModeGet]
    tkW3NavigateGoto $address [tkW3OutputGetScrollPosition]
    # tkW3NavigateGoto automatically puts us in browse mode.
    # Correct that if necessary.
    if {$mode == "edit"} { phxW3ModeToggle }
}

proc tkW3NavigateInitialize {} {
    global tkW3ConfigFileSuffixes
    foreach i $tkW3ConfigFileSuffixes {
	tkW3NavigateServerSend [concat HtSetSuffix $i]
    }

    global tkW3NavigateIsIndex
    set tkW3NavigateIsIndex 0

    global NavigateMethodHTTP
    set NavigateMethodHTTP GET

    global phxW3NavigateAuthorizationFailures
    set phxW3NavigateAuthorizationFailures 0
}

proc tkW3NavigateStartupServer {} {
    global tkW3NavigateServerInfo tkW3ServerCommand
    set tkW3NavigateServerInfo \
	[tkW3ServerOpen $tkW3ServerCommand "phoenix:"]
}

proc tkW3NavigateServerSend {w} {
    global tkW3NavigateServerInfo
    
    tkW3ServerSend $tkW3NavigateServerInfo $w 
}

proc tkW3NavigateServerClose {} {
    global tkW3NavigateServerInfo
    tkW3ServerClose $tkW3NavigateServerInfo
}

# phxW3NavigateEscapeURL escapes characters which are
# significant within a URL.  For total safety, anything that is
# not alphanumeric is escaped.  An escape sequence is a "%"
# character followed by the ASCII value in hexadecimal.

proc phxW3NavigateEncodeURL {decode} {
    set encode ""
    foreach char [split $decode {}] {
	if {$char >= "A" && $char <= "Z" || \
	    $char >= "a" && $char <= "z" || \
	    $char >= "0" && $char <= "9"} \
	{
	    append encode $char
	} else {
	    scan $char "%c" decimal
	    append encode "%" [format "%02X" $decimal]
	}
    }
    return $encode
}

# phxW3NavigateDecodeURL finds occurrences of "%XX" (where each
# X is a hexadecimal digit) and replaces each with the ASCII
# character for XX.  It is an error to have a "%" followed by
# other and two hexadecimal digits.

proc phxW3NavigateDecodeURL {encode} {
    set decode ""
    set length [string length $encode]
    for {set i 0} {$i < $length} {incr i} {
	set c [string index $encode $i]
	if {$c != "%"} {
	    append decode $c
	} else {
	    scan [string range $encode [expr {$i + 1}] [expr {$i + 2}]] "%x" \
		decimal
	    append decode [format "%c" $decimal]
	    incr i 2
	}
    }
    return $decode
}
