################################################################################
#
#  Menubar functions
#
################################################################################


proc PhxApp_build_menubar {mbframe} {

	global phxcolor phxpixmap phxfont
	global currentwindowid

	if {[winfo exists $mbframe] == 0} {
		frame $mbframe  -background $phxcolor(menubg) -borderwidth 0
	}

	set window [winfo toplevel $mbframe]
	wm protocol $window WM_DELETE_WINDOW {exit}

	if { [info procs PhxApp_name_window] != "" } {
		set windowname [PhxApp_name_window]
		if {$windowname == ""} {
			set windowname "Untitled"
		}
	} else {
		set windowname "[winfo name $window] window"
	}

	PhxApp_add_phoenixmenu_tomenubar $mbframe
	PhxApp_add_helpmenu_tomenubar $mbframe

	pack append [winfo parent $mbframe] $mbframe {top frame center fillx}


	if [PhxApp_get_windowlock] {

		PhxApp_send_mgrcommand PhxMgr_add_window [winfo name .] $mbframe $windowname [set currentwindowid [winfo id $window]]

		PhxApp_release_windowlock
	}
}

proc PhxApp_add_phoenixmenu_tomenubar {mbframe} {

	global phxcolor phxfont phxpixmap


	############
	# Create menu button phoenix and pack
	############

	if { [winfo exists $mbframe.phoenix] == 0 } {
		menubutton $mbframe.phoenix  \
			-menu $mbframe.phoenix.menu  \
			-bitmap $phxpixmap(phoenixbutton)
	}

	pack append $mbframe  $mbframe.phoenix {right frame center}
}


proc PhxApp_add_helpmenu_tomenubar {mbframe} {

	global phxcolor phxfont phxpixmap phxbindir

	############
	# Create menu button help
	############

	menubutton $mbframe.help  \
		-menu "$mbframe.help.menu"  \
		-bitmap $phxpixmap(helpbutton)


	############
	# Create menu help.menu
	############

	menu $mbframe.help.menu

#	foreach i [PhxApp_send_mgrcommand PhxMgr_get_appmenupaths] {
#		$mbframe.help.menu add command \
#			-font $phxfont(menuitemlabel) \
#			-label $i 
#	}
 
        $mbframe.help.menu add command \
                -font $phxfont(menuitemlabel) \
                -label Phoenix \
                -command "exec echo \"setenv WWW_HOME http://www.bsd.uchicago.edu/Help/help.html \; $phxbindir/phxclone\" | csh -tf &"

	$mbframe.help.menu add separator
        $mbframe.help.menu add command \
                -font $phxfont(menuitemlabel) \
                -label "Web-Editor" \
                -command "exec echo \"setenv WWW_HOME http://www.bsd.uchicago.edu/Help/WWW/Web11.html \; $phxbindir/phxclone\" | csh -tf &"

	$mbframe.help.menu add separator
        $mbframe.help.menu add command \
                -font $phxfont(menuitemlabel) \
                -label Mail \
                -command "exec echo \"setenv WWW_HOME http://www.bsd.uchicago.edu/Help/Mail/mail.html \; $phxbindir/phxclone\" | csh -tf &"

	$mbframe.help.menu add separator
        $mbframe.help.menu add command \
                -font $phxfont(menuitemlabel) \
                -label Ovid/Medline \
                -command "exec echo \"setenv WWW_HOME http://www.lib.uchicago.edu/keith/ovid/toc.html \; $phxbindir/phxclone\" | csh -tf &"

	# Pack button into menubar
	pack append $mbframe  $mbframe.help {right frame center}

	# Make sure it packs to the top in the app
	pack append [winfo parent $mbframe]  $mbframe {top frame center fillx}

	# Menubar - it
	tk_menuBar $mbframe $mbframe.help $mbframe.phoenix
}




proc PhxApp_standardize_menuwidg { mw } {

	global phxcolor phxfont

	case [winfo class $mw] in {
		{Button Menubutton Menu} { 
			$mw config \
				-foreground $phxcolor(fg) \
				-background $phxcolor(menubg) \
				-activebackground $phxcolor(menuactbg) \
				-activeforeground $phxcolor(menuactfg) \
				-font $phxfont(buttonlabel)
		}
		{Label} {
			$mw config \
				-foreground $phxcolor(fg)\
				-background $phxcolor(menubg)
		}
		{Frame} {
			$mw config \
				-background $phxcolor(menubg)
		}
	}
}

proc PhxApp_standardize_menubar { mf } {

	foreach i [PhxUtil_get_descendants $mf] {
		PhxApp_standardize_menuwidg $i
	}
	PhxApp_standardize_menuwidg $mf

	set leftl ""
	set rightl ""
	set widg ""
	foreach mb [pack info $mf] {

		if {$widg == ""} {
			set widg $mb
			continue
		}

		set side [lindex $mb 0] 
		case $side in {
			{left top} {
				lappend leftl $widg
			}
			{right bottom} {
				lappend rightl $widg
			}
		}
		set widg ""
	}

	set mblist $leftl
	for {set i [expr [llength $rightl]-1]} {$i >= 0} {incr i -1} {
		lappend mblist [lindex $rightl $i]
	}

	tk_menuBar $mf 
	eval tk_menuBar $mf $mblist
	tk_bindForTraversal .
}

proc PhxApp_add_window_tomenubar { menu appnamepath windowname } {

	set menu $menu.phoenix.menu

	set menu [PhxApp_get_menu $menu $appnamepath]

	set appname [lindex $appnamepath [expr [llength $appnamepath]-1]]

	PhxApp_add_windowmenu $menu $appname $windowname
}

proc PhxApp_get_menu {me anp} {

	#if {[llength $anp] == 1} {
		#set anp [lindex $anp 0]
	#}
	foreach j [lrange $anp 0 [expr [llength $anp]-2]] {
		set me $me.[string tolower $j]
	}

	return $me
}

proc PhxApp_change_password {} {

	global phxsrcdir

	exec xterm -T "Changing Phoenix Password" -e $phxsrcdir/chpwd.csh

}

proc PhxApp_delete_window_frommenubar { menubar appnamepath windowname } {

	#set menubar [join $menubar]
	#set appnamepath [join $appnamepath]
	#set windowname [join $windowname]
	set menu $menubar.phoenix.menu
	set menu [PhxApp_get_menu $menu $appnamepath]

	set appname [lindex $appnamepath [expr [llength $appnamepath]-1]]

	set apptitle [lindex $appnamepath [expr [llength $appnamepath]-1]]

	set menuw [string tolower [PhxUtil_replace_spaces_withdashes $apptitle]]
	set menuw $menu.${menuw}windows
	$menuw delete "[join "$windowname"]"

	if {[$menuw index 0] == "none"} {
		PhxApp_refresh_phoenixmenu $menubar
	}

}
	

proc PhxApp_refresh_phoenixmenu { mbframe } {
	global phxcolor phxfont phxpixmap

	# Create menu phoenix.menu
	if { [winfo exists $mbframe.phoenix.menu] } {
		destroy $mbframe.phoenix.menu
	}
	menu $mbframe.phoenix.menu

	# Add every application registered with PhxMgr to phoenix.menu
	foreach i [PhxApp_send_mgrcommand PhxMgr_get_appmenupaths] {

		# Initialize menu path
		set mpath $mbframe.phoenix.menu

		# Iterate for each menu level of the app
		set count [llength $i]
		foreach uj $i {

			set j [string tolower $uj]
			incr count -1

			# If this is the last level, create command menu entry
			if { $count == 0 } {
				set windowlist [PhxApp_send_mgrcommand \
						PhxMgr_get_windows $i]
				if {$windowlist == ""} {
					PhxApp_add_appmenubutton $mpath $i 
				} else {
					foreach w $windowlist {
						PhxApp_add_windowmenu $mpath $i $w
					}
				}
			} else {
			# Else create cascade menu if not already existing
				if {[winfo exists $mpath.$j] == 0} {
					menu $mpath.$j
					$mpath add cascade \
						-label "$uj..." \
						-menu $mpath.$j \
						-font $phxfont(menuitemlabel)
				}
				set mpath $mpath.$j
			}
		}
	}

	$mbframe.phoenix.menu add separator

if 0 {
	catch {
	menu $mbframe.phoenix.menu.configure
	$mbframe.phoenix.menu add cascade \
		-label "Configure . . ." \
		-font $phxfont(menuitemlabel) \
		-menu $mbframe.phoenix.menu.configure
	}

	catch {
	$mbframe.phoenix.menu.configure add command \
		-label "Change Password" \
		-font $phxfont(menuitemlabel) \
		-command PhxApp_change_password
	}
}

	catch {
	$mbframe.phoenix.menu add command \
		-label "Exit Phoenix" \
		-font $phxfont(menuitemlabel) \
		-command "PhxApp_send_mgrcommand PhxMgr_exit"
	}

	foreach i [PhxUtil_get_descendants $mbframe.phoenix.menu] {
		PhxApp_standardize_menuwidg $i
	}
	PhxApp_standardize_menuwidg $mbframe.phoenix.menu

	
}


proc PhxApp_add_appmenubutton {menu appnamepath} {

	global phxcolor phxfont phxpixmap

	#set apptitle [lindex $appnamepath 0] 
	set apptitle $appnamepath
	set apptitle [lindex $apptitle [expr [llength $apptitle]-1]]

	set menuw [string tolower [PhxUtil_replace_spaces_withdashes $apptitle]]
	set menuw $menu.${menuw}windows

	if {[info exists $menuw]} {
		destroy $menuw
	}

	if {[catch {$menu entryconfig $apptitle}] == 0} {
		$menu delete $apptitle
	}

	$menu add command \
		-label $apptitle \
		-font $phxfont(menuitemlabel) \
		-command [list PhxApp_send_mgrcommand PhxMgr_run_me $appnamepath]
}




proc PhxApp_add_windowmenu {menu appnamepath windowname} {
	global phxcolor phxfont phxpixmap

	#set apptitle [lindex $appnamepath 0] 
	set apptitle $appnamepath
	set apptitle [lindex $apptitle [expr [llength $apptitle]-1]]

	set menuw [string tolower [PhxUtil_replace_spaces_withdashes $apptitle]]
	set menuw $menu.${menuw}windows


	if {[catch {$menu entryconfig $apptitle -menu}] == 1} {
		if {[catch {$menu entryconfig $apptitle}] == 0} {
			$menu delete $apptitle 
		}
		$menu add cascade \
			-label $apptitle \
			-background $phxcolor(runappbg)  \
			-activebackground  $phxcolor(runappbg) \
			-font $phxfont(runningapplabel) \
			-menu $menuw
	
		if {[winfo exists $menuw]} {
			destroy $menuw
		}	
		menu $menuw
	}

	$menuw add command \
		-label $windowname \
		-font $phxfont(menuitemlabel) \
		-background $phxcolor(menubg) \
		-activebackground $phxcolor(menubg) \
		-command "PhxApp_send_mgrcommand PhxMgr_raise_window \"$windowname\""
}







################################################################################
#
#  Buttonbar functions
#
################################################################################


proc PhxApp_build_buttonbar {b} {

	global phxcolor

	if {[winfo exists $b] == 0} {
		frame $b -background $phxcolor(bg)
	}

	button $b.quit -text Quit -command {exit}
	pack append $b $b.quit {right frame s}
	pack append [winfo parent $b] $b {bottom frame center fillx}


}


proc PhxApp_standardize_buttonwidg { mw } {

	global phxcolor phxfont

	case [winfo class $mw] in {
		{Button Menubutton} { 
			  $mw config \
				-foreground $phxcolor(fg) \
				-background $phxcolor(bg) \
				-activeforeground $phxcolor(buttonactfg) \
				-activebackground $phxcolor(buttonactbg) \
				-font $phxfont(buttonlabel)  ; # pk 940325: see notes by set phxcolor
		}

		{Label} {
			$mw config \
				-foreground $phxcolor(fg)\
				-background $phxcolor(bg)
		}
		{Frame} {
			$mw config -background $phxcolor(bg)
		}
	}
}

proc PhxApp_standardize_buttonbar {bf} {

	foreach i [PhxUtil_get_descendants $bf] {
		PhxApp_standardize_buttonwidg $i
	}
	PhxApp_standardize_buttonwidg $bf
}




################################################################################
#
#  Window functions
#
################################################################################

proc PhxApp_standardize_window {w} {

	foreach ww [PhxUtil_get_descendants $w]  {
		PhxApp_standardize_windowwidg $ww
	}
	PhxApp_standardize_windowwidg $w
}

proc PhxApp_standardize_windowwidg {ww} {

	global phxcolor phxfont

	set menubarlist [PhxApp_send_mgrcommand PhxMgr_get_menubars [winfo name .]]

	case [winfo class $ww] in {
		{Frame} {
			if {[lsearch $menubarlist $ww] == -1} {
				$ww config -background $phxcolor(bg)
			}
		}
		{Hypertext Listbox Photo Hypertext Text} {
			$ww config -background $phxcolor(bg)
			$ww config -foreground $phxcolor(fg)
		}
		{Label} {
			$ww config -background $phxcolor(bg)
			$ww config -foreground $phxcolor(fg)
		}
		{Scrollbar} {
			$ww config -background $phxcolor(bg)
			$ww config -foreground $phxcolor(bg)
			#$ww config -activebackground $phxcolor(bg)
			$ww config -activeforeground $phxcolor(bg)
		}
		{Button Checkbutton Radiobutton} {
			$ww config -background $phxcolor(bg)
			$ww config -foreground $phxcolor(fg)
			$ww config -activebackground $phxcolor(bg)
			$ww config -activeforeground $phxcolor(fg)
		}
	}
}

################################################################################
#
# Phoenix Manager Communication
#
################################################################################

proc PhxApp_send_mgrcommand {args} {

	global phxmgrname

	set result ""

	if {[lsearch [winfo interps] $phxmgrname] >= 0 && \
	    [catch {set result [send $phxmgrname $args]} errmsg] == 0} {
		return $result
	} 
	

	for {set i 18} \
	    {[expr $i] && \
		[info exists errmsg] && \
		[regexp "did not respond" $errmsg]} \
	    { incr i -1} {
	    	if {[catch {set result [send $phxmgrname $args]} errmsg] == 0} {
			return $result
		} 
	}
			
	global errorInfo
	puts "This command sent to the PhxMgr caused an error:$args"
	puts $errorInfo

	PhxApp_connectto_phxmgr
	if {[lsearch [winfo interps] $phxmgrname] >= 0 && \
	    [catch {set result [send $phxmgrname $args]}] == 0} {
		return $result
	}

}

proc PhxApp_connectto_phxmgr {} {

	global phxsrcdir

	if { [PhxApp_find_phxmgr] != "" } {
		return
	}

	puts ""
	puts "Nonstandard method of running application"
	puts "Please run the comand \"phoenix\" and invoke applications"
	puts "through the Phoenix menu"

	destroy .
	exit
}

proc PhxApp_find_phxmgr {} {

	global phxmgrname

	foreach i [winfo interps] {
		if [regexp Phoenix* $i] {
			if {[catch {send $i {list ""}} mesg]} {
				#global errorInfo
				#set goop $errorInfo
				#error $mesg $goop
			} {
				set phxmgrname $i
				return $i
			}
		}
	}
	
	return ""
}


proc PhxApp_get_windowlock {} {
	global windowlock_tryagain


	while 1 {

		case [PhxApp_send_mgrcommand PhxMgr_requesting_windowlock \
			[winfo name .]] {
			0 	{tkwait variable windowlock_tryagain}
			1 	{ return  1}
			-1	{ return 0}
		}
	}	
}

proc PhxApp_release_windowlock {} {


	PhxApp_send_mgrcommand PhxMgr_releasing_windowlock [winfo name .]

}


################################################################################
#
#  Startup/shutdown Application functions
#
################################################################################



proc PhxApp_init {} {

	global phxsrcdir

	#wm protocol . WM_DELETE_WINDOW {exit}
	signal trap {SIGINT SIGTERM} {exit}

	global windowlock_tryagain
	set windowlock_tryagain 0

	global phxmgrname
	set phxmgrname "phoenix"

	global phxcolor
	set phxcolor(bg) gray
	set phxcolor(fg) black
	set phxcolor(databg) gray85
	set phxcolor(activefg) gray
	set phxcolor(activebg) gray
	set phxcolor(menubg) white 
	set phxcolor(menuactfg) black
	set phxcolor(menuactbg) white
	set phxcolor(buttonactfg) blue
	set phxcolor(buttonactbg) $phxcolor(activebg) 	; # as originally set
	set phxcolor(runappbg) white

	global phxpixmap
	#set phxpixmap(phoenixbutton) @$phxsrcdir/Phoenix.xbm
	set phxpixmap(phoenixbutton) @$phxsrcdir/phoenix.icon
	set phxpixmap(helpbutton) @$phxsrcdir/help.icon

	global phxfont
	set phxfont(runningapplabel) "-adobe-helvetica-bold-r-normal--10-100-75-75-p-60-iso8859-1"
	set phxfont(menuitemlabel) "-adobe-helvetica-bold-r-normal--10-100-75-75-p-60-iso8859-1"
	set phxfont(buttonlabel) "-adobe-helvetica-bold-r-normal--10-100-75-75-p-60-iso8859-1"
	set phxfont(bigbuttonlabel) "-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1"

	global phxwidgettypes
	set phxwidgettypes {Menubar Buttonbar Window}

	global currentwindowid
	set currentwindowname 0x0
}

# All exits in the application must be converted to exit

rename exit PhxApp_exit
proc exit {{status 0}} {

	global destroyyourself currentwindowid

	PhxApp_change_status destroyyourself 0 
	if [PhxApp_get_windowlock] {
		PhxApp_send_mgrcommand PhxMgr_delete_window $currentwindowid
		PhxApp_release_windowlock
	}
	if {$destroyyourself == 0} {
		tkwait variable destroyyourself 
	}

	PhxApp_kill $status

}

proc PhxApp_kill {{status 0}} {

	#after 2000 destroy .
	after 2000 PhxApp_exit $status

	return
}

proc PhxApp_change_status {var val} {
	global $var
	set $var $val
}


proc PhxApp_rename_window {new} {
	global currentwindowid

	if {$new == ""} {
		set new  "Untitled"
	}

	if [PhxApp_get_windowlock] {

		catch {set newwin [PhxApp_send_mgrcommand PhxMgr_rename_window $currentwindowid $new]}
		PhxApp_release_windowlock
	}

	if [info exists newwin] {
		set currentwindowname $newwin
	}

	return $currentwindowname
}
