################################################################################
#
#  Run the Phoenix manager
#
################################################################################

proc PhxMgr_run {} {
	global env argv

	# Get environment variables from the shell and transfer them to 
	# Tcl global variables.  Where none, use the default.

	foreach variable [list \
	    {PHOENIX_DIR envphxdir /opt/phoenix} \
	    {TKWWW_DIR tkwwwdir $envphxdir/WWW/TkWWW/Tcl} \
	    {PHXMGR_DIR phxsrcdir $envphxdir/Phxmgr} \
	    {PHXBIN_DIR phxbindir $envphxdir/bin} \
	    {HOME envhome ""} \
	    {LOGNAME envlogname ""} \
	    {WWW_HOME envwwwhome ""}] \
	{
		set shell [lindex $variable 0]
		set tcl [lindex $variable 1]
		set default [lindex $variable 2]
		global $tcl
		catch {unset $tcl}
		if {[catch {set $tcl $env($shell)}] && $default != ""} {
			eval "set $tcl $default"
		}
	}

	# Add the source path to the auto load table
	global auto_path
	set auto_path "$phxsrcdir $tkwwwdir $auto_path"

	global phxwindowlist
	set phxwindowlist ""
	Phxmgr_phxapplistInitialize
	eval exec $phxbindir/phxclone $argv &

	wm withdraw .

	signal trap {SIGINT SIGTERM} {PhxMgr_exit}

	PhxMgr_append_windowmenu_to_phxapplist

	global windowmgrlock waiting MAX_WINDOWS
	set windowmgrlock ""
	set waiting(1) 1 ; unset waiting(1)
	set MAX_WINDOWS 8

	after 5000 PhxMgr_monitor_lock

	global phxwindowlist windowdestroyed phxmgrexiting
	while 1 {
		set windowdestroyed 0
		tkwait variable phxwindowlist
		if {[llength $phxwindowlist] == 0} {
			if [info exists phxmgrexiting] {
				return
			}
			PhxMgr_kill
		}
	}
}

proc PhxMgr_exit {} {

	global windowdestroyed
	global phxwindowlist
	global phxmgrexiting

	set phxmgrexiting 1

	set ra [PhxMgr_get_runningapps]
	set phxwindowlist ""

	foreach i $ra {

		set windowdestroyed 0
		catch {send $i PhxApp_kill}
	}

	PhxMgr_kill
}

proc PhxMgr_kill {} {

	global phxsrcdir

	global windowmgrlock waiting
	while {$windowmgrlock != ""} {
		tkwait variable windowmgrlock
	}
	while {[array names waiting] != ""} {
		tkwait variable waiting
	}
	destroy . 
	exit
}



proc PhxMgr_change_status {var val} {

	global $var
	set $var $val
}


proc PhxMgr_append_windowmenu_to_phxapplist {} {

	global phxapplist

	set newphxapplist ""

	foreach a $phxapplist {

		set wm ".phoenix"
		set appmenupath [PhxMgr_get_appmenupath_from_appentry $a]

		foreach i $appmenupath {
			
			set i [string tolower [PhxUtil_replace_spaces_withdashes $i]]
			set wm $wm.menu.$i
		}
		lappend a ${wm}windows
		lappend newphxapplist $a
	}

	set phxapplist $newphxapplist
}




################################################################################
#
#  Run/kill Phoenix applications
#
################################################################################




proc PhxMgr_run_me {appmenupath} {

	set command [PhxMgr_get_command_from_appentry \
		[PhxMgr_get_appentry_from_appmenupath $appmenupath]]

	if {$command == ""} {

		return 
	}

	eval exec $command &
}




################################################################################
#
#  Phoenix Windows
#
################################################################################


proc PhxMgr_raise_window {windowname} {

	set windowentry [PhxMgr_get_windowentry_from_windowname $windowname]
	set interp [PhxMgr_get_interpname_from_windowentry $windowentry]
	set window [PhxMgr_get_window_from_windowentry $windowentry]

	send $interp wm raise $window
}

proc PhxMgr_add_window {interpname menubar windowname windowid} {

	global phxwindowlist	
	global MAX_WINDOWS


	if {[llength $phxwindowlist] >= $MAX_WINDOWS} {
		PhxMgr_signal_toomanywindows $interpname
		send $interpname PhxApp_kill
		return
	}

	set window [send $interpname winfo toplevel $menubar]

	set windowname [PhxMgr_find_uniquewindowname $windowname]

	set we [list $interpname $menubar $windowname $window $windowid]
	lappend phxwindowlist $we

	foreach i $phxwindowlist {
		set in [PhxMgr_get_interpname_from_windowentry $i]
		set me [PhxMgr_get_menubar_from_windowentry $i]
		send $in PhxApp_refresh_phoenixmenu $me
		#send $in PhxApp_add_windowmenu $menubar \"[PhxMgr_get_appmenupath_from_windowentry $we]\" \"$windowname\"
	}
	
}

proc PhxMgr_signal_toomanywindows {interp} {

	global MAX_WINDOWS

	PhxMgr_releasing_windowlock $interp
	if [winfo exists .toomany] {
		return
	}

	toplevel .toomany
	wm title .toomany "Too many windows"
	wm geometry .toomany +200+100
	message .toomany.message -text "You have summoned more than $MAX_WINDOWS Phoenix windows.\nYou need to quit another window and try again." -justify center -aspect 1500
	button .toomany.button -text "OK" -command {destroy .toomany}
	pack append .toomany .toomany.message top
	pack append .toomany .toomany.button top
}

proc PhxMgr_delete_window {windowid} {

	global phxwindowlist

	for {set i 0} {$i < [llength $phxwindowlist]} { incr i 1} {

		set w [lindex $phxwindowlist $i]
		set wid [PhxMgr_get_windowid_from_windowentry $w]
		if {$wid == $windowid} {
			break
		}
	}

	if {$i >= [llength $phxwindowlist]} {
		puts "$windowid not found in $phxwindowlist"
		return
	}

	set amp	[join [PhxMgr_get_appmenupath_from_windowentry $w]]
	set windowname [join [PhxMgr_get_windowname_from_windowentry $w]]
	set interpname [join [PhxMgr_get_interpname_from_windowentry $w]]

	set xx [lreplace $phxwindowlist $i $i]
	set phxwindowlist $xx

	foreach x $xx {
		set in [PhxMgr_get_interpname_from_windowentry $x]
		set me [PhxMgr_get_menubar_from_windowentry $x]

		send $in PhxApp_delete_window_frommenubar \"$me\" \"$amp\" \"$windowname\"
	}


	send $interpname PhxApp_change_status destroyyourself 1
}
	
proc PhxMgr_find_uniquewindowname { windowname } {

	global phxwindowlist

	while {$windowname != [join $windowname]} {
		set windowname [join $windowname]
	}

	set windownamelist  [PhxMgr_get_windows] 

	while {[lsearch $windownamelist $windowname] > -1} {

		regexp {#([0-9]*)$} $windowname {} dupnum
		if {[info exists dupnum] && $dupnum != ""} {
			incr dupnum
			regsub {[0-9]*$}  $windowname $dupnum newwindowname
			set windowname $newwindowname
		} else {
			set windowname "$windowname #2"
		}
	}	
	return $windowname
}






################################################################################
#
#  Phoenix Menus
#
################################################################################

proc PhxMgr_inactivate_app {amp} {

	global phxwindowlist

	foreach i $phxwindowlist {
		set in [PhxMgr_get_interpname_from_windowentry $i]
		set me [PhxMgr_get_menubar_from_windowentry $i]
		send $in \[PhxApp_get_menu $me.phoenix.menu \"$amp\"\] disable  \"[lindex $amp [expr [llength $amp]-1]]"
	}
}
		
proc PhxMgr_activate_app {appnamepath} {

	global phxwindowlist

	foreach i $phxwindowlist {
		set in [lindex $i 0]
		set me [lindex $i 1]
		send $in \[PhxApp_get_menu $me.phoenix.menu [list $appnamepath]\] enable \"[lindex $appnamepath [expr [llength $appnamepath]-1]]\"
	}
}


proc PhxMgr_rename_window {windowid new} {

	global phxwindowlist


	set we [PhxMgr_get_windowentry_from_windowid $windowid]
	set newwe [lreplace $we 2 2 \
			    [set newwin [PhxMgr_find_uniquewindowname $new]]]
	set old [PhxMgr_get_windowname_from_windowentry $we]
	set index [lsearch $phxwindowlist $we]
	set oldphxwindowlist $phxwindowlist
	set phxwindowlist [lreplace $phxwindowlist $index $index $newwe]
	set windowmenu [PhxMgr_get_windowmenu_from_appentry [PhxMgr_get_appentry_from_windowentry $we]]

	foreach i $oldphxwindowlist {

		set interpname [PhxMgr_get_interpname_from_windowentry $i]
		set menubar [PhxMgr_get_menubar_from_windowentry $i]

		if {[join $i] == [join $we]} {
			#send $interpname $menubar$windowmenu entryconfig \[join \"$old\"\]  -background yellow
		} 
		catch {send $interpname $menubar$windowmenu entryconfig \[join \"$old\"\]  -label \[join \"$newwin"\] -command "\"PhxApp_send_mgrcommand PhxMgr_raise_window \\\"[join $newwin]\\\"\"" }
	}

	return $newwin
}
		


################################################################################
#
#  Get from phxwindowlist
#
################################################################################


proc PhxMgr_get_windowentry_from_windowname {windowname} {

	global phxwindowlist

	foreach i $phxwindowlist {
		if {[join $windowname] == \
			[join [PhxMgr_get_windowname_from_windowentry $i]]} {
			return $i
		}
	}

	return ""
}

proc PhxMgr_get_windowentry_from_windowid {windowid} {

	global phxwindowlist

	foreach i $phxwindowlist {
		if {[join $windowid] == \
			[join [PhxMgr_get_windowid_from_windowentry $i]]} {
			return $i
		}
	}

	return ""
}


proc PhxMgr_get_interpname_from_windowentry {we} {
	return [lindex $we 0]
}

proc PhxMgr_get_menubar_from_windowentry {we} {
	return [lindex $we 1]
}

proc PhxMgr_get_windowname_from_windowentry {we} {
	return [lindex $we 2]
}

proc PhxMgr_get_window_from_windowentry {we} {
	return [lindex $we 3]
}

proc PhxMgr_get_windowid_from_windowentry {we} {
	return [lindex $we 4]
}


################################################################################
#
#  Get from phxapplist
#
################################################################################


proc PhxMgr_get_appentry_from_appmenupath {amp} {

	global phxapplist

	foreach i $phxapplist {
		if {[join [PhxMgr_get_appmenupath_from_appentry $i]] == \
			[join $amp]} {
			return $i
		}
	}
	return ""
}

proc PhxMgr_get_appentry_from_interpname {interpname} {

	global phxapplist

	set interpname [lindex $interpname 0]
	foreach i $phxapplist {
		if {$interpname == [PhxMgr_get_interpname_from_appentry $i]} {
			return $i
		}
	}

	return ""
}

proc PhxMgr_get_appmenupath_from_appentry {ae} {
	return [lindex $ae 0]
}

proc PhxMgr_get_interpname_from_appentry {ae} {
	return [lindex $ae 1]
}

proc PhxMgr_get_command_from_appentry {ae} {
	return [lindex $ae 2]
}

proc PhxMgr_get_windowmenu_from_appentry {ae} {
	return [lindex $ae 3]
}

##### Get-from-phxapplist macros

proc PhxMgr_get_appmenupath_from_windowentry {we} {

	return [PhxMgr_get_appmenupath_from_interpname \
		[PhxMgr_get_interpname_from_windowentry $we]]
}

proc PhxMgr_get_appmenupath_from_interpname {i} {

	return [PhxMgr_get_appmenupath_from_appentry [PhxMgr_get_appentry_from_interpname $i]]
}

proc PhxMgr_get_interpname_from_appmenupath {a} {
	return [PhxMgr_get_interpname_from_appentry [PhxMgr_get_appentry_from_appmenupath $a]]
}

proc PhxMgr_get_appentry_from_windowentry {we} {

	return [PhxMgr_get_appentry_from_interpname \
		[PhxMgr_get_interpname_from_windowentry $we]]
}



################################################################################
#
#  Get lists of window or app values
#
################################################################################

proc PhxMgr_get_appmenupaths {} {

	global phxapplist

	set l ""
	foreach i $phxapplist {
		lappend l [PhxMgr_get_appmenupath_from_appentry $i]
	}

	return $l
}


		
proc PhxMgr_get_windows {{amp ""}} {
	global phxwindowlist

	set appwlist ""

	set interpname [PhxMgr_get_interpname_from_appmenupath $amp]

	foreach i $phxwindowlist {

		if {$amp == "" || \
		    [regexp $interpname [PhxMgr_get_interpname_from_windowentry $i]]} {
			lappend appwlist [PhxMgr_get_windowname_from_windowentry $i]
		}
	}

	return $appwlist
}

proc PhxMgr_get_menubars {interpname} {

	global phxwindowlist

	set l ""
	foreach i $phxwindowlist {
		if {[lindex $i 0] == $interpname} {
			lappend l [lindex $i 1]
		}
	}

	return $l
}

proc PhxMgr_get_runningapps {} {

	global phxwindowlist

	set l ""
	foreach i $phxwindowlist {
		
		set interpname [PhxMgr_get_interpname_from_windowentry $i]
		if {[lsearch $l $interpname] < 0} {
			lappend l $interpname
		}
	}

	return $l
}





################################################################################
#
#  Locking the "window manager"
#
################################################################################

proc PhxMgr_requesting_windowlock {interp} {

	global windowmgrlock
	global waiting

	# Make sure sender is still alive
	if {[lsearch [winfo interps] $interp] == -1} {
		return -1
	}

	# Nab the lock if unlocked
	if {"$windowmgrlock" == ""} {
		set windowmgrlock $interp
		return 1
	}

	regsub -all -- " " $interp "" iinterp
	set waiting($iinterp) $interp

	return 0
}

proc PhxMgr_monitor_lock {} {

	global windowmgrlock

	# Make sure locker is still alive
	if {"$windowmgrlock" != "" && \
		[lsearch [winfo interps] $windowmgrlock] == -1} {
		PhxMgr_releasing_windowlock $windowmgrlock
	}

	after 5000 PhxMgr_monitor_lock
}


proc PhxMgr_releasing_windowlock {interp} {
	
	global windowmgrlock waiting

	if {"$windowmgrlock" != "$interp"} {
		return
	}

	set windowmgrlock ""

	if [array size waiting] {
		set iinterp [lindex [array names waiting] 0]
		if {$iinterp == ""} {
			unset waiting($iinterp)
			continue
		}
			
		set interp $waiting($iinterp)
		
		unset waiting($iinterp)
		set result ""
		catch {set result [send $interp incr windowlock_tryagain]}
		if {[regexp "did not respond" $result] || \
			$result == ""} {
			PhxMgr_releasing_windowlock $interp
		}
	}
}



################################################################################
#
#  Go
#
################################################################################

PhxMgr_run
