# Some better bindings for text and entry
bind Text <Up> {
  selection clear %W
  %W mark set insert {insert - 1 line}
  %W yview -pickplace insert
}
bind Text <Down> {
  selection clear %W
  %W mark set insert {insert + 1 line}
  %W yview -pickplace insert
}
bind Text <Left> {
  selection clear %W
  %W mark set insert {insert - 1 char}
  %W yview -pickplace insert
}
bind Text <Right> {
  selection clear %W
  %W mark set insert {insert + 1 char}
  %W yview -pickplace insert
}
bind Text <Home> {%W mark set insert {insert linestart}
     selection clear %W
     %W yview -pickplace insert}
bind Text <End> {%W mark set insert {insert lineend}
     selection clear %W
     %W yview -pickplace insert}
bind Text <Control-Home> {%W mark set insert 1.0
     selection clear %W
     %W yview -pickplace insert}
bind Text <Control-End> {%W mark set insert end
     selection clear %W
     %W yview -pickplace insert}

bind Text <Next> {
  global textscroll
  selection clear %W
  %W mark set insert [lindex $textscroll(%W) 3].0
  %W yview insert
}
bind Text <Prior> {
  global textscroll
  selection clear %W
  %W mark set insert [expr \
	[lindex $textscroll(%W) 2]-[lindex $textscroll(%W) 1]].0
  %W yview insert
}

bind Text <Any-KeyPress> {
     if {"%A" != ""} {
         catch {%W delete sel.first sel.last}
	 %W insert insert %A
        %W yview -pickplace insert
    }
}

bind Text <KeyPress-Delete> {
   if [catch {%W delete sel.first sel.last}] {
       %W delete [%W index insert-1c]
   }
}
bind Text <KeyPress-BackSpace> {
   if [catch {%W delete sel.first sel.last}] {
       %W delete [%W index insert-1c]
   }
}
bind Text <KeyPress-Return> {
   catch {%W delete sel.first sel.last}
   %W insert insert "\n"
   %W yview -pickplace insert
}

bind Text <1> "+selection clear %W"
bind Text <2> {%W insert insert [selection_if_any]}
bind Text <3> {selection clear %W}

bind Text <Shift-Control-KeyPress-S> \
	"mf_searchtxt %W 1"
bind Text <Control-KeyPress-s> \
	"mf_searchtxt %W 0"
bind Text <Alt-X> "mf_cutedit %W"
bind Text <Alt-C> "mf_copyedit %W"
bind Text <Alt-V> "mf_pasteedit %W"
bind Text <Meta-x> "mf_cutedit %W"
bind Text <Meta-c> "mf_copyedit %W"
bind Text <Meta-v> "mf_pasteedit %W"

bind Text <B1-Motion> {textB1Motion %W @%x,%y}

set mfp(txnd) 0
set mfp(xdelay) 100
proc textB1Motion  { w loc } {
    global mfp

    set ypos [lindex [split $loc ","] 1]
    if {$ypos > [winfo height $w]} {
            if {!$mfp(txnd)} {after $mfp(xdelay) textExtend $w}
            set mfp(txnd) 1
            set mfp(direction) down
    } elseif {$ypos < 0} {
            if {!$mfp(txnd)} {after $mfp(xdelay) textExtend $w}
            set mfp(txnd) 1
            set mfp(direction) up
    } else {
            set mfp(txnd) 0
            set mfp(direction) 0
    }

    if {!$mfp(txnd)} {
            tk_textSelectTo $w $loc
    }

}

bind Text <ButtonRelease-1> { global mfp; set mfp(txnd) 0 }

proc textExtend { w } {
     global mfp

     if {$mfp(txnd)} {
         if {$mfp(direction) == "down"} {
             tk_textSelectTo $w sel.last+1l
             $w yview -pickplace sel.last+1l
         } elseif {$mfp(direction) == "up"} {
             tk_textSelectTo $w sel.first-1l
             $w yview -pickplace sel.first-1l
         } else { return }
         after $mfp(xdelay) textExtend $w
     }
}

bind Entry <Delete> {
	if {[catch {%W delete sel.first sel.last}] != 0} {
		tk_entryForwspace %W
		tk_entrySeeCaret %W
	}
}
bind Entry <BackSpace> {
	if {[catch {%W delete sel.first sel.last}] != 0} {
		tk_entryBackspace %W
		tk_entrySeeCaret %W
	}
}
bind Entry <Left> {
	%W select clear
	%W icursor [expr {[%W index insert] - 1}]
	tk_entrySeeCaret %W
}
bind Entry <Right> {
	%W select clear
	%W icursor [expr {[%W index insert] + 1}]
	tk_entrySeeCaret %W
}
bind Entry <Home> {
	%W select clear
	%W icursor 0
	tk_entrySeeCaret %W
}
bind Entry <End> {
	%W select clear
	%W icursor end
	tk_entrySeeCaret %W
}
bind Entry <Any-KeyPress> {
    if {"%A" != ""} {
        catch {%W delete sel.first sel.last}
        %W insert insert %A
        tk_entrySeeCaret %W
    }
}

bind Entry <Double-Button-1> {%W select from 0; %W select to end}
bind Entry <2> {%W insert insert [selection_if_any]}
bind Entry <B2-Motion> { }
bind Entry <3> {selection clear %W}
bind Entry <Shift-2> {%W scan mark %x}
bind Entry <Shift-B2-Motion> {%W scan dragto %x}

bind Entry <Meta-KeyPress-x> "mf_cutedit %W"
bind Entry <Meta-KeyPress-c> "mf_copyedit %W"
bind Entry <Meta-KeyPress-v> "mf_pasteedit %W"

# selection_if_any - return selection if it exists, else {}
#   this is from kjx@comp.vuw.ac.nz (R. James Noble)
proc selection_if_any {} {
  if {[catch {selection get} s]} {return ""} {return $s}
}

proc mf_searchtxt { tw getnew } {
    global mf mfp

    set xpos [expr [winfo rootx $mfp(top)]+[winfo width $mfp(top)]/3]
    set ypos [expr [winfo rooty $mfp(top)]+[winfo height $mfp(top)]/3]
    if {$getnew || $mfp(searchstr) == ""} {
	set mfp(searchstr) [tkgetstring "Search for:" $mfp(searchstr) $xpos $ypos]
	if {$mfp(searchstr) == ""} {return 0 }
    }
    focus $tw

    if {![TextSearch $tw $mfp(searchstr) [$tw index insert]]} {
	eval $mf(beep); return 0
    }
    return 1
}

proc mf_cutedit { tw } {
    global mf mfp

    if {![mf_copyedit $tw]} return
    if {[catch "$tw delete sel.first sel.last"]} return
}

proc mf_copyedit { tw } {
    global mf mfp

    if {[winfo class $tw] == "Entry"} {
	if {[catch "$tw index sel.first" fndx]} {return 0}
	set lndx [$tw index sel.last]
	set mfp(cutbuffer) [$tw get]
	set mfp(cutbuffer) [string range $mfp(cutbuffer) $fndx $lndx]
    } else {
        if {[catch "$tw get sel.first sel.last" res]} {return 0}
	set mfp(cutbuffer) $res
    }
    return 1
}

proc mf_pasteedit { tw } {
    global mf mfp

    $tw insert insert $mfp(cutbuffer)
}

