################################################################
# Modifications beyond tkWWW 0.9 are Copyright 1993, 1994 by
# The University of Chicago Biological Sciences Division, Academic Computing
# Philip Stylianos
# Lee Newberg
################################################################

################################################################################
#
# Widgets - this file contains procedures to insert different widgets into
# the current hypertext widget representing the current page.  Currently,
# text boxes (called para so as not to be confused when text widgets w.r.t.
# forms are later added), images (labels), images with anchors (buttons) and
# horizontal rules are currently implemented
#
# Notes/thoughts
#
# * probalbly all the insert routines should be in just one, with the widget
# specific details in a case statement. What I mean is CurrentText and NewImage
# should be just createwidget, and the type of widget is passed to the proc.
# I started CurrentText like that, but did not complete it that way.
#
################################################################################

# phxW3WidgetInitialize 
# called when a new page is read in. Do all new page init stuff here
# 
proc phxW3WidgetInitialize {} {
	global phxW3CurrentWidget
	global phxW3WidgetCount

	# No current widget for a new page
	set phxW3CurrentWidget ""

	# Don't clear this; save in case user goes back to a page
	if {[info exists phxW3WidgetCount]} {
		#unset phxW3WidgetCount
	}
}

# phxW3WidgetAddHR 
# 
# This inserts a horizontal rule into the current htext.
# "before" is the widget to insert the HR before.  If none
# given, it's placed at the end of the page

proc phxW3WidgetAddHR {{before ""}} {

	global phxW3CurrentWidget

	# Find a unique name for the widget
	set instance [phxW3WidgetNewName rule]
#	set widget [phxW3CurrentHtext].frame.$instance
	set widget [phxW3CurrentFrame].$instance

	# Build the new widget from scratch
	canvas $widget -height 10 -borderwidth 0

	# Draw the line in the canvas.
	$widget create line 0 5 1000 5 -width 1

	# Bind the widget so that selecting it is possible
	phxW3SelectionWidgetInitialize $widget

	# Color the widget according to edit/browse mode
	phxW3ModeConfigure $widget

	# Pack widget, either at end of page or before given widget
	if {$before == ""} {
# "[phxW3CurrentHtext].frame" goes here
		pack configure $widget \
			-in [phxW3CurrentFrame] \
			-side top \
			-fill x
	} else {
		pack configure $widget -before $before \
		    -side top -fill x
	}

	# Set newly drawn rule to be the new widget
	set phxW3CurrentWidget $widget
}



# phxW3WidgetCreateText 
# Call this to create a new para widget, their names should be changed to
# CreatePara to distinguish between widgets holding text and form widgets
# allowing text entry.
#
proc phxW3WidgetCreateText {} {

	# Call current text forcing a new widget to be built
	phxW3WidgetCurrentText new
}

# phxW3WidgetCurrentText 
# This was the first insert widget command written, I had big plans for it
# to be the proc to insert any kind of widget. Just pass it the wanted class,
# and a case statement will handle the widget-specific stuff, but for now it 
# just handles the text (para) widget.

proc phxW3WidgetCurrentText {{new ""} {wantedclass para}} {
	global phxW3CurrentWidget
	global tkW3HtAnchors
	global phxW3HtBuildingDocument

	# If we're not forcing a new widget to be created, either
	# return the name of the CurrentWidget if it is a $wantedclass, or
	# create a new one and return that widget's name.  This non-new
	# mode is used extensively in hypertext.tcl when building a page
	# When text is to be output, a call to this proc conveniently
	# handles outputting that text, whether or not the last widget
	# created was a text widget.
	if {$new == ""} {

	    # If we're not building a page, return the para or list widget
	    # with the focus, or null if no such widget has the focus
	    if {$phxW3HtBuildingDocument == 0} {
		set class [phxW3WidgetClass [focus]]

		if {$class != "para" && ![string match $class "list"]} {
			return ""
		}
		return [focus]
	    }

	    # Find class of current widget

	    # If it's the right kind of class, no need to create a new one
	    if {[phxW3WidgetClassCurrentWidget] == $wantedclass} {
		return $phxW3CurrentWidget
	    }

	}

	#
	# Create a new widget of the given class
	#

	# Set class to the wanted class
	set class $wantedclass

	# Find a unique name for the new instance (para, para2, etc.)
	set instance [phxW3WidgetNewName $class]

	# Set widget name
	set frame [phxW3CurrentFrame]

	# We change CurrentFrame to the parent of an item because when
	# creating a new list, there seems to be some problem with
	# packing it within an "item" widget, whereas it works fine
	# if packed directly into the parent list widget.
	# Not a good solution!  Fix eventually!
	if {[phxW3WidgetClass $frame] == "item"} {
		phxW3CurrentFrame [winfo parent $frame]
	}

	set widget [phxW3CurrentFrame].$instance

	switch -glob $class {
	    para {
		text $widget \
		    -width 0 -height 0 \
		    -borderwidth 0 \
		    -relief flat \
		    -yscrollcommand "phxW3WidgetAdjustHeight $widget"
		}
	    list_* {
		frame $widget \
		    -width 0 -height 0 \
		    -borderwidth 0 \
		    -relief flat
		# add space on left
		frame $widget.space \
		    -width 20 -height 0 \
		    -borderwidth 0 \
		    -relief flat
		pack configure $widget.space -in $widget -side left \
		    -fill y -expand no
	    }
 	}

	phxW3SelectionWidgetInitialize $widget
	phxW3ModeConfigure $widget
	if {[winfo class $widget] == "Text"} {
	    tkW3FontSetTextFonts $widget
	    global phxW3ModeTextState
	    $widget config -state $phxW3ModeTextState
	}

	# If this is the first widget, pack it at the top, otherwise place
	# after the last widget
	pack configure $widget -in [phxW3CurrentFrame] \
	    -side top -anchor nw -fill x -expand yes

	if [string match "list_*" $class] {
	    phxW3CurrentFrame $widget
	}

	# Now this widget is the current widget
	set phxW3CurrentWidget $widget

	cmdtrace off

	if {[info exists tkW3HtAnchors(in_anchor)] && \
		$tkW3HtAnchors(in_anchor)} {
		tkW3HtMakeAnchor $tkW3HtAnchors(index) \
		    $tkW3HtAnchors(name.$tkW3HtAnchors(index)) \
		    $tkW3HtAnchors(href.$tkW3HtAnchors(index))
	}

	return $widget
}

# phxW3WidgetCreateList
# Create new frame and text widgets in which to store a current list.

proc phxW3WidgetCreateList {elem} {
	phxW3WidgetCurrentText new list_$elem
}

# phxW3WidgetCreateListItem
# Create a frame for a particular list item, packed into the frame for
# the current list.  Arg "indent" is the amount to which to indent the list.
# Optional arg "previtem" is a widget after which to pack the new one.

proc phxW3WidgetCreateListItem {indent {previtem ""}} {
    global phxW3CurrentWidget tkW3HtAnchors

    # get a name for the new list item
    set instance [phxW3WidgetNewName "item"]

    set frame [phxW3CurrentFrame]
    if {[phxW3WidgetClass $frame] == "item"} {
	phxW3CurrentFrame [winfo parent $frame]
    }

    set widget [phxW3CurrentFrame].$instance

    # Create and configure this widget.
    frame $widget \
	-width 20 -height 0 \
	-borderwidth 0 \
	-relief flat

    if {$previtem != ""} {
	pack configure $widget -in [phxW3CurrentFrame] \
	    -side top -after $previtem -anchor nw -fill x
    } else {
	pack configure $widget -in [phxW3CurrentFrame] \
	    -side top -anchor nw -fill x
    }

    text $widget.bullet \
	-width $indent -height 0 \
	-borderwidth 0 \
	-relief flat

    text $widget.para \
	-width 0 -height 0 \
	-borderwidth 0 \
	-relief flat \
	-yscrollcommand "phxW3WidgetAdjustHeight $widget.para"

    phxW3SelectionWidgetInitialize $widget.para
    phxW3ModeConfigure $widget
    phxW3ModeConfigure $widget.para
    phxW3ModeConfigure $widget.bullet
    tkW3FontSetTextFonts $widget.para
    global phxW3ModeTextState
    $widget.bullet config -state $phxW3ModeTextState
    $widget.para config -state $phxW3ModeTextState

    pack configure $widget.bullet -in $widget -side left -anchor nw
    pack configure $widget.para -in $widget -side left -fill x -expand yes

    # Now this widget is the current widget
    set phxW3CurrentWidget $widget.para
    phxW3CurrentFrame $widget

    if {[info exists tkW3HtAnchors(in_anchor)] && \
	$tkW3HtAnchors(in_anchor)} {
	tkW3HtMakeAnchor $tkW3HtAnchors(index)
    }
}

# phxW3WidgetAdjustHeight
# A yscrollcommand for a para widget so that it adjusts its own height.
# 
# Note that there are THREE ways to measure the optimal height
# of a text widget.
#   
#   First: $total, $shown, $first, and $last are measured in
#   units in which $total is the same as the line number in [$w
#   index end].
#   
#   Second: [set height [lindex [$w configure -height] 4]] is
#   $total plus one for each time any line wraps.
#   
#   Third: The actual pixel height of a widget is determined by
#   the $height times a fixed value which depends on the
#   widget's font PLUS corrections for any lines which contain
#   fonts of a different height (due to tags within the widget).
# 
# Unfortunately, the easiest input we can get about what a widget
# thinks its optimal height is comes from $total which is
# measured according to the first method.
# 
# Our only way to affect the height of the widget is through the
# [$w configure -height $height] which guesses the pixel height
# based on the second measure.
# 
# What we want is the actual height according to the third
# measure.


proc phxW3WidgetAdjustHeight {w total shown first last} {
    global phxW3WidgetHeight

    # The title widget is created before fonts are initialized, so
    # we must be careful.
    if {[set fontHeight [phxW3FontGetHeight]] == ""} {return}

    # Unfortunately, $height may be updated before $pixel is.  Thus,
    # the following values don't necessarily correspond to each other!
    set height [lindex [$w configure -height] 4]
    set pixel [winfo height $w]

    # Several calls to phxW3WidgetAdjustHeight may occur between
    # actual screen updates to the widget.  We wish to process only
    # the first of each collection so we will proceed only if some
    # parameter has changed.
    set parameters "$height $pixel $fontHeight $total $shown $first $last"
    if {[info exists phxW3WidgetHeight($w,parameters)] && \
	$parameters == $phxW3WidgetHeight($w,parameters)} { return }

    # Something has changed.  Remember the new parameters and then proceed.
    set phxW3WidgetHeight($w,parameters) $parameters

    # If we haven't set a stage number, start with the first stage.
    if {![info exists phxW3WidgetHeight($w,stage)]} {
	set phxW3WidgetHeight($w,stage) 1
    }

    if {$phxW3WidgetHeight($w,stage) == 1} {
	# We don't know the right height yet.
	if {[$w compare "end - 1 c" == end]} {
	    # We've got a good estimate for the widget's height.
	    set phxW3WidgetHeight($w,stage) 2
	    set newHeight 1
	} else {
	    set y1 [lindex [phxW3EditFindPlace $w "end - 1 c"] 1]
	    set y2 [lindex [phxW3EditFindPlace $w "end"] 1]
	    if {$y1 == $y2} {
		# The widget is too small to get an accurate measure.
		set newHeight [expr {($height*$total)/$shown}]
		if {$newHeight == $height} {incr newHeight}
	    } else {
		# We've got a good estimate of the widget's pixel height.
		set phxW3WidgetHeight($w,stage) 2
		if {[lindex [split [$w index end] .] 1] == 0} {
		    incr y1 [expr $fontHeight/2]
		}
		set border [lindex [$w config -bd] 4]
		if {$border == 0} {incr border}
		# 2 works to display the bottommost pixel:
		set newHeight [expr {($y1+$fontHeight-2-$border)/$fontHeight}]
	    }
	}
	if {$height != $newHeight} {
	    $w configure -height $newHeight
	} else {
	    # No change, so we must have already moved to stage 2.  Now we
	    # move beyond it.
	    unset phxW3WidgetHeight($w,stage)
	}
    } else {
	# Everything is great.  Mark it as so.
	# Note that this case must exist because phxW3WidgetAdjustHeight
	# will be called after the height is set correctly.
	unset phxW3WidgetHeight($w,stage)
    }
}

# phxW3WidgetYview
# Don't let the user display the blank space after the widget.
    
proc phxW3WidgetYview {scrollbar widget line} {

    set got [$scrollbar get]

    # Determine the highest legal yview argument.
    set highest [expr [lindex $got 0] - [lindex $got 1] + 1]
    if {$highest < 0} { set highest 0 }

    # Proceed with the yview...
    if {$line <= $highest} {
	$widget yview $line
    } else {
	$widget yview $highest
    }
}

proc phxW3WidgetInsertRule {w} {
    if {[phxW3ModeGet] != "edit"} {return}
    set right [phxW3SelectionGetRightOfFocus]

    switch [phxW3WidgetClass $right] {
	end {set before ""}
	para {
	    if {[phxW3SelectionGetLeftOfFocus] == $right} {
		set before [phxW3WidgetSplitPara $right]
		phxW3SelectionSetFocus $right $before
	    }
	}
	image {
	    # Split the imageholder if necessary.
	    set before [phxW3WidgetSplitImageHolder $right]
	    phxW3SelectionSetFocus $right $before
	}
	default {set before $right}
    }

    phxW3WidgetAddHR $before
    if {$before == ""} {set before "end"}
    phxW3SelectionSetFocus [phxW3WidgetLeft $before] $before
}

# phxW3WidgetSplitPara
#
# Split the given para widget into two adjacent para widgets, divided
# at the insertion (cursor) point.  Returns the name of the right-hand
# para widget.  Called by routines which need to add a new kind of
# widget -- e.g. HR, image, list -- in the middle of a para widget.

proc phxW3WidgetSplitPara {w} {

    if {[phxW3WidgetClass $w] != "para"} { return }

    set place [$w index insert]
    if {$place == "1.0"} { return $w }
    if {$place == "end"} { return [phxW3WidgetRight $w] }

    set newt [phxW3WidgetCreateText]
    pack $newt -after $w

    # Copy the text from after the insert mark.
    phxW3WidgetCopyText $w insert end $newt 1.0

    $w delete insert end

    phxW3ModeConfigure $newt
    return $newt
}

# phxW3WidgetCopyText
# Copies text and tags from one text widget to another
# Returns the location in $wTo of the end of the inserted selection.

proc phxW3WidgetCopyText {wFrom iStart iStop wTo iBegin \
    {tags 1} {configurations 1} {bindings 1}} \
{
    # Simplify the supplied indices.
    set iStart [$wFrom index $iStart]
    set iStop [$wFrom index $iStop]
    set iBegin [$wTo index $iBegin]

    # Copy the text, keeping track of where it ends in $wTo
    $wTo mark set WCTinsert $iBegin
    $wTo insert $iBegin [$wFrom get $iStart $iStop]
    set iEnd [$wTo index WCTinsert]
    $wTo mark unset WCTinsert

    # If we're not supposed to copy the tags, then we're done.
    if {$tags == 0} { return $iEnd }

    # Remove any inherited tags
    foreach tag [$wTo tag names] {
	$wTo tag remove $tag $iBegin $iEnd
    }

    # Copy tags from the source
    foreach tag [$wFrom tag names] {
	if {$tag == "sel"} continue;
	for {set index 1.0} \
	    {[set range [$wFrom tag nextrange $tag $index]] != ""} \
	    {set index $eFrom} \
	{
	    set bFrom [lindex $range 0]
	    set eFrom [lindex $range 1]
	    if {[$wFrom compare $eFrom > $iStart] && \
		[$wFrom compare $bFrom < $iStop]} \
	    {
		# The range includes some inserted text.
		# Crop the range if necessary.
		if {[$wFrom compare $bFrom < $iStart]} {
		    set bFrom $iStart
		}
		if {[$wFrom compare $eFrom > $iStop]} {
		    set eFrom $iStop
		}

		# Compute the corresponding locations in $wTo.
		set bTo [phxW3WidgetCombineTextIndices $bFrom $iStart $iBegin]
		set eTo [phxW3WidgetCombineTextIndices $eFrom $iStart $iBegin]

		# Tag the inserted text.
		$wTo tag add $tag $bTo $eTo
	    }
	}	

	if {$configurations} {
	    # Copy the "configure" attributes of the tag.
	    foreach attr [$wFrom tag configure $tag] {
		$wTo tag configure $tag [lindex $attr 0] [lindex $attr 4]
	    }
	}

	if {$bindings} {
	    # Copy the bindings for the tag
	    foreach binding [$wFrom tag bind $tag] {
		$wTo tag bind $tag $binding [$wFrom tag bind $tag $binding]
	    }
	}
    }

    return $iEnd
}

# phxW3WidgetCombineTextIndices

# It is assumed that text from index $b through index $a was
# inserted into another widget starting at index $c.  The goal
# is to compute the index in the second widget, corresponding to
# index $a.  In some sense, we're computing ($a-$b) + $c.  It is
# assumed that $a, $b, and $c are already of the form that would
# be produced by a "Text index" command.

proc phxW3WidgetCombineTextIndices {a b c} {
    regexp {([0-9]+)\.([0-9]+)} $a a aLine aChar
    regexp {([0-9]+)\.([0-9]+)} $b b bLine bChar
    regexp {([0-9]+)\.([0-9]+)} $c c cLine cChar

    incr aLine "-$bLine"

    if {$aLine == 0} {
	# $a is on the first line of the range starting at $b
	incr cChar [expr {$aChar - $bChar}]
    } else {
	# $a is not on the first line of the range starting at $b so
	# $bChar and $cChar are irrelevant.
	incr cLine $aLine
	set cChar $aChar
    }

    return $cLine.$cChar
}


# phxW3WidgetMergePara
# The opposite of phxW3WidgetSplitPara.
# Takes the given text widget and merges it with the previous widget if
# it is also a text widget.

proc phxW3WidgetMergePara {right} {

    set left [phxW3WidgetLeft $right]
    # Check that there is work to do.
    if {$left == "start" || [phxW3WidgetClass $left] != "para"} { return }

    if {[$left compare 1.0 == end] && [$right get 1.0] != "\n"} {
	# The left widget appears as a blank line.  Preserve that.
	$left insert 1.0 "\n"
    }

    # Do the merging
    set lStart [$left index end]
    set lStop [phxW3WidgetCopyText $right 1.0 end $left end]

    # Cannot concatenate headers with plain text or other headers.
    if {[$left compare $lStart > 1.0] && \
	[phxW3EditDetectParaStyle $left "$lStart - 1 c"] != \
	[phxW3EditDetectParaStyle $left $lStart]} \
    {
	# Preserving the insert mark, insert a paragraph separator.
	set insert [$left index insert]
	$left mark set insert $lStart
	phxW3EditParaInsert $left "\n"
	$left mark set insert $insert
    }

    # Manage the focus
    switch [phxW3SelectionGetLeftOfFocus] \
	$left {
	    switch [phxW3SelectionGetRightOfFocus] \
		$left {
		    # Insert was within $left.  Nothing to update.
		} \
		$right {
		    # Insert was between $left and $right
		    phxW3SelectionSetFocus $left $left
		    $left mark set insert $lStart
		} \
		default {
		    # $right was empty and contained the insert
		    $left mark set insert end
		}
	} \
	$right {
	    switch [phxW3SelectionGetRightOfFocus] \
		$left {
		    # Impossible
		} \
		$right {
		    # Insert was within $right
		    phxW3SelectionSetFocus $left $left
		    $left mark set insert \
			[phxW3WidgetCombineTextIndices \
			    [$right index insert] 1.0 $lStart]
		} \
		default {
		    # Insert was at the end of $right
		    phxW3SelectionSetFocus $left [phxW3SelectionGetRightOfFocus]
		    $left mark set insert end
		}
	} \
	default {
	    switch [phxW3SelectionGetRightOfFocus] \
		$left {
		    # Insert was at beginning of $left.  Nothing to update.
		} \
		$right {
		    # $left was empty and contained the insert.
		    phxW3SelectionSetFocus $left $left
		    $left mark set insert 1.0
		} \
		default {
		    # The focus was unrelated to the text widgets.
		    # Nothing to do
		}
	}

    if {$right == [phxW3SelectionGet]} { phxW3SelectionSet $left }

    destroy $right
}

# Create the name for a widget of the requested "class."  Note
# that the widget is named but not instantiated.  Also, note
# that the "class" is not the X class as returned by "winfo
# class $x".  It is a string which Phoenix wishes to associate
# with the widget.

proc phxW3WidgetNewName {class} {

	global phxW3WidgetCount

	if {[info exists phxW3WidgetCount($class)]} {
		incr phxW3WidgetCount($class)
	} else {
		set phxW3WidgetCount($class) 1
	}
	return $class$phxW3WidgetCount($class)
}

# phxW3WidgetClass returns the "class" of the supplied widget.
# The "class" is the same as that used when the widget's name is
# originally requested with phxW3WidgetNewName command.  If
# $widget is "start" or "end" then this routine should return
# $widget.  If $widget is in a .foo_dialog path, then treat it
# normally so that entry widgets will be treated sanely.
# Other widgets outside of .f that accept keyboard input will
# be treated as type "text".

proc phxW3WidgetClass {widget} {
	set class ""

	switch -glob $widget {
		"" -
		start -
		end {set class $widget}
		.*_dialog* -
		.f* {
			set class ""
			regexp {([^0-9\.]*)[0-9]*$} .$widget junk class
		}
		default {set class [string tolower [winfo class $widget]]}
	}

	return $class
}

# phxW3WidgetClassCurrentWidget is similar to phxW3WidgetClass
# except that it only handles the current widget.  If there is
# no current widget (because either phxW3CurrentWidget is not
# defined, or its value does not correspond to an existing
# widget) then it returns "".

proc phxW3WidgetClassCurrentWidget {} {
	global phxW3CurrentWidget
	if {[catch {winfo exists $phxW3CurrentWidget}] == 0} {
	    return [phxW3WidgetClass $phxW3CurrentWidget]
	} else {
	    return ""
	}
}

# phxW3WidgetInsertImage
# prompts the user for the filename of a gif and then inserts
# the image into the current document at the focus.

proc phxW3WidgetInsertImage {} {
    phxW3SelectionClear

    set focusLeft [phxW3SelectionGetLeftOfFocus]
    set focusRight [phxW3SelectionGetRightOfFocus]

    set file [phxW3Filebox "r"]
    if {$file != ""} {
	set URL [phxW3InfoConvertToHTTP $file]

	# This code strongly mimics part of phxW3ClipboardInsert!!
	if {[phxW3WidgetClass $focusLeft]=="para" && $focusLeft==$focusRight} {
	    # Split the text widget in two.
	    set focusRight [phxW3WidgetSplitPara $focusRight]
	    phxW3SelectionSetFocus $focusLeft $focusRight
	} else {
	    # Split the imageholder if necessary.
	    set focusRight [phxW3WidgetSplitImageHolder $focusRight]
	    phxW3SelectionSetFocus $focusLeft $focusRight
	}

	# phxW3WidgetNewImage looks at the current widget to
	# see if a new imageholder is to be created.  Also
	# looks at its "before" argument to figure out where
	# to pack the image once it is created.  We must set
	# both appropriately.
	phxW3WidgetSetCurrentWidget $focusLeft

	# Set "before" to the anscestor which is a child of the appropriate 
	# frame.
	switch [phxW3WidgetClass $focusRight] {
	    end {set before ""}
	    image -
	    textarea {set before [winfo parent $focusRight]}
	    default {set before $focusRight}
	}

	# Make the widget.
	set image [phxW3WidgetNewImage $URL $file 0 $before]    
	phxW3ModeConfigure $image
	phxW3SelectionSetFocus $image $focusRight
	phxW3WidgetWrap $image
	phxW3SaveDirtyBit
    }
}

proc phxW3WidgetImageDestroy {w} {

	global tkW3HtImages
	global tkW3HtAnchors

	if {[catch {set i $tkW3HtImages($w)}] == 0} {
		catch {unset tkW3HtImages($i.ismap)}
		catch {unset tkW3HtImages($i.anchor)}
		catch {unset tkW3HtImages($i.anchortag)}
		catch {unset tkW3HtImages($w)}
		catch {unset tkW3HtImages($i.source)}
		catch {unset tkW3HtImages($i.url)}
	}

	#tkW3ImageReleaseColorCells $w
}

proc phxW3WidgetImageUnmap {w} {

	set p [winfo parent $w]
	set ch [pack info $p]

	if {$ch == "" || [llength $ch] == 2 && [lindex $ch 0] == $w} {
		after 1000 destroy $p
	}
}


	
# phxW3WidgetNewImage
# High level routine to create a new image.  It does a bunch of
# preparation, calls phxW3WidgetCreateImage to instantiate the widget,
# and then winds down.

# url = absolute URL for image
# source = local file containing the bitmap
# ismap = whether the image is an ismap
# before = a widget to place the imageholder before	

proc phxW3WidgetNewImage {url source ismap {before ""}} {
	global phxW3CurrentWidget
	global phxW3WidgetCount
	global tkW3HtImages tkW3HtAnchors

	# Find new name for the image
	set instance [phxW3WidgetNewName image]
	
	# Find or build the imgframeholder the image is to go in
	if {[phxW3WidgetClassCurrentWidget] == "image"} {
		set image [winfo parent $phxW3CurrentWidget].$instance
	} else {  
		set imageholder [phxW3WidgetAppendNewFrame $before]
		set image $imageholder.$instance
	}

	if {[phxW3WidgetCreateImage $image $source $tkW3HtAnchors(in_anchor) \
		$tkW3HtAnchors(index) $ismap] == ""} \
	{
		# Source couldn't be loaded.
		return
	}

	# Set array vars for new image
	set i $tkW3HtImages(index)
	set tkW3HtImages($i.ismap) $ismap   
	if {$tkW3HtAnchors(in_anchor)} {
		set tkW3HtImages($i.anchor) \
			$tkW3HtAnchors(href.$tkW3HtAnchors(index))
		set tkW3HtImages($i.anchortag) $tkW3HtAnchors(index)
	} else {
		set tkW3HtImages($i.anchor) ""
		set tkW3HtImages($i.anchortag) ""
	}

	update idletasks

	if {[winfo reqwidth $image] != [winfo width $image] &&
		[llength [pack info [winfo parent $image]]] > 2} {
		destroy $image
		set imageholder [phxW3WidgetAppendNewFrame $before]
		set image $imageholder.$instance
		phxW3WidgetCreateImage $image $source \
			$tkW3HtAnchors(in_anchor) $tkW3HtAnchors(index) $ismap
	}

	# Overlay the image on the frame widget
	set tkW3HtImages($image)  $i

	set tkW3HtImages($i.url) $url
	set tkW3HtImages($i.source) $source
	incr tkW3HtImages(index)

	# We have a new current widget
	set phxW3CurrentWidget $image
	return $image
}

# phxW3WidgetCreateImage
# Low-level workhorse to create a widget.

# image = name for widget to be created
# source = local file containing bitmap
# anchor = whether it's an anchor
# index = index of the anchor (if it is one.)
# ismap = whether it's an ismap

proc phxW3WidgetCreateImage {image source anchor index ismap} {
    global tkW3HtAnchors

    if {$anchor  && $tkW3HtAnchors(href.$index) != ""} {
	button $image -borderwidth 1 -padx 0 -pady 0 -relief raised \
	    -activebackground blue
	if {$ismap == 0} {
	    $image configure -command "phxW3ModeSingleClickAnchor %W $index"
	    bind $image <Double-Button-1> \
		"phxW3ModeDoubleClickAnchor %W $index"
	} else {
	    # It's an ismap.  Create our own version of a button
	    bind $image <ButtonRelease-1> \
		"phxW3ModeSingleClickIsmap %W $index %x %y"
	    bind $image <Double-Button-1> \
		"phxW3ModeDoubleClickAnchor %W $index"
	}
    } else {
	label $image -borderwidth 1 -padx 0 -pady 0 -relief flat
    }
    if {[catch {$image configure -bitmap @$source}]} {
	$image configure -text "Image unloadable"
    }

    if {![winfo exists $image]} { return "" }
    phxW3SelectionWidgetInitialize $image

    phxW3ModeConfigure $image

    # Prepend <Destroy> binding
    phxW3SelectionSetBinding $image <Destroy> "phxW3WidgetImageDestroy %W"
    # Promote class binding to a window binding
    phxW3SelectionSetBinding $image <1> ""
    # Append to the window binding.
    bind $image <1> "+global phxW3CurrentWidget
	set phxW3CurrentWidget %W"

    pack configure $image -side left -anchor w 

    return $image
}

proc phxW3WidgetAppendNewFrame {{before ""}} {

	global phxW3CurrentWidget

	if {$before == ""} {
		set imageholder \
			[phxW3CurrentFrame].[phxW3WidgetNewName imageholder]
		frame $imageholder -relief flat -borderwidth 0
		phxW3ModeConfigure $imageholder
		pack configure $imageholder \
			-in [phxW3CurrentFrame] -side top -fill x
	} else {
		set imageholder [winfo parent $before].[phxW3WidgetNewName imageholder]
		frame $imageholder -relief flat -borderwidth 0
		phxW3ModeConfigure $imageholder
		pack configure $imageholder -before $before \
			-side top -fill x
	}

	return $imageholder
}

# phxW3WidgetSplitImageHolder
# 
# If the argument is an image within an imageholder, but not the
# first within the imageholder, then split the imageholder
# between the image and the preceeding image.  The return value
# is the first image (within the new imageholder) which contains
# the right half of the original imageholder.  The name of the
# left half must remain unchanged.  There is no need to update
# the Focus or Selection.

proc phxW3WidgetSplitImageHolder {image} {

    # Only do something if $image and [phxW3WidgetLeft $image] are in 
    # the same imageholder.
    if {$image == "end"} { return $image }
    set imageLeft [phxW3WidgetLeft $image]
    if {$imageLeft == "start"} { return $image }

    set classLeft [phxW3WidgetClass $imageLeft]
    set classRight [phxW3WidgetClass $image]

    if {$classLeft != "image" || $classRight != "image" || \
	[winfo parent $image] != [winfo parent $imageLeft]} \
    {
	return $image
    }
    
    # We've got work to do.

    global tkW3HtImages tkW3HtAnchors

    set imageholderLeft [winfo parent $image]

    # Create a new imageholder
    set imageholderRight [phxW3WidgetAppendNewFrame]
    # Move the new imageholder from the end of the file to the
    # correct place.
    pack $imageholderRight -after $imageholderLeft
    phxW3ModeConfigure $imageholderRight
    
    # Copy the images one by one.
    for {set w $image} \
	{$w != "end" && [winfo parent $w] == $imageholderLeft} \
	{} \
    {
	set imageNew $imageholderRight.[phxW3WidgetNewName image]	
	# Make the new window have the same image index.
	set i $tkW3HtImages($w)
	unset tkW3HtImages($w)
	set tkW3HtImages($imageNew) $i
	

	# Create the image widget
	phxW3WidgetCreateImage $imageNew $tkW3HtImages($i.source) \
	    [expr {$tkW3HtImages($i.anchor) != ""}] \
	    $tkW3HtImages($i.anchortag) $tkW3HtImages($i.ismap)
	phxW3ModeConfigure $imageNew

	# Prepare for the next iteration
	set wNext [phxW3WidgetRight $w]
	destroy $w
	set w $wNext
    }

    return [phxW3WidgetRight $imageLeft]
}

# phxW3WidgetWrap
# Re-wraps images or merges adjacent text widgets.
# If the Focus or Selection is renamed, calls must be made to update
# that information.

proc phxW3WidgetWrap {w} {
    # Nothing to do unless $w is a real widget.
    if {$w == "start" || $w == "end"} { return }

    switch [phxW3WidgetClass $w] {
	para { phxW3WidgetMergePara $w }
	image { phxW3WidgetImageWrap $w }
    }
}

# phxW3WidgetImageWrap
# Re-wrap all images contiguous with $image.

proc phxW3WidgetImageWrap {image} {

    # Find the first image
    set class [phxW3WidgetClass $image]

    while {$class == "image"} {
	set imageFirst $image
	set image [phxW3WidgetLeft $image]
	set class "start"
	if {$image != "start"} { set class [phxW3WidgetClass $image] }
    }

    set imageholderFirst [winfo parent $imageFirst]

    # Create the first new imageholder

    set imageholderNew [phxW3WidgetAppendNewFrame]
    # Move the new imageholder from the end of the file to the
    # correct place.
    pack $imageholderNew -before $imageholderFirst
    phxW3ModeConfigure $imageholderNew

    # Move the images

    global tkW3HtImages tkW3HtAnchors

    set imageOld $imageFirst
    set class [phxW3WidgetClass $imageFirst]
    while {$class == "image"} {
	set instance [phxW3WidgetNewName image]
	set imageNew $imageholderNew.$instance

	# Make the new window have the same image index.
	set i $tkW3HtImages($imageOld)
	unset tkW3HtImages($imageOld)
	set tkW3HtImages($imageNew) $i

	# Create the new image
	phxW3WidgetCreateImage $imageNew $tkW3HtImages($i.source) \
	    [expr {$tkW3HtImages($i.anchor) != ""}] \
	    $tkW3HtImages($i.anchortag) $tkW3HtImages($i.ismap)
    
	update idletasks
	if {[winfo reqwidth $imageNew] != [winfo width $imageNew] &&
		[llength [winfo children $imageholderNew]] > 1} \
	{
	    # The image didn't fit.  Destroy it.
	    unset tkW3HtImages($imageNew)
	    destroy $imageNew

	    # Create another imageholder
	    set imageholderNext [phxW3WidgetAppendNewFrame]
	    pack $imageholderNext -after $imageholderNew
	    set imageholderNew $imageholderNext
	    phxW3ModeConfigure $imageholderNew

	    set imageNew $imageholderNew.$instance
	    set tkW3HtImages($imageNew) $i
	    phxW3WidgetCreateImage $imageNew $tkW3HtImages($i.source) \
		[expr {$tkW3HtImages($i.anchor) != ""}] \
		$tkW3HtImages($i.anchortag) $tkW3HtImages($i.ismap)
	}

	phxW3ModeConfigure $imageNew

	# Let Selection know about the new image names
	set leftSelect [phxW3SelectionGetLeftOfFocus]
	if {$leftSelect == $imageOld} {set leftSelect $imageNew}
	set rightSelect [phxW3SelectionGetRightOfFocus]
	if {$rightSelect == $imageOld} {set rightSelect $imageNew}
	phxW3SelectionSetFocus $leftSelect $rightSelect

	if {$imageOld == [phxW3SelectionGet]} {phxW3SelectionSet $imageNew}

	# Destroy the old image
	set imageNext [phxW3WidgetRight $imageOld]
	set imageholderOld [winfo parent $imageOld]
	destroy $imageOld
	if {[winfo children $imageholderOld] == ""} {
	    destroy $imageholderOld
	}

	# Prepare for the next iteration
	set imageOld $imageNext
	set class "end"
	if {$imageOld != "end"} { set class [phxW3WidgetClass $imageOld] }
    }
}


proc Im_get_stats {filename} {

	set info [exec xli -ident $filename]
	set imtype ""
	set size ""
	set colors ""

	foreach i $info {
		if {$i == "GIF"} {
			set imtype GIF
		}
		if {$i == "JPEG"} {
			set imtype JPEG
		}
		if {$i == "Sun rasterfile"} {
			set imtype RS
		}

		if {$i == "found" && [info exists last] && $last == "not"} {
		Im_put_text .editwindow.misc.message.text "Picture not found"
		return 
		}
		
		if {[string match {*[0-9]x[0-9]*} $i] > 0} {
			set size $i
		}
		
		if {$i == "colors" && [info exists last]} {
			set colors $last
		}

		set last $i
	}

	return [list $imtype $size $colors]
}




proc tkW3ImageReleaseColorCells {w} {

    set window [winfo id $w]
    if {"$window" != "0x0"} {
       global tkW3ConfigViewer    
       set output \
	   [open  "| $tkW3ConfigViewer(image) -windowid $window stdin > /dev/null" "w"]
       puts $output "P1 1 1 0"
       close $output
   }
}

################################################################
#       Widgets used within the <FORM>...</FORM> markup.
################################################################

# The phxW3WidgetInsertForms* routines create a widget useful in
# the <FORM>...</FORM> markup.  The widget is inserted at the
# end of the document by default.  If the "before" argument is
# not "" then the created widget is inserted before the "before"
# widget.  All other arguments must be supplied.  Any argument
# which needs to be given a default value should have been given
# it prior to the calling of these functions.

# phxW3WidgetInsertFormsPassword
#
# Insert a password entry field into a form.  Returns the name of
# the created widget.

proc phxW3WidgetInsertFormsPassword {maxlength name size value {before ""}} {

	global phxW3CurrentWidget
	set frame [phxW3CurrentFrame]

	set instance [phxW3WidgetNewName password]

	# $size is either "cols" or "cols,rows"
	set size [split $size ","]
	set cols [lindex $size 0]
	set rows [expr {[llength $size]==1 ? 1 : [lindex $size 1]}]

	text [set widget $frame.$instance] -width $cols -height $rows \
	    -relief sunken
	if {$maxlength != "unlimited"} {
		# Promote any existing binding to the widget level.
		phxW3SelectionSetBinding $widget <Any-KeyPress> ""
		# Append new binding
		bind $widget <Any-KeyPress> \
			"+phxW3WidgetEnforceTextMaxlength $widget $maxlength"
	}
	phxW3SelectionWidgetInitialize $widget
	phxW3ModeConfigure $widget
	if {$before == ""} {
		pack configure $widget -anchor w -side top -in $frame
	} else {
		pack configure $widget -anchor w -side top -before $before
	}
	set phxW3CurrentWidget $widget

	# Make sure the text is invisible even when selected.
	# (phxW3ModeConfigure takes care of the unselected case.)
	$widget configure -selectforeground \
	    [lindex [$widget configure -selectbackground] 4]
	$widget insert 1.0 $value
	return $widget
}

# phxW3WidgetInsertFormsText
#
# Insert a text entry field into a form.  Returns the name of
# the created widget.

proc phxW3WidgetInsertFormsText {maxlength name size value {before ""}} {

	global phxW3CurrentWidget
	set frame [phxW3CurrentFrame]

	set instance [phxW3WidgetNewName text]

	# $size is either "cols" or "cols,rows"
	set size [split $size ","]
	set cols [lindex $size 0]
	set rows [expr {[llength $size]==1 ? 1 : [lindex $size 1]}]

	text [set widget $frame.$instance] -width $cols -height $rows \
	    -relief sunken
	if {$maxlength != "unlimited"} {
		# Promote any existing binding to the widget level.
		phxW3SelectionSetBinding $widget <Any-KeyPress> ""
		# Append new binding
		bind $widget <Any-KeyPress> \
			"+phxW3WidgetEnforceTextMaxlength $widget $maxlength"
	}
	phxW3SelectionWidgetInitialize $widget
	phxW3ModeConfigure $widget
	if {$before == ""} {
		pack configure $widget -anchor w -side top -in $frame
	} else {
		pack configure $widget -anchor w -side top -before $before
	}
	set phxW3CurrentWidget $widget
	$widget insert 1.0 $value
	return $widget
}

proc phxW3WidgetEnforceTextMaxlength {widget maxlength} {
    if {[string length [$widget get 1.0 end] > $maxlength} {tk_textBackspace $widget}
}

# phxW3WidgetInsertFormsTextarea
#
# Insert a textarea field into a form.  Returns the name of
# the created widget.

proc phxW3WidgetInsertFormsTextarea {name rows cols {before ""}} {
	global phxW3CurrentWidget
	set frame [phxW3CurrentFrame]

	set widget $frame.[phxW3WidgetNewName textareaholder]
	set vscroll $widget.[phxW3WidgetNewName vscroll]	
	set textarea $widget.[phxW3WidgetNewName textarea]
	frame $widget -relief flat
	scrollbar $vscroll -command "$textarea yview" -bd 1 -relief sunken
	text $textarea -width $cols -height $rows -relief sunken \
	    -yscrollcommand "$vscroll set"
	pack configure $vscroll -in $widget -side left -fill y
	pack configure $textarea -in $widget -side top -fill both -expand yes

	phxW3SelectionWidgetInitialize $textarea
	phxW3ModeConfigure $widget
	phxW3ModeConfigure $vscroll
	phxW3ModeConfigure $textarea
	if {$before == ""} {
		pack configure $widget -anchor w -side top -in $frame
	} else {
		pack configure $widget -anchor w -side top -before $before
	}
	set phxW3CurrentWidget $widget
	return $widget
}

# phxW3WidgetInsertFormsSelect
#
# Insert a select field into a form.  Returns the name of
# the created widget.

proc phxW3WidgetInsertFormsSelect {name multiple {before ""}} {
	global phxW3CurrentWidget
	set frame [phxW3CurrentFrame]

	set instance [phxW3WidgetNewName select]
	menubutton [set widget $frame.$instance] -height 1 -relief sunken
	$widget configure -menu [menu $widget.menu]
	phxW3SelectionWidgetInitialize $widget
	phxW3ModeConfigure $widget
	if {$before == ""} {
		pack configure $widget -anchor w -side top -in $frame
	} else {
		pack configure $widget -anchor w -side top -before $before
	}
	set phxW3CurrentWidget $widget
	return $widget
}

# phxW3WidgetInsertFormsCheckbox
#
# Insert a checkbox into a form.  Returns the name of the created widget.

proc phxW3WidgetInsertFormsCheckbox {checked name size value {before ""}} {
	global phxW3CurrentWidget
	set frame [phxW3CurrentFrame]

	set instance [phxW3WidgetNewName checkbox]
	checkbutton [set widget $frame.$instance]
	if {$checked} {$widget select} else {$widget deselect}
	phxW3SelectionWidgetInitialize $widget
	phxW3ModeConfigure $widget
	if {$before == ""} {
		pack configure $widget -anchor w -side top -in $frame
	} else {
		pack configure $widget -anchor w -side top -before $before
	}
	set phxW3CurrentWidget $widget
	return $widget
}

# phxW3WidgetInsertFormsImage
#
# Insert an image into a form.  Returns the name of the created widget.

proc phxW3WidgetInsertFormsImage {align name size src {before ""}} {
	global phxW3CurrentWidget
	set frame [phxW3CurrentFrame]

	set instance [phxW3WidgetNewName formsimage]
	button [set widget $frame.$instance] -borderwidth 1 -padx 0 -pady 0 \
	    -relief raised
	if {[catch {$widget configure -bitmap @$src}]} {
	    $widget configure -text "Image unloadable"
	}
	phxW3SelectionWidgetInitialize $widget
	phxW3ModeConfigure $widget
	if {$before == ""} {
		pack configure $widget -anchor w -side top -in $frame
	} else {
		pack configure $widget -anchor w -side top -before $before
	}
	set phxW3CurrentWidget $widget
	return $widget
}

# phxW3WidgetInsertFormsRadio
#
# Insert a radio button into a form.  Returns the name of the created widget.

proc phxW3WidgetInsertFormsRadio {checked name size value {before ""}} {
	global phxW3CurrentWidget
	set frame [phxW3CurrentFrame]

	set instance [phxW3WidgetNewName radio]
	radiobutton [set widget $frame.$instance]
	phxW3SelectionWidgetInitialize $widget
	phxW3ModeConfigure $widget
	if {$before == ""} {
		pack configure $widget -anchor w -side top -in $frame
	} else {
		pack configure $widget -anchor w -side top -before $before
	}
	set phxW3CurrentWidget $widget
	return $widget
}

# phxW3WidgetInsertFormsReset
#
# Insert a reset button into a form.  Returns the name of the created widget.

proc phxW3WidgetInsertFormsReset {size value {before ""}} {
	global phxW3CurrentWidget
	set frame [phxW3CurrentFrame]

	set instance [phxW3WidgetNewName reset]
	button [set widget $frame.$instance] -text $value
	phxW3SelectionWidgetInitialize $widget
	phxW3ModeConfigure $widget
	if {$before == ""} {
		pack configure $widget -anchor w -side top -in $frame
	} else {
		pack configure $widget -anchor w -side top -before $before
	}
	set phxW3CurrentWidget $widget
	return $widget
}

# phxW3WidgetInsertFormsSubmit
#
# Insert a submit button into a form.  Returns the name of the created widget.

proc phxW3WidgetInsertFormsSubmit {name size value {before ""}} {
	global phxW3CurrentWidget
	set frame [phxW3CurrentFrame]

	set instance [phxW3WidgetNewName submit]
	button [set widget $frame.$instance] -text $value
	phxW3SelectionWidgetInitialize $widget
	phxW3ModeConfigure $widget
	if {$before == ""} {
		pack configure $widget -anchor w -side top -in $frame
	} else {
		pack configure $widget -anchor w -side top -before $before
	}
	set phxW3CurrentWidget $widget
	return $widget
}

################################################################
# Navigating around widgets
################################################################

# procW3WidgetLeft
# 
# Finds the widget immediately before the supplied widget.  Only
# widgets which are representations of HTML (e.g., para, rule,
# image, password, text, textarea, select, checkbox, formsimage,
# radio, reset, submit, but not the main frame, or imageholders)
# will be returned by this procedure.  If this is the first
# widget then "start" will be returned.

proc phxW3WidgetLeft {w} {

    switch -glob $w {
	.f.* -
	start -
	end {}
	default { return $w }
    }

    if {$w == "start" || [phxW3WidgetClass $w] == "frame"} { return "start" }
    if {$w == "end"} {
	return [phxW3WidgetLeftwardTraversal [phxW3CurrentHtext].frame]
    }

    # Step through list of siblings to the "left" of $w and find next
    # appropriate widget to move to; call phxW3WidgetLeftwardTraversal
    # to find widget if in a list frame etc.  If no suitable widget
    # found, then find most appropriate widget to left of $w's *parent*.
    set parent [winfo parent $w]
    set siblings [pack slaves $parent]
    set result ""
    for {set index [lsearch $siblings $w]} \
	{[incr index -1] >= 0 && $result == ""} \
	{} \
    {
	set result [phxW3WidgetLeftwardTraversal [lindex $siblings $index]]
    }
    if {$result == ""} { set result [phxW3WidgetLeft $parent] }
    return $result
}

# phxW3WidgetRight
# 
# Finds the widget immediately after the supplied widget.  If
# this is the last widget then "end" will be returned.  Only
# certain widgets are returned; see phxW3WidgetLeft.

proc phxW3WidgetRight {w} {

    switch -glob $w {
	.f.* -
	start -
	end {}
	default { return $w }
    }

    if {$w == "end" || [phxW3WidgetClass $w] == "frame"} { return "end" }
    if {$w == "start"} {
	return [phxW3WidgetRightwardTraversal [phxW3CurrentHtext].frame]
    }

    # Step through list of siblings to the "right" of $w and find next
    # appropriate widget to move to; call phxW3WidgetRightwardTraversal
    # to find widget if in a list frame etc.  If no suitable widget
    # found, then find most appropriate widget to right of $w's *parent*.
    set parent [winfo parent $w]
    set siblings [pack slaves $parent]
    set length [llength $siblings]
    set result ""
    for {set index [lsearch $siblings $w]} \
	{[incr index] < $length && $result == ""} \
	{} \
    {
	set result [phxW3WidgetRightwardTraversal [lindex $siblings $index]]
    }
    if {$result == ""} { set result [phxW3WidgetRight $parent] }
    return $result
}

# phxW3WidgetLeftwardTraversal
#
# Perform depth-first search of the frame tree to which arg $w points,
# approaching $w as if from the left.  Return next deletable or
# selectable widget (i.e. appropriate for editing, as opposed to
# "invisible" widgets like a list_UL frame.

proc phxW3WidgetLeftwardTraversal {w} {
    set result ""
    if {[phxW3WidgetIsHTML $w]} {set result $w} \
    elseif {[winfo class $w] == "Frame"} {
	set siblings [pack slaves $w]
	for {set index [llength $siblings]} \
	    {[incr index -1] >= 0 && $result == ""} \
	    {} \
	{
	    set result [phxW3WidgetLeftwardTraversal [lindex $siblings $index]]
	}
    }
    return $result
}

# phxW3WidgetRightwardTraversal
#
# Perform depth-first search of the frame tree to which arg $w points,
# approaching $w as if from the right.

proc phxW3WidgetRightwardTraversal {w} {
    set result ""
    if {[phxW3WidgetIsHTML $w]} {set result $w} \
    elseif {[winfo class $w] == "Frame"} {
	set siblings [pack slaves $w]
	set length [llength $siblings]
	for {set index -1} \
	    {[incr index] < $length && $result == ""} \
	    {} \
	{
	    set result [phxW3WidgetRightwardTraversal [lindex $siblings $index]]
	}
    }
    return $result
}

# phxW3WidgetIsHTML
#
# Returns 1 or 0 depending on whether widget $w is one that
# represents a chunk of HTML.  Used by phxW3WidgetLeft and
# phxW3WidgetRight.

proc phxW3WidgetIsHTML {w} {
	switch [phxW3WidgetClass $w] {
		para -
		bullet -
		rule -
		image -
		password -
		text -
		textarea -
		select -
		checkbox -
		formsimage -
		radio -
		reset -
		submit {return 1}
	}
	return 0
}

# phxW3WidgetPackBefore
# 
# Packs the new widget before the supplied right widget.  No
# re-wrapping of images occurs.  Only certain widgets should be
# supplied to this routine; see phxW3WidgetLeft.  If the new
# widget is an image, it must be the last image in its
# imageholder.  If the right widget is an image, it must be the
# first in its imageholder.

proc phxW3WidgetPackBefore {wNew wRight} {
    set class [phxW3WidgetClass $wNew]

    # From images, we really want to pack the imageholder.
    if {$class == "image"} {
	set wNew [winfo parent $wNew]
    }

    # Figure out where to pack the widget
    if {$wRight == "end"} {
	# The widget goes last
	set where "-in [winfo parent $wNew]"
    } else {
	# From images, we really want to pack the imageholder.
	set class [phxW3WidgetClass $wRight]
	if {$class == "image"} {
	    set wRight [winfo parent $wRight]
	}
	set where "-before $wRight"
    }

    pack $wNew [lindex $where 0] [lindex $where 1] -side top -fill x
}

proc phxW3WidgetSetCurrentWidget {w} {
    global phxW3CurrentWidget
    set phxW3CurrentWidget $w
}
