/***************************************************************
* This implementation of RFC1421 is Copyright 1994 by
* The University of Chicago Biological Sciences Division, Academic Computing
* Lee Newberg
***************************************************************/

#include <stdio.h>
#include <tcl.h>
#include "HTAuth.h"

static unsigned char encode[64] = {'A', 'B', 'C', 'D', 'E', 'F', 'G',
'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T',
'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g',
'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't',
'u', 'v', 'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6',
'7', '8', '9', '+', '/'};

#define NOP 255

static unsigned char decode[256] = {
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP};

static void outputcrlf ARGS2(unsigned char *, output, int *,index)
{
  output[(*index)++] = 13;
  output[(*index)++] = 10;
}

/*
Test program.

int main ARGS2(int, argc, char *[], argv)
{
  int a, b;
  unsigned char result[20], result2[20];
  a = rfc1421encode(12, (unsigned char *) "hello world", result, 1);
  printf ("length = %d string = {%s}\n", a, result);
  b = rfc1421decode(a, result, result2);
  printf ("length = %d string = {%s}\n", b, result2);
  return 0;
}
*/

/* rfc1421encode: Converts a buffer "input" of length "length"
   containing arbitrary, unsigned bytes into printable, 6-bit
   values in the buffer "output."  A carriage-return/line-feed
   pair is inserted after every 64 printed characters if "crlf"
   is true.  The length of "output" is returned. */

int rfc1421encode ARGS4(int, length,	/* length of input */
		  unsigned char *, input, /* input buffer */
		  unsigned char *, output, /* output buffer */
		  int, crlf	/* insert ^M^J ? */
		  )
{
  unsigned char a, b, c;
  extern unsigned char encode[];
  int iOutput = 0;
  int iInput = 0;
  for (iInput = 0; iInput <= length-3; iInput+=3)
  {
    if (crlf && iInput != 0 && iInput % 48 == 0) outputcrlf(output, &iOutput);

    /* Output 4 printable bytes */
    a = input[iInput];
    b = input[iInput + 1];
    c = input[iInput + 2];
    output[iOutput++] = encode[                  (a >> 2)];
    output[iOutput++] = encode[((a & 3) << 4)  + (b >> 4)];
    output[iOutput++] = encode[((b & 15) << 2) + (c >> 6)];
    output[iOutput++] = encode[((c & 63))];
  }

  /* If the number of bytes is not a multiple of three,
     do the remaining bytes. */

  switch (length - iInput)
  {
  case 0:
    break;
  case 1:
    if (crlf && iInput != 0 && iInput % 48 == 0) outputcrlf(output, &iOutput);

    a = input[iInput];
    output[iOutput++] = encode[                  (a >> 2)];
    output[iOutput++] = encode[((a & 3) << 4)            ];
    output[iOutput++] = '=';
    output[iOutput++] = '=';
    break;
  case 2:
    if (crlf && iInput != 0 && iInput % 48 == 0) outputcrlf(output, &iOutput);

    a = input[iInput];
    b = input[iInput + 1];
    output[iOutput++] = encode[                  (a >> 2)];
    output[iOutput++] = encode[((a & 3) << 4)  + (b >> 4)];
    output[iOutput++] = encode[((b & 15) << 2)           ];
    output[iOutput++] = '=';
    break;
  }

  if (crlf) outputcrlf(output, &iOutput);
  return iOutput;
}

/* rfc1421decode: Converts a buffer "input" of length "length"
   containing printable, 6-bit values into arbitrary, unsigned
   bytes in the buffer "output."  The length of "output" is
   returned. */

int rfc1421decode ARGS3(int, length, unsigned char *, input, unsigned char *, output)
{
  extern unsigned char decode[];
  unsigned char a, b, c, d;
  int iInput, iOutput;

  if (decode['A'] == NOP)
  {
    decode['A'] = 0;
    decode['B'] = 1;
    decode['C'] = 2;
    decode['D'] = 3;
    decode['E'] = 4;
    decode['F'] = 5;
    decode['G'] = 6;
    decode['H'] = 7;
    decode['I'] = 8;
    decode['J'] = 9;
    decode['K'] = 10;
    decode['L'] = 11;
    decode['M'] = 12;
    decode['N'] = 13;
    decode['O'] = 14;
    decode['P'] = 15;
    decode['Q'] = 16;
    decode['R'] = 17;
    decode['S'] = 18;
    decode['T'] = 19;
    decode['U'] = 20;
    decode['V'] = 21;
    decode['W'] = 22;
    decode['X'] = 23;
    decode['Y'] = 24;
    decode['Z'] = 25;
    decode['a'] = 26;
    decode['b'] = 27;
    decode['c'] = 28;
    decode['d'] = 29;
    decode['e'] = 30;
    decode['f'] = 31;
    decode['g'] = 32;
    decode['h'] = 33;
    decode['i'] = 34;
    decode['j'] = 35;
    decode['k'] = 36;
    decode['l'] = 37;
    decode['m'] = 38;
    decode['n'] = 39;
    decode['o'] = 40;
    decode['p'] = 41;
    decode['q'] = 42;
    decode['r'] = 43;
    decode['s'] = 44;
    decode['t'] = 45;
    decode['u'] = 46;
    decode['v'] = 47;
    decode['w'] = 48;
    decode['x'] = 49;
    decode['y'] = 50;
    decode['z'] = 51;
    decode['0'] = 52;
    decode['1'] = 53;
    decode['2'] = 54;
    decode['3'] = 55;
    decode['4'] = 56;
    decode['5'] = 57;
    decode['6'] = 58;
    decode['7'] = 59;
    decode['8'] = 60;
    decode['9'] = 61;
    decode['+'] = 62;
    decode['/'] = 63;
    /* '=' is a valid character even though it doesn't represent
       bits.  We'll assign it value 64 arbitrarily. */
    decode['='] = 64;
  }

  iInput = iOutput = 0;
  while (1)
  {
    while (iInput < length && (a = decode[input[iInput]]) == NOP) iInput++;
    if (iInput++ == length) break;

    while ((b = decode[input[iInput++]]) == NOP);
    while ((c = decode[input[iInput++]]) == NOP);
    while ((d = decode[input[iInput++]]) == NOP);
    if (d != 64) 
    {
      /* No padding present */
      output[iOutput++] = (a << 2) + (b >> 4);
      output[iOutput++] = ((b & 15) << 4) + (c >> 2);
      output[iOutput++] = ((c & 3) << 6) + d;
    }
    else if (c != 64)
    {
      /* One '=' present. */
      output[iOutput++] = (a << 2) + (b >> 4);
      output[iOutput++] = ((b & 15) << 4) + (c >> 2);
    }
    else
    {
      /* Two '=' present. */
      output[iOutput++] = (a << 2) + (b >> 4);
    }
  }

  return iOutput;
}

static int HtauthorizationInit = 0;	/* Whether memory has been
				   allocated for user and passwd */
static Tcl_DString HTuser;	/* Supplied user name for authentication */
static Tcl_DString HTpasswd;	/* Supplied password for authentication */

void get_user_and_passwd ARGS2(char **, user, char **, passwd)
{
  extern int HtauthorizationInit;
  extern Tcl_DString HTuser;
  extern Tcl_DString HTpasswd;

  if (HtauthorizationInit == 0)
  {
    /* Must initialize the dynamic strings to "" */
    Tcl_DStringInit(&HTuser);
    Tcl_DStringInit(&HTpasswd);
    HtauthorizationInit = 1;
  }

  *user = HTuser.string;
  *passwd = HTpasswd.string;
  return;
}

void set_user_and_passwd ARGS2(char *, user, char *, passwd)
{
  extern int HtauthorizationInit;
  extern Tcl_DString HTuser;
  extern Tcl_DString HTpasswd;

  if (HtauthorizationInit == 0)
  {
    /* Must allocate memory for the strings */
    Tcl_DStringInit(&HTuser);
    Tcl_DStringInit(&HTpasswd);
    HtauthorizationInit = 1;
  }

  Tcl_DStringFree(&HTuser);	/* Remove any old value */
  Tcl_DStringFree(&HTpasswd);	/* Remove any old value */

  Tcl_DStringAppend(&HTuser, user, -1);	/* Store new value */
  Tcl_DStringAppend(&HTpasswd, passwd, -1); /* Store new value */
}
